@@ -1,12 +1,14 @@
 /* Generic assert.h */
+/* $OpenLDAP: pkg/ldap/include/ac/assert.h,v 1.16 2003/01/03 19:20:50 kurt Exp $ */
 /*
- * Copyright 1999 The OpenLDAP Foundation, Redwood City, California, USA
+ * Copyright 1998-2003 The OpenLDAP Foundation, Redwood City, California, USA
  * All rights reserved.
  *
- * Redistribution and use in source and binary forms are permitted only
- * as authorized by the OpenLDAP Public License.  A copy of this
- * license is available at http://www.OpenLDAP.org/license.html or
- * in file LICENSE in the top-level directory of the distribution.
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.  A copy of this license is available at
+ * http://www.OpenLDAP.org/license.html or in file LICENSE in the
+ * top-level directory of the distribution.
  */
 
 #ifndef _AC_ASSERT_H
@@ -30,8 +32,8 @@
  * create a replacement and hope it works
  */
 
-LDAP_F(void) ber_pvt_assert LDAP_P((
-	char* file, int line, char* test));
+LBER_F (void) ber_pvt_assert LDAP_P(( const char *file, int line,
+					 const char *test ));
 
 /* Can't use LDAP_STRING(test), that'd expand to "test" */
 #if defined(__STDC__) || defined(__cplusplus)
