@@ -1,6 +1,7 @@
 /* decode.c - ber input decoding routines */
+/* $OpenLDAP: pkg/ldap/libraries/liblber/decode.c,v 1.96 2003/05/02 11:29:28 hallvard Exp $ */
 /*
- * Copyright 1998-1999 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /* Portions
@@ -25,12 +26,9 @@
 #include <ac/string.h>
 #include <ac/socket.h>
 
-#undef LDAP_F_PRE
-#define LDAP_F_PRE LDAP_F_EXPORT
-
 #include "lber-int.h"
 
-static ber_tag_t ber_getnint LDAP_P((
+static ber_len_t ber_getnint LDAP_P((
 	BerElement *ber,
 	ber_int_t *num,
 	ber_len_t len ));
@@ -41,36 +39,44 @@ ber_get_tag( BerElement *ber )
 {
 	unsigned char	xbyte;
 	ber_tag_t	tag;
-	char		*tagp;
 	unsigned int	i;
 
 	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
-	if ( ber_read( ber, (char *) &xbyte, 1 ) != 1 )
-		return( LBER_DEFAULT );
+	if ( ber_pvt_ber_remaining( ber ) < 1 ) {
+		return LBER_DEFAULT;
+	}
 
-	if ( (xbyte & LBER_BIG_TAG_MASK) != LBER_BIG_TAG_MASK )
-		return( (ber_tag_t) xbyte );
+	if ( ber->ber_ptr == ber->ber_buf )
+		tag = *(unsigned char *)ber->ber_ptr;
+	else
+		tag = ber->ber_tag;
+	ber->ber_ptr++;
+
+	if ( (tag & LBER_BIG_TAG_MASK) != LBER_BIG_TAG_MASK ) {
+		return tag;
+	}
 
-	tagp = (char *) &tag;
-	tagp[0] = xbyte;
 	for ( i = 1; i < sizeof(ber_tag_t); i++ ) {
-		if ( ber_read( ber, (char *) &xbyte, 1 ) != 1 )
-			return( LBER_DEFAULT );
+		if ( ber_read( ber, (char *) &xbyte, 1 ) != 1 ) {
+			return LBER_DEFAULT;
+		}
 
-		tagp[i] = xbyte;
+		tag <<= 8;
+		tag |= 0x00ffUL & (ber_tag_t) xbyte;
 
-		if ( ! (xbyte & LBER_MORE_TAG_MASK) )
+		if ( ! (xbyte & LBER_MORE_TAG_MASK) ) {
 			break;
+		}
 	}
 
 	/* tag too big! */
-	if ( i == sizeof(ber_tag_t) )
-		return( LBER_DEFAULT );
+	if ( i == sizeof(ber_tag_t) ) {
+		return LBER_DEFAULT;
+	}
 
-	/* want leading, not trailing 0's */
-	return( tag >> (sizeof(ber_tag_t) - i - 1) );
+	return tag;
 }
 
 ber_tag_t
@@ -78,13 +84,12 @@ ber_skip_tag( BerElement *ber, ber_len_t *len )
 {
 	ber_tag_t	tag;
 	unsigned char	lc;
-	ber_len_t	noctets;
-	int		diff;
-	ber_len_t	netlen;
+	ber_len_t	i, noctets;
+	unsigned char netlen[sizeof(ber_len_t)];
 
 	assert( ber != NULL );
 	assert( len != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	/*
 	 * Any ber element looks like this: tag length contents.
@@ -96,65 +101,78 @@ ber_skip_tag( BerElement *ber, ber_len_t *len )
 	 *	2) primitive encodings used whenever possible
 	 */
 
+	*len = 0;
+
 	/*
 	 * First, we read the tag.
 	 */
 
-	if ( (tag = ber_get_tag( ber )) == LBER_DEFAULT )
-		return( LBER_DEFAULT );
+	if ( (tag = ber_get_tag( ber )) == LBER_DEFAULT ) {
+		return LBER_DEFAULT;
+	}
 
 	/*
 	 * Next, read the length.  The first byte contains the length of
-	 * the length.  If bit 8 is set, the length is the long form,
+	 * the length.	If bit 8 is set, the length is the long form,
 	 * otherwise it's the short form.  We don't allow a length that's
-	 * greater than what we can hold in an unsigned long.
+	 * greater than what we can hold in a ber_len_t.
 	 */
 
-	*len = netlen = 0;
 	if ( ber_read( ber, (char *) &lc, 1 ) != 1 )
-		return( LBER_DEFAULT );
+		return LBER_DEFAULT;
+
 	if ( lc & 0x80U ) {
 		noctets = (lc & 0x7fU);
-		if ( noctets > sizeof(ber_len_t) )
-			return( LBER_DEFAULT );
-		diff = sizeof(ber_len_t) - noctets;
-		if ( (unsigned) ber_read( ber, (char *) &netlen + diff, noctets )
-		    != noctets )
-			return( LBER_DEFAULT );
-		*len = LBER_LEN_NTOH( netlen );
+
+		if ( noctets > sizeof(ber_len_t) ) {
+			return LBER_DEFAULT;
+		}
+
+		if( (unsigned) ber_read( ber, (char *) netlen, noctets ) != noctets ) {
+			return LBER_DEFAULT;
+		}
+
+		for( i = 0; i < noctets; i++ ) {
+			*len <<= 8;
+			*len |= netlen[i];
+		}
+
 	} else {
 		*len = lc;
 	}
 
-	return( tag );
+	/* BER element should have enough data left */
+	if( *len > (ber_len_t) ber_pvt_ber_remaining( ber ) ) {
+		return LBER_DEFAULT;
+	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
+
+	return tag;
 }
 
 ber_tag_t
 ber_peek_tag(
-	LDAP_CONST BerElement *ber_in,
+	BerElement *ber,
 	ber_len_t *len )
 {
-	ber_tag_t	tag;
-	BerElement *ber;
+	/*
+	 * This implementation assumes ber_skip_tag() only
+	 * modifies ber_ptr field of the BerElement.
+	 */
 
-	assert( ber_in != NULL );
-	assert( BER_VALID( ber_in ) );
-
-	ber = ber_dup( ber_in );
-
-	if( ber == NULL ) {
-		return LBER_ERROR;
-	}
-
-	assert( BER_VALID( ber ) );
+	char *save;
+	ber_tag_t	tag, old;
 
+	old = ber->ber_tag;
+	save = ber->ber_ptr;
 	tag = ber_skip_tag( ber, len );
+	ber->ber_ptr = save;
+	ber->ber_tag = old;
 
-	ber_free( ber, 0 );
-	return( tag );
+	return tag;
 }
 
-static ber_tag_t
+static ber_len_t
 ber_getnint(
 	BerElement *ber,
 	ber_int_t *num,
@@ -164,8 +182,7 @@ ber_getnint(
 
 	assert( ber != NULL );
 	assert( num != NULL );
-
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	/*
 	 * The tag and length have already been stripped off.  We should
@@ -174,12 +191,13 @@ ber_getnint(
 	 * extend after we read it in.
 	 */
 
-	if ( len > sizeof(ber_int_t) )
-		return( -1 );
+	if ( len > sizeof(ber_int_t) ) {
+		return -1;
+	}
 
 	/* read into the low-order bytes of our buffer */
-	if ( (ber_len_t) ber_read( ber, buf, len ) != len ) {
-		return( -1 );
+	if ( (ber_len_t) ber_read( ber, (char *) buf, len ) != len ) {
+		return -1;
 	}
 
 	if( len ) {
@@ -197,8 +215,9 @@ ber_getnint(
 	} else {
 		*num = 0;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
-	return( len );
+	return len;
 }
 
 ber_tag_t
@@ -210,15 +229,25 @@ ber_get_int(
 	ber_len_t	len;
 
 	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
-	if ( (tag = ber_skip_tag( ber, &len )) == LBER_DEFAULT )
-		return( LBER_DEFAULT );
+	if ( (tag = ber_skip_tag( ber, &len )) == LBER_DEFAULT ) {
+		return LBER_DEFAULT;
+	}
 
-	if ( (unsigned long) ber_getnint( ber, num, len ) != len )
-		return( LBER_DEFAULT );
-	else
-		return( tag );
+	if ( ber_getnint( ber, num, len ) != len ) {
+		return LBER_DEFAULT;
+	}
+	
+	return tag;
+}
+
+ber_tag_t
+ber_get_enum(
+	BerElement *ber,
+	ber_int_t *num )
+{
+	return ber_get_int( ber, num );
 }
 
 ber_tag_t
@@ -230,136 +259,238 @@ ber_get_stringb(
 	ber_len_t	datalen;
 	ber_tag_t	tag;
 
-#ifdef STR_TRANSLATION
-	char		*transbuf;
-#endif /* STR_TRANSLATION */
-
 	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
-	if ( (tag = ber_skip_tag( ber, &datalen )) == LBER_DEFAULT )
-		return( LBER_DEFAULT );
-	if ( datalen > (*len - 1) )
-		return( LBER_DEFAULT );
+	if ( (tag = ber_skip_tag( ber, &datalen )) == LBER_DEFAULT ) {
+		return LBER_DEFAULT;
+	}
 
-	if ( (unsigned long) ber_read( ber, buf, datalen ) != datalen )
-		return( LBER_DEFAULT );
+	/* must fit within allocated space with termination */
+	if ( datalen >= *len ) {
+		return LBER_DEFAULT;
+	}
+
+	if ( (ber_len_t) ber_read( ber, buf, datalen ) != datalen ) {
+		return LBER_DEFAULT;
+	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	buf[datalen] = '\0';
 
-#ifdef STR_TRANSLATION
-	if ( datalen > 0 && ( ber->ber_options & LBER_TRANSLATE_STRINGS ) != 0
-	    && ber->ber_decode_translate_proc ) {
-		transbuf = buf;
-		++datalen;
-		if ( (*(ber->ber_decode_translate_proc))( &transbuf, &datalen,
-		    0 ) != 0 ) {
-			return( LBER_DEFAULT );
-		}
-		if ( datalen > *len ) {
-			LBER_FREE( transbuf );
-			return( LBER_DEFAULT );
-		}
-		SAFEMEMCPY( buf, transbuf, datalen );
-		LBER_FREE( transbuf );
-		--datalen;
-	}
-#endif /* STR_TRANSLATION */
-
 	*len = datalen;
-	return( tag );
+	return tag;
 }
 
-ber_tag_t
-ber_get_stringa( BerElement *ber, char **buf )
+/* Definitions for get_string vector
+ *
+ * ChArray, BvArray, and BvVec are self-explanatory.
+ * BvOff is a struct berval embedded in an array of larger structures
+ * of siz bytes at off bytes from the beginning of the struct.
+ */
+enum bgbvc { ChArray, BvArray, BvVec, BvOff };
+
+/* Use this single cookie for state, to keep actual
+ * stack use to the absolute minimum.
+ */
+typedef struct bgbvr {
+	enum bgbvc choice;
+	BerElement *ber;
+	int alloc;
+	ber_len_t siz;
+	ber_len_t off;
+	union {
+		char ***c;
+		BerVarray *ba;
+		struct berval ***bv;
+	} res;
+} bgbvr;
+
+static ber_tag_t
+ber_get_stringbvl( bgbvr *b, ber_len_t *rlen )
 {
-	ber_len_t	datalen;
-	ber_tag_t	tag;
+	int i = 0, n;
+	ber_tag_t tag;
+	ber_len_t len;
+	char *last, *orig;
+	struct berval bv, *bvp = NULL;
 
-	assert( ber != NULL );
-	assert( buf != NULL );
+	/* For rewinding, just like ber_peek_tag() */
+	orig = b->ber->ber_ptr;
+	tag = b->ber->ber_tag;
 
-	assert( BER_VALID( ber ) );
-
-	if ( (tag = ber_skip_tag( ber, &datalen )) == LBER_DEFAULT ) {
-		*buf = NULL;
-		return( LBER_DEFAULT );
-	}
-
-	if ( (*buf = (char *) LBER_MALLOC( datalen + 1 )) == NULL )
-		return( LBER_DEFAULT );
-
-	if ( (unsigned long) ber_read( ber, *buf, datalen ) != datalen ) {
-		LBER_FREE( *buf );
-		*buf = NULL;
-		return( LBER_DEFAULT );
-	}
-	(*buf)[datalen] = '\0';
-
-#ifdef STR_TRANSLATION
-	if ( datalen > 0 && ( ber->ber_options & LBER_TRANSLATE_STRINGS ) != 0
-	    && ber->ber_decode_translate_proc ) {
-		++datalen;
-		if ( (*(ber->ber_decode_translate_proc))( buf, &datalen, 1 )
-		    != 0 ) {
-			LBER_FREE( *buf );
-			*buf = NULL;
-			return( LBER_DEFAULT );
+	if ( ber_first_element( b->ber, &len, &last ) != LBER_DEFAULT ) {
+		for ( ; b->ber->ber_ptr < last; i++ )
+		{
+			if (ber_skip_tag( b->ber, &len ) == LBER_DEFAULT) break;
+			b->ber->ber_ptr += len;
+			b->ber->ber_tag = *(unsigned char *)b->ber->ber_ptr;
 		}
 	}
-#endif /* STR_TRANSLATION */
 
-	return( tag );
+	if ( rlen ) *rlen = i;
+
+	if ( i == 0 )
+	{
+		*b->res.c = NULL;
+		return 0;
+	}
+
+	n = i;
+
+	/* Allocate the result vector */
+	switch (b->choice) {
+	case ChArray:
+		*b->res.c = ber_memalloc_x( (n+1) * sizeof( char * ), b->ber->ber_memctx);
+		if ( *b->res.c == NULL )
+			return LBER_DEFAULT;
+		(*b->res.c)[n] = NULL;
+		break;
+	case BvArray:
+		*b->res.ba = ber_memalloc_x( (n+1) * sizeof( struct berval ), b->ber->ber_memctx);
+		if ( *b->res.ba == NULL )
+			return LBER_DEFAULT;
+		(*b->res.ba)[n].bv_val = NULL;
+		break;
+	case BvVec:
+		*b->res.bv = ber_memalloc_x( (n+1) * sizeof( struct berval *), b->ber->ber_memctx);
+		if ( *b->res.bv == NULL )
+			return LBER_DEFAULT;
+		(*b->res.bv)[n] = NULL;
+		break;
+	case BvOff:
+		*b->res.ba = ber_memalloc_x( (n+1) * b->siz, b->ber->ber_memctx );
+		if ( *b->res.ba == NULL )
+			return LBER_DEFAULT;
+		((struct berval *)((long)(*b->res.ba) + n*b->siz +
+			b->off))->bv_val = NULL;
+		break;
+	}
+	b->ber->ber_ptr = orig;
+	b->ber->ber_tag = tag;
+	ber_skip_tag( b->ber, &len );
+	
+	for (n=0; n<i; n++)
+	{
+		tag = ber_next_element( b->ber, &len, last );
+		if ( ber_get_stringbv( b->ber, &bv, b->alloc ) == LBER_DEFAULT )
+			goto nomem;
+
+		/* store my result */
+		switch (b->choice) {
+		case ChArray:
+			(*b->res.c)[n] = bv.bv_val;
+			break;
+		case BvArray:
+			(*b->res.ba)[n] = bv;
+			break;
+		case BvVec:
+			bvp = ber_memalloc_x( sizeof( struct berval ), b->ber->ber_memctx);
+			if ( !bvp ) {
+				LBER_FREE(bv.bv_val);
+				goto nomem;
+			}
+			(*b->res.bv)[n] = bvp;
+			*bvp = bv;
+			break;
+		case BvOff:
+			*(BerVarray)((long)(*b->res.ba)+n*b->siz+b->off) = bv;
+			break;
+		}
+	}
+	return tag;
+nomem:
+	if (b->alloc || b->choice == BvVec)
+	{
+		for (--n; n>=0; n--)
+		{
+			switch(b->choice) {
+			case ChArray: LBER_FREE((*b->res.c)[n]); break;
+			case BvArray: LBER_FREE((*b->res.ba)[n].bv_val); break;
+			case BvVec: LBER_FREE((*b->res.bv)[n]->bv_val);
+				LBER_FREE((*b->res.bv)[n]); break;
+			default: break;
+			}
+		}
+	}
+	LBER_FREE(*b->res.c);
+	*b->res.c = NULL;
+	return LBER_DEFAULT;
 }
 
 ber_tag_t
-ber_get_stringal( BerElement *ber, struct berval **bv )
+ber_get_stringbv( BerElement *ber, struct berval *bv, int alloc )
 {
-	ber_len_t	len;
 	ber_tag_t	tag;
 
 	assert( ber != NULL );
 	assert( bv != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
-	if ( (tag = ber_skip_tag( ber, &len )) == LBER_DEFAULT ) {
-		*bv = NULL;
-		return( LBER_DEFAULT );
+	if ( (tag = ber_skip_tag( ber, &bv->bv_len )) == LBER_DEFAULT ) {
+		bv->bv_val = NULL;
+		return LBER_DEFAULT;
 	}
 
-	if ( (*bv = (struct berval *) LBER_MALLOC( sizeof(struct berval) )) == NULL )
-		return( LBER_DEFAULT );
+	if ( (ber_len_t) ber_pvt_ber_remaining( ber ) < bv->bv_len ) {
+		return LBER_DEFAULT;
+	}
 
-	if ( ((*bv)->bv_val = (char *) LBER_MALLOC( len + 1 )) == NULL ) {
+	if ( alloc ) {
+		if ( (bv->bv_val = (char *) ber_memalloc_x( bv->bv_len + 1, ber->ber_memctx )) == NULL ) {
+			return LBER_DEFAULT;
+		}
+
+		if ( bv->bv_len > 0 && (ber_len_t) ber_read( ber, bv->bv_val,
+			bv->bv_len ) != bv->bv_len ) {
+			LBER_FREE( bv->bv_val );
+			bv->bv_val = NULL;
+			return LBER_DEFAULT;
+		}
+	} else {
+		bv->bv_val = ber->ber_ptr;
+		ber->ber_ptr += bv->bv_len;
+	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
+	bv->bv_val[bv->bv_len] = '\0';
+
+	return tag;
+}
+
+ber_tag_t
+ber_get_stringa( BerElement *ber, char **buf )
+{
+	BerValue	bv;
+	ber_tag_t	tag;
+
+	assert( buf != NULL );
+
+	tag = ber_get_stringbv( ber, &bv, 1 );
+	*buf = bv.bv_val;
+
+	return tag;
+}
+
+ber_tag_t
+ber_get_stringal( BerElement *ber, struct berval **bv )
+{
+	ber_tag_t	tag;
+
+	assert( ber != NULL );
+	assert( bv != NULL );
+
+	*bv = (struct berval *) ber_memalloc_x( sizeof(struct berval), ber->ber_memctx );
+	if ( *bv == NULL ) {
+		return LBER_DEFAULT;
+	}
+
+	tag = ber_get_stringbv( ber, *bv, 1 );
+	if ( tag == LBER_DEFAULT ) {
 		LBER_FREE( *bv );
 		*bv = NULL;
-		return( LBER_DEFAULT );
 	}
-
-	if ( (ber_len_t) ber_read( ber, (*bv)->bv_val, len ) != len ) {
-		ber_bvfree( *bv );
-		*bv = NULL;
-		return( LBER_DEFAULT );
-	}
-	((*bv)->bv_val)[len] = '\0';
-	(*bv)->bv_len = len;
-
-#ifdef STR_TRANSLATION
-	if ( len > 0 && ( ber->ber_options & LBER_TRANSLATE_STRINGS ) != 0
-	    && ber->ber_decode_translate_proc ) {
-		++len;
-		if ( (*(ber->ber_decode_translate_proc))( &((*bv)->bv_val),
-		    &len, 1 ) != 0 ) {
-			ber_bvfree( *bv );
-			*bv = NULL;
-			return( LBER_DEFAULT );
-		}
-		(*bv)->bv_len = len - 1;
-	}
-#endif /* STR_TRANSLATION */
-
-	return( tag );
+	return tag;
 }
 
 ber_tag_t
@@ -376,31 +507,33 @@ ber_get_bitstringa(
 	assert( buf != NULL );
 	assert( blen != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	if ( (tag = ber_skip_tag( ber, &datalen )) == LBER_DEFAULT ) {
 		*buf = NULL;
-		return( LBER_DEFAULT );
+		return LBER_DEFAULT;
 	}
 	--datalen;
 
-	if ( (*buf = (char *) LBER_MALLOC( datalen )) == NULL )
-		return( LBER_DEFAULT );
+	if ( (*buf = (char *) ber_memalloc_x( datalen, ber->ber_memctx )) == NULL ) {
+		return LBER_DEFAULT;
+	}
 
 	if ( ber_read( ber, (char *)&unusedbits, 1 ) != 1 ) {
 		LBER_FREE( buf );
 		*buf = NULL;
-		return( LBER_DEFAULT );
+		return LBER_DEFAULT;
 	}
 
-	if ( (unsigned long) ber_read( ber, *buf, datalen ) != datalen ) {
+	if ( (ber_len_t) ber_read( ber, *buf, datalen ) != datalen ) {
 		LBER_FREE( buf );
 		*buf = NULL;
-		return( LBER_DEFAULT );
+		return LBER_DEFAULT;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	*blen = datalen * 8 - unusedbits;
-	return( tag );
+	return tag;
 }
 
 ber_tag_t
@@ -410,13 +543,16 @@ ber_get_null( BerElement *ber )
 	ber_tag_t	tag;
 
 	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
-	if ( (tag = ber_skip_tag( ber, &len )) == LBER_DEFAULT )
-		return( LBER_DEFAULT );
+	if ( (tag = ber_skip_tag( ber, &len )) == LBER_DEFAULT ) {
+		return LBER_DEFAULT;
+	}
 
-	if ( len != 0 )
-		return( LBER_DEFAULT );
+	if ( len != 0 ) {
+		return LBER_DEFAULT;
+	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	return( tag );
 }
@@ -432,12 +568,12 @@ ber_get_boolean(
 	assert( ber != NULL );
 	assert( boolval != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	rc = ber_get_int( ber, &longbool );
 	*boolval = longbool;
 
-	return( rc );
+	return rc;
 }
 
 ber_tag_t
@@ -453,83 +589,73 @@ ber_first_element(
 	/* skip the sequence header, use the len to mark where to stop */
 	if ( ber_skip_tag( ber, len ) == LBER_DEFAULT ) {
 		*last = NULL;
-		return( LBER_DEFAULT );
+		return LBER_DEFAULT;
 	}
+	ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 
 	*last = ber->ber_ptr + *len;
 
 	if ( *last == ber->ber_ptr ) {
-		return( LBER_DEFAULT );
+		return LBER_DEFAULT;
 	}
 
-	return( ber_peek_tag( ber, len ) );
+	return ber_peek_tag( ber, len );
 }
 
 ber_tag_t
 ber_next_element(
 	BerElement *ber,
 	ber_len_t *len,
-	char *last )
+	LDAP_CONST char *last )
 {
 	assert( ber != NULL );
 	assert( len != NULL );
 	assert( last != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
-	if ( ber->ber_ptr == last ) {
-		return( LBER_DEFAULT );
+	if ( ber->ber_ptr >= last ) {
+		return LBER_DEFAULT;
 	}
 
-	return( ber_peek_tag( ber, len ) );
+	return ber_peek_tag( ber, len );
 }
 
 /* VARARGS */
 ber_tag_t
-ber_scanf
-#if HAVE_STDARG
-	( BerElement *ber,
+ber_scanf ( BerElement *ber,
 	LDAP_CONST char *fmt,
 	... )
-#else
-	( va_alist )
-va_dcl
-#endif
 {
 	va_list		ap;
-#ifndef HAVE_STDARG
-	BerElement	*ber;
-	char		*fmt;
-#endif
 	LDAP_CONST char		*fmt_reset;
-	char		*last;
-	char		*s, **ss, ***sss;
-	struct berval 	***bv, **bvp, *bval;
+	char		*s, **ss;
+	struct berval	**bvp, *bval;
 	ber_int_t	*i;
-	int j;
 	ber_len_t	*l;
 	ber_tag_t	*t;
-	ber_tag_t	rc, tag;
+	ber_tag_t	rc;
 	ber_len_t	len;
 
-#ifdef HAVE_STDARG
 	va_start( ap, fmt );
-#else
-	va_start( ap );
-	ber = va_arg( ap, BerElement * );
-	fmt = va_arg( ap, char * );
-#endif
 
 	assert( ber != NULL );
 	assert( fmt != NULL );
 
-	assert( BER_VALID( ber ) );
+	assert( LBER_VALID( ber ) );
 
 	fmt_reset = fmt;
 
+#ifdef NEW_LOGGING
+	LDAP_LOG( BER, ENTRY, "ber_scanf fmt (%s) ber:\n", fmt, 0, 0 );
+
+	if ( LDAP_LOGS_TEST(BER, DETAIL2 ))
+			BER_DUMP(( "liblber", LDAP_LEVEL_DETAIL2, ber, 1 ));
+#else
 	ber_log_printf( LDAP_DEBUG_TRACE, ber->ber_debug,
 		"ber_scanf fmt (%s) ber:\n", fmt );
 	ber_log_dump( LDAP_DEBUG_BER, ber->ber_debug, ber, 1 );
+#endif
 
 	for ( rc = 0; *fmt && rc != LBER_DEFAULT; fmt++ ) {
 		/* When this is modified, remember to update
@@ -576,10 +702,14 @@ va_dcl
 			rc = ber_get_stringb( ber, s, l );
 			break;
 
+		case 'm':	/* octet string in berval, in-place */
+			bval = va_arg( ap, struct berval * );
+			rc = ber_get_stringbv( ber, bval, 0 );
+			break;
+
 		case 'o':	/* octet string in a supplied berval */
 			bval = va_arg( ap, struct berval * );
-			ber_peek_tag( ber, &bval->bv_len );
-			rc = ber_get_stringa( ber, &bval->bv_val );
+			rc = ber_get_stringbv( ber, bval, 1 );
 			break;
 
 		case 'O':	/* octet string - allocate & include length */
@@ -604,50 +734,63 @@ va_dcl
 			break;
 
 		case 'v':	/* sequence of strings */
-			sss = va_arg( ap, char *** );
-			*sss = NULL;
-			j = 0;
-			for ( tag = ber_first_element( ber, &len, &last );
-			    tag != LBER_DEFAULT && rc != LBER_DEFAULT;
-			    tag = ber_next_element( ber, &len, last ) )
-			{
-				*sss = (char **) LBER_REALLOC( *sss,
-					(j + 2) * sizeof(char *) );
-
-				rc = ber_get_stringa( ber, &((*sss)[j]) );
-				j++;
-			}
-			if ( j > 0 )
-				(*sss)[j] = NULL;
+		{
+			bgbvr cookie = { ChArray };
+			cookie.ber = ber;
+			cookie.res.c = va_arg( ap, char *** );
+			cookie.alloc = 1;
+			rc = ber_get_stringbvl( &cookie, NULL );
 			break;
+		}
 
 		case 'V':	/* sequence of strings + lengths */
-			bv = va_arg( ap, struct berval *** );
-			*bv = NULL;
-			j = 0;
-			for ( tag = ber_first_element( ber, &len, &last );
-			    tag != LBER_DEFAULT && rc != LBER_DEFAULT;
-			    tag = ber_next_element( ber, &len, last ) )
-			{
-				*bv = (struct berval **) LBER_REALLOC( *bv,
-					(j + 2) * sizeof(struct berval *) );
-		
-				rc = ber_get_stringal( ber, &((*bv)[j]) );
-				j++;
-			}
-			if ( j > 0 )
-				(*bv)[j] = NULL;
+		{
+			bgbvr cookie = { BvVec };
+			cookie.ber = ber;
+			cookie.res.bv = va_arg( ap, struct berval *** );
+			cookie.alloc = 1;
+			rc = ber_get_stringbvl( &cookie, NULL );
 			break;
+		}
+
+		case 'W':	/* bvarray */
+		{
+			bgbvr cookie = { BvArray };
+			cookie.ber = ber;
+			cookie.res.ba = va_arg( ap, struct berval ** );
+			cookie.alloc = 1;
+			rc = ber_get_stringbvl( &cookie, NULL );
+			break;
+		}
+
+		case 'M':	/* bvoffarray - must include address of
+				 * a record len, and record offset.
+				 * number of records will be returned thru
+				 * len ptr on finish. parsed in-place.
+				 */
+		{
+			bgbvr cookie = { BvOff };
+			cookie.ber = ber;
+			cookie.res.ba = va_arg( ap, struct berval ** );
+			cookie.alloc = 0;
+			l = va_arg( ap, ber_len_t * );
+			cookie.siz = *l;
+			cookie.off = va_arg( ap, ber_len_t );
+			rc = ber_get_stringbvl( &cookie, l );
+			break;
+		}
 
 		case 'x':	/* skip the next element - whatever it is */
 			if ( (rc = ber_skip_tag( ber, &len )) == LBER_DEFAULT )
 				break;
 			ber->ber_ptr += len;
+			ber->ber_tag = *(unsigned char *)ber->ber_ptr;
 			break;
 
 		case '{':	/* begin sequence */
 		case '[':	/* begin set */
-			if ( *(fmt + 1) != 'v' && *(fmt + 1) != 'V' )
+			if ( *(fmt + 1) != 'v' && *(fmt + 1) != 'V'
+				&& *(fmt + 1) != 'W' && *(fmt + 1) != 'M' )
 				rc = ber_skip_tag( ber, &len );
 			break;
 
@@ -657,8 +800,13 @@ va_dcl
 
 		default:
 			if( ber->ber_debug ) {
+#ifdef NEW_LOGGING
+				LDAP_LOG( BER, ERR, 
+					"ber_scanf: unknown fmt %c\n", *fmt, 0, 0 );
+#else
 				ber_log_printf( LDAP_DEBUG_ANY, ber->ber_debug,
 					"ber_scanf: unknown fmt %c\n", *fmt );
+#endif
 			}
 			rc = LBER_DEFAULT;
 			break;
@@ -666,19 +814,12 @@ va_dcl
 	}
 
 	va_end( ap );
-
 	if ( rc == LBER_DEFAULT ) {
 	    /*
 	     * Error.  Reclaim malloced memory that was given to the caller.
 	     * Set allocated pointers to NULL, "data length" outvalues to 0.
 	     */
-#ifdef HAVE_STDARG
 	    va_start( ap, fmt );
-#else
-	    va_start( ap );
-	    (void) va_arg( ap, BerElement * );
-	    (void) va_arg( ap, char * );
-#endif
 
 	    for ( ; fmt_reset < fmt; fmt_reset++ ) {
 		switch ( *fmt_reset ) {
@@ -747,25 +888,10 @@ va_dcl
 			break;
 
 		case 'v':	/* sequence of strings */
-			sss = va_arg( ap, char *** );
-			if ( *sss ) {
-				for (j = 0;  (*sss)[j];  j++) {
-					LBER_FREE( (*sss)[j] );
-					(*sss)[j] = NULL;
-				}
-				LBER_FREE( *sss );
-				*sss = NULL;
-			}
-			break;
-
 		case 'V':	/* sequence of strings + lengths */
-			bv = va_arg( ap, struct berval *** );
-			if ( *bv ) {
-				ber_bvecfree( *bv );
-				*bv = NULL;
-			}
-			break;
-
+		case 'W':	/* BerVarray */
+		case 'm':	/* berval in-place */
+		case 'M':	/* BVoff array in-place */
 		case 'n':	/* null */
 		case 'x':	/* skip the next element - whatever it is */
 		case '{':	/* begin sequence */
@@ -783,19 +909,5 @@ va_dcl
 	    va_end( ap );
 	}
 
-	return( rc );
+	return rc;
 }
-
-
-#ifdef STR_TRANSLATION
-void
-ber_set_string_translators( BerElement *ber, BERTranslateProc encode_proc,
-	BERTranslateProc decode_proc )
-{
-	assert( ber != NULL );
-	assert( BER_VALID( ber ) );
-
-    ber->ber_encode_translate_proc = encode_proc;
-    ber->ber_decode_translate_proc = decode_proc;
-}
-#endif /* STR_TRANSLATION */
