@@ -1,7 +1,7 @@
 /* Encode.c - ber output encoding routines */
-/* $OpenLDAP: pkg/ldap/libraries/liblber/encode.c,v 1.50 2002/02/13 20:26:24 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/liblber/encode.c,v 1.55 2003/05/02 11:29:28 hallvard Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /* Portions
@@ -45,7 +45,7 @@ static int ber_put_int_or_enum LDAP_P((
 	ber_tag_t tag ));
 
 
-static ber_len_t
+static int
 ber_calc_taglen( ber_tag_t tag )
 {
 	int	i;
@@ -68,8 +68,8 @@ ber_put_tag(
 	int nosos )
 {
 	int rc;
-	ber_len_t	taglen;
-	ber_len_t	i;
+	int	taglen;
+	int	i;
 	unsigned char nettag[sizeof(ber_tag_t)];
 
 	assert( ber != NULL );
@@ -83,7 +83,7 @@ ber_put_tag(
 	}
 
 	rc = ber_write( ber,
-		&nettag[sizeof(ber_tag_t) - taglen],
+		(char *) &nettag[sizeof(ber_tag_t) - taglen],
 	    taglen, nosos );
 
 	return rc;
@@ -165,7 +165,7 @@ ber_put_len( BerElement *ber, ber_len_t len, int nosos )
 
 	/* write the length itself */
 	rc = ber_write( ber,
-		&netlen[sizeof(ber_len_t)-i],
+		(char *) &netlen[sizeof(ber_len_t)-i],
 		i, nosos );
 
 	return rc == i ?  i+1 : -1;
@@ -178,8 +178,8 @@ ber_put_int_or_enum(
 	ber_tag_t tag )
 {
 	int rc;
-	int	i, j, sign;
-	ber_len_t	len, lenlen, taglen;
+	int	i, j, sign, taglen, lenlen;
+	ber_len_t	len;
 	ber_uint_t	unum, mask;
 	unsigned char netnum[sizeof(ber_uint_t)];
 
@@ -230,7 +230,7 @@ ber_put_int_or_enum(
 	}
 
 	rc = ber_write( ber,
-		&netnum[sizeof(ber_int_t) - i],
+		(char *) &netnum[sizeof(ber_int_t) - i],
 		i, 0 );
 
 	/* length of tag + length + contents */
@@ -276,8 +276,7 @@ ber_put_ostring(
 	ber_len_t len,
 	ber_tag_t tag )
 {
-	ber_len_t	taglen, lenlen;
-	int rc;
+	int taglen, lenlen, rc;
 
 	assert( ber != NULL );
 	assert( str != NULL );
@@ -339,7 +338,8 @@ ber_put_bitstring(
 	ber_len_t blen /* in bits */,
 	ber_tag_t tag )
 {
-	ber_len_t		taglen, lenlen, len;
+	int				taglen, lenlen;
+	ber_len_t		len;
 	unsigned char	unusedbits;
 
 	assert( ber != NULL );
@@ -376,7 +376,7 @@ ber_put_bitstring(
 int
 ber_put_null( BerElement *ber, ber_tag_t tag )
 {
-	ber_len_t	taglen;
+	int	taglen;
 
 	assert( ber != NULL );
 	assert( LBER_VALID( ber ) );
@@ -402,7 +402,7 @@ ber_put_boolean(
 	ber_int_t boolval,
 	ber_tag_t tag )
 {
-	ber_len_t		taglen;
+	int				taglen;
 	unsigned char	c;
 
 	assert( ber != NULL );
@@ -442,7 +442,7 @@ ber_start_seqorset(
 	assert( ber != NULL );
 	assert( LBER_VALID( ber ) );
 
-	new = (Seqorset *) LBER_CALLOC( 1, sizeof(Seqorset) );
+	new = (Seqorset *) ber_memcalloc_x( 1, sizeof(Seqorset), ber->ber_memctx );
 
 	if ( new == NULL ) {
 		return -1;
@@ -497,7 +497,8 @@ ber_put_seqorset( BerElement *ber )
 	int rc;
 	ber_len_t	len;
 	unsigned char netlen[sizeof(ber_len_t)];
-	ber_len_t	taglen, lenlen;
+	int			taglen;
+	ber_len_t	lenlen;
 	unsigned char	ltag = 0x80U + FOUR_BYTE_LEN - 1;
 	Seqorset	*next;
 	Seqorset	**sos = &ber->ber_sos;
@@ -568,7 +569,7 @@ ber_put_seqorset( BerElement *ber )
 
 			/* the length itself */
 			rc  = ber_write( ber,
-				&netlen[sizeof(ber_len_t) - (FOUR_BYTE_LEN-1)],
+				(char *) &netlen[sizeof(ber_len_t) - (FOUR_BYTE_LEN-1)],
 				FOUR_BYTE_LEN-1, 1 );
 
 			if( rc != FOUR_BYTE_LEN - 1 ) {
@@ -579,7 +580,7 @@ ber_put_seqorset( BerElement *ber )
 		(*sos)->sos_ber->ber_ptr += len;
 
 	} else {
-		ber_len_t i;
+		int i;
 		unsigned char nettag[sizeof(ber_tag_t)];
 		ber_tag_t tmptag = (*sos)->sos_tag;
 
@@ -648,7 +649,7 @@ ber_put_seqorset( BerElement *ber )
 	}
 
 	/* we're done with this seqorset, so free it up */
-	LBER_FREE( (char *) (*sos) );
+	ber_memfree_x( (char *) (*sos), ber->ber_memctx );
 	*sos = next;
 
 	return taglen + lenlen + len;
@@ -811,8 +812,8 @@ ber_printf( BerElement *ber, LDAP_CONST char *fmt, ... )
 		default:
 			if( ber->ber_debug ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "liblber", LDAP_LEVEL_ERR,
-					   "ber_printf: unknown fmt %c\n", *fmt ));
+				LDAP_LOG( BER, ERR, 
+					"ber_printf: unknown fmt %c\n", *fmt, 0, 0 );
 #else
 				ber_log_printf( LDAP_DEBUG_ANY, ber->ber_debug,
 					"ber_printf: unknown fmt %c\n", *fmt );
