@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/error.c,v 1.37 2002/01/04 20:17:38 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/error.c,v 1.50.2.1 2003/05/31 19:01:39 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -22,75 +22,92 @@ struct ldaperror {
 };
 
 static struct ldaperror ldap_builtin_errlist[] = {
-	{LDAP_SUCCESS, 					"Success" },
-	{LDAP_OPERATIONS_ERROR, 		"Operations error" },
-	{LDAP_PROTOCOL_ERROR, 			"Protocol error" },
-	{LDAP_TIMELIMIT_EXCEEDED,		"Time limit exceeded" },
-	{LDAP_SIZELIMIT_EXCEEDED, 		"Size limit exceeded" },
-	{LDAP_COMPARE_FALSE, 			"Compare False" },
-	{LDAP_COMPARE_TRUE, 			"Compare True" },
-	{LDAP_STRONG_AUTH_NOT_SUPPORTED, "Authentication method not supported" },
-	{LDAP_STRONG_AUTH_REQUIRED, 	"Strong authentication required" },
-	{LDAP_PARTIAL_RESULTS, 			"Partial results and referral received" },
+	{LDAP_SUCCESS, 					N_("Success")},
+	{LDAP_OPERATIONS_ERROR, 		N_("Operations error")},
+	{LDAP_PROTOCOL_ERROR, 			N_("Protocol error")},
+	{LDAP_TIMELIMIT_EXCEEDED,		N_("Time limit exceeded")},
+	{LDAP_SIZELIMIT_EXCEEDED, 		N_("Size limit exceeded")},
+	{LDAP_COMPARE_FALSE, 			N_("Compare False")},
+	{LDAP_COMPARE_TRUE, 			N_("Compare True")},
+	{LDAP_STRONG_AUTH_NOT_SUPPORTED, N_("Authentication method not supported")},
+	{LDAP_STRONG_AUTH_REQUIRED, 	N_("Strong(er) authentication required")},
+	{LDAP_PARTIAL_RESULTS, 			N_("Partial results and referral received")},
 
-	{LDAP_REFERRAL,					"Referral"},
-	{LDAP_ADMINLIMIT_EXCEEDED,		"Administrative limit exceeded"},
+	{LDAP_REFERRAL,					N_("Referral")},
+	{LDAP_ADMINLIMIT_EXCEEDED,		N_("Administrative limit exceeded")},
 	{LDAP_UNAVAILABLE_CRITICAL_EXTENSION,
-									"Criticial extension is unavailable"},
-	{LDAP_CONFIDENTIALITY_REQUIRED,	"Confidentiality required"},
-	{LDAP_SASL_BIND_IN_PROGRESS,	"SASL bind in progress"},
+									N_("Critical extension is unavailable")},
+	{LDAP_CONFIDENTIALITY_REQUIRED,	N_("Confidentiality required")},
+	{LDAP_SASL_BIND_IN_PROGRESS,	N_("SASL bind in progress")},
 
-	{LDAP_NO_SUCH_ATTRIBUTE, 		"No such attribute" },
-	{LDAP_UNDEFINED_TYPE, 			"Undefined attribute type" },
-	{LDAP_INAPPROPRIATE_MATCHING, 	"Inappropriate matching" },
-	{LDAP_CONSTRAINT_VIOLATION, 	"Constraint violation" },
-	{LDAP_TYPE_OR_VALUE_EXISTS, 	"Type or value exists" },
-	{LDAP_INVALID_SYNTAX, 			"Invalid syntax" },
+	{LDAP_NO_SUCH_ATTRIBUTE, 		N_("No such attribute")},
+	{LDAP_UNDEFINED_TYPE, 			N_("Undefined attribute type")},
+	{LDAP_INAPPROPRIATE_MATCHING, 	N_("Inappropriate matching")},
+	{LDAP_CONSTRAINT_VIOLATION, 	N_("Constraint violation")},
+	{LDAP_TYPE_OR_VALUE_EXISTS, 	N_("Type or value exists")},
+	{LDAP_INVALID_SYNTAX, 			N_("Invalid syntax")},
 
-	{LDAP_NO_SUCH_OBJECT, 			"No such object" },
-	{LDAP_ALIAS_PROBLEM, 			"Alias problem" },
-	{LDAP_INVALID_DN_SYNTAX,		"Invalid DN syntax" },
-	{LDAP_IS_LEAF, 					"Entry is a leaf" },
-	{LDAP_ALIAS_DEREF_PROBLEM,	 	"Alias dereferencing problem" },
+	{LDAP_NO_SUCH_OBJECT, 			N_("No such object")},
+	{LDAP_ALIAS_PROBLEM, 			N_("Alias problem")},
+	{LDAP_INVALID_DN_SYNTAX,		N_("Invalid DN syntax")},
+	{LDAP_IS_LEAF, 					N_("Entry is a leaf")},
+	{LDAP_ALIAS_DEREF_PROBLEM,	 	N_("Alias dereferencing problem")},
 
-	{LDAP_INAPPROPRIATE_AUTH, 		"Inappropriate authentication" },
-	{LDAP_INVALID_CREDENTIALS, 		"Invalid credentials" },
-	{LDAP_INSUFFICIENT_ACCESS, 		"Insufficient access" },
-	{LDAP_BUSY, 					"DSA is busy" },
-	{LDAP_UNAVAILABLE, 				"DSA is unavailable" },
-	{LDAP_UNWILLING_TO_PERFORM, 	"DSA is unwilling to perform" },
-	{LDAP_LOOP_DETECT, 				"Loop detected" },
+	{LDAP_PROXY_AUTHZ_FAILURE,		N_("Proxy Authorization Failure")},
+	{LDAP_INAPPROPRIATE_AUTH, 		N_("Inappropriate authentication")},
+	{LDAP_INVALID_CREDENTIALS, 		N_("Invalid credentials")},
+	{LDAP_INSUFFICIENT_ACCESS, 		N_("Insufficient access")},
+	{LDAP_BUSY, 					N_("Server is busy")},
+	{LDAP_UNAVAILABLE, 				N_("Server is unavailable")},
+	{LDAP_UNWILLING_TO_PERFORM, 	N_("Server is unwilling to perform")},
+	{LDAP_LOOP_DETECT, 				N_("Loop detected")},
 
-	{LDAP_NAMING_VIOLATION, 		"Naming violation" },
-	{LDAP_OBJECT_CLASS_VIOLATION, 	"Object class violation" },
-	{LDAP_NOT_ALLOWED_ON_NONLEAF, 	"Operation not allowed on nonleaf" },
-	{LDAP_NOT_ALLOWED_ON_RDN,	 	"Operation not allowed on RDN" },
-	{LDAP_ALREADY_EXISTS, 			"Already exists" },
-	{LDAP_NO_OBJECT_CLASS_MODS, 	"Cannot modify object class" },
-	{LDAP_RESULTS_TOO_LARGE,		"Results too large" },
-	{LDAP_AFFECTS_MULTIPLE_DSAS,	"Operation affects multiple DSAs" },
+	{LDAP_NAMING_VIOLATION, 		N_("Naming violation")},
+	{LDAP_OBJECT_CLASS_VIOLATION, 	N_("Object class violation")},
+	{LDAP_NOT_ALLOWED_ON_NONLEAF, 	N_("Operation not allowed on non-leaf")},
+	{LDAP_NOT_ALLOWED_ON_RDN,	 	N_("Operation not allowed on RDN")},
+	{LDAP_ALREADY_EXISTS, 			N_("Already exists")},
+	{LDAP_NO_OBJECT_CLASS_MODS, 	N_("Cannot modify object class")},
+	{LDAP_RESULTS_TOO_LARGE,		N_("Results too large")},
+	{LDAP_AFFECTS_MULTIPLE_DSAS,	N_("Operation affects multiple DSAs")},
 
-	{LDAP_OTHER, 					"Unknown error" },
+	{LDAP_OTHER, 					N_("Internal (implementation specific) error")},
 
 	/* API ResultCodes */
-	{LDAP_SERVER_DOWN,				"Can't contact LDAP server" },
-	{LDAP_LOCAL_ERROR,				"Local error" },
-	{LDAP_ENCODING_ERROR,			"Encoding error" },
-	{LDAP_DECODING_ERROR,			"Decoding error" },
-	{LDAP_TIMEOUT,					"Timed out" },
-	{LDAP_AUTH_UNKNOWN,				"Unknown authentication method" },
-	{LDAP_FILTER_ERROR,				"Bad search filter" },
-	{LDAP_USER_CANCELLED,			"User cancelled operation" },
-	{LDAP_PARAM_ERROR,				"Bad parameter to an ldap routine" },
-	{LDAP_NO_MEMORY,				"Out of memory" },
+	{LDAP_SERVER_DOWN,				N_("Can't contact LDAP server")},
+	{LDAP_LOCAL_ERROR,				N_("Local error")},
+	{LDAP_ENCODING_ERROR,			N_("Encoding error")},
+	{LDAP_DECODING_ERROR,			N_("Decoding error")},
+	{LDAP_TIMEOUT,					N_("Timed out")},
+	{LDAP_AUTH_UNKNOWN,				N_("Unknown authentication method")},
+	{LDAP_FILTER_ERROR,				N_("Bad search filter")},
+	{LDAP_USER_CANCELLED,			N_("User cancelled operation")},
+	{LDAP_PARAM_ERROR,				N_("Bad parameter to an ldap routine")},
+	{LDAP_NO_MEMORY,				N_("Out of memory")},
 
-	{LDAP_CONNECT_ERROR,			"Connect error" },
-	{LDAP_NOT_SUPPORTED,			"Not Supported" },
-	{LDAP_CONTROL_NOT_FOUND,		"Control not found" },
-	{LDAP_NO_RESULTS_RETURNED,		"No results returned" },
-	{LDAP_MORE_RESULTS_TO_RETURN,	"More results to return" },
-	{LDAP_CLIENT_LOOP,				"Client Loop" },
-	{LDAP_REFERRAL_LIMIT_EXCEEDED,	"Referral Limit Exceeded" },
+	{LDAP_CONNECT_ERROR,			N_("Connect error")},
+	{LDAP_NOT_SUPPORTED,			N_("Not Supported")},
+	{LDAP_CONTROL_NOT_FOUND,		N_("Control not found")},
+	{LDAP_NO_RESULTS_RETURNED,		N_("No results returned")},
+	{LDAP_MORE_RESULTS_TO_RETURN,	N_("More results to return")},
+	{LDAP_CLIENT_LOOP,				N_("Client Loop")},
+	{LDAP_REFERRAL_LIMIT_EXCEEDED,	N_("Referral Limit Exceeded")},
+
+#ifdef LDAP_CLIENT_UPDATE
+	{LDAP_CUP_RESOURCES_EXHAUSTED,	N_("Client Update Resource Exhausted")},
+	{LDAP_CUP_SECURITY_VIOLATION,	N_("Client Update Security Violation")},
+	{LDAP_CUP_INVALID_COOKIE,		N_("Client Update Invalid Cookie")},
+	{LDAP_CUP_UNSUPPORTED_SCHEME,	N_("Client Update Unsupported Scheme")},
+	{LDAP_CUP_CLIENT_DISCONNECT,	N_("Client Update Client Disconnect")},
+	{LDAP_CUP_RELOAD_REQUIRED,		N_("Client Update Reload Required")},
+#endif
+
+#ifdef LDAP_EXOP_X_CANCEL
+	{LDAP_CANCELLED,				N_("Cancelled")},
+	{LDAP_NO_SUCH_OPERATION,		N_("No Operation to Cancel")},
+	{LDAP_TOO_LATE,					N_("Too Late to Cancel")},
+	{LDAP_CANNOT_CANCEL,			N_("Cannot Cancel")},
+#endif
 
 	{-1, NULL}
 };
@@ -98,33 +115,6 @@ static struct ldaperror ldap_builtin_errlist[] = {
 static struct ldaperror *ldap_errlist = ldap_builtin_errlist; 
 
 void ldap_int_error_init( void ) {
-#ifdef LDAP_NLS
-#define LDAP_NLS_SDK_CAT "openldap_sdk"
-#define LDAP_NLS_LIBLDAP_SET (0)
-
-	int	i;
-	nl_catd catd = catopen( LDAP_NLS_SDK_CAT, NL_CAT_LOCALE );
-
-	if( catd == -1 ) {
-		return;
-	}
-
-	for ( i=0; ldap_errlist[i].e_reason != NULL; i++ ) {
-		char *msg = catgets( catd,
-			LDAP_NLS_LIBLDAP_SET,
-			ldap_errlist[i].e_code, NULL );
-
-		if( msg != NULL ) {
-			msg = LDAP_STRDUP( msg );
-
-			if( msg != NULL ) {
-				ldap_errlist[i].e_reason = msg;
-			}
-		}
-	}
-
-	catclose( catd );
-#endif
 }
 
 static const struct ldaperror *
@@ -132,6 +122,7 @@ ldap_int_error( int err )
 {
 	int	i;
 
+	/* XXYYZ: O(n) search instead of O(1) lookup */
 	for ( i=0; ldap_errlist[i].e_reason != NULL; i++ ) {
 		if ( err == ldap_errlist[i].e_code ) {
 			return &ldap_errlist[i];
@@ -146,42 +137,53 @@ ldap_err2string( int err )
 {
 	const struct ldaperror *e;
 	
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_err2string\n", 0,0,0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_err2string\n", 0, 0, 0 );
+#endif
 
 	e = ldap_int_error( err );
 
-	return e ? e->e_reason : "Unknown error";
+	return e ? _(e->e_reason) : _("Unknown error");
 }
 
 /* deprecated */
 void
 ldap_perror( LDAP *ld, LDAP_CONST char *str )
 {
+    int i;
 	const struct ldaperror *e;
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_perror\n", 0,0,0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_perror\n", 0, 0, 0 );
+#endif
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
 	assert( str );
 
-	if ( ld == NULL ) {
-		fprintf( stderr, "ldap_perror: invalid session handle\n" );
-		return;
-	}
-
 	e = ldap_int_error( ld->ld_errno );
 
 	fprintf( stderr, "%s: %s (%d)\n",
 		str ? str : "ldap_perror",
-		e ? e->e_reason : "unknown LDAP result code",
+		e ? _(e->e_reason) : _("unknown LDAP result code"),
 		ld->ld_errno );
 
 	if ( ld->ld_matched != NULL && ld->ld_matched[0] != '\0' ) {
-		fprintf( stderr, "\tmatched DN: %s\n", ld->ld_matched );
+		fprintf( stderr, _("\tmatched DN: %s\n"), ld->ld_matched );
 	}
 
 	if ( ld->ld_error != NULL && ld->ld_error[0] != '\0' ) {
-		fprintf( stderr, "\tadditional info: %s\n", ld->ld_error );
+		fprintf( stderr, _("\tadditional info: %s\n"), ld->ld_error );
+	}
+
+	if ( ld->ld_referrals != NULL && ld->ld_referrals[0] != NULL) {
+		fprintf( stderr, _("\treferrals:\n") );
+		for (i=0; ld->ld_referrals[i]; i++) {
+			fprintf( stderr, _("\t\t%s\n"), ld->ld_referrals[i] );
+		}
 	}
 
 	fflush( stderr );
@@ -239,28 +241,31 @@ ldap_parse_result(
 	ber_tag_t tag;
 	BerElement	*ber;
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_parse_result\n", 0,0,0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_parse_result\n", 0, 0, 0 );
+#endif
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
 	assert( r != NULL );
 
-	if ( ld == NULL || r == NULL ) {
-		return LDAP_PARAM_ERROR;
-	}
-
 	if(errcodep != NULL) *errcodep = LDAP_SUCCESS;
 	if(matcheddnp != NULL) *matcheddnp = NULL;
 	if(errmsgp != NULL) *errmsgp = NULL;
 	if(referralsp != NULL) *referralsp = NULL;
 	if(serverctrls != NULL) *serverctrls = NULL;
 
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_lock( &ld->ld_res_mutex );
+#endif
 	/* Find the next result... */
 	for ( lm = r; lm != NULL; lm = lm->lm_chain ) {
 		/* skip over entries and references */
 		if( lm->lm_msgtype != LDAP_RES_SEARCH_ENTRY &&
 			lm->lm_msgtype != LDAP_RES_SEARCH_REFERENCE &&
-			lm->lm_msgtype != LDAP_RES_EXTENDED_PARTIAL )
+			lm->lm_msgtype != LDAP_RES_INTERMEDIATE )
 		{
 			break;
 		}
@@ -268,6 +273,9 @@ ldap_parse_result(
 
 	if( lm == NULL ) {
 		ld->ld_errno = LDAP_NO_RESULTS_RETURNED;
+#ifdef LDAP_R_COMPILE
+		ldap_pvt_thread_mutex_unlock( &ld->ld_res_mutex );
+#endif
 		return ld->ld_errno;
 	}
 
@@ -279,6 +287,10 @@ ldap_parse_result(
 		LDAP_FREE( ld->ld_matched );
 		ld->ld_matched = NULL;
 	}
+	if ( ld->ld_referrals ) {
+		LDAP_VFREE( ld->ld_referrals );
+		ld->ld_referrals = NULL;
+	}
 
 	/* parse results */
 
@@ -295,13 +307,7 @@ ldap_parse_result(
 		if( tag != LBER_ERROR ) {
 			/* peek for referrals */
 			if( ber_peek_tag(ber, &len) == LDAP_TAG_REFERRAL ) {
-				if( referralsp != NULL ) {
-					tag = ber_scanf( ber, "v", referralsp );
-
-				} else {
-					/* no place to put them so skip 'em */
-					tag = ber_scanf( ber, "x" );
-				}
+				tag = ber_scanf( ber, "v", &ld->ld_referrals );
 			}
 		}
 
@@ -363,12 +369,16 @@ ldap_parse_result(
 			*errmsgp = LDAP_STRDUP( ld->ld_error );
 		}
 
+		if( referralsp != NULL) {
+			*referralsp = ldap_value_dup( ld->ld_referrals );
+		}
+
 		/* Find the next result... */
 		for ( lm = lm->lm_chain; lm != NULL; lm = lm->lm_chain ) {
 			/* skip over entries and references */
 			if( lm->lm_msgtype != LDAP_RES_SEARCH_ENTRY &&
 				lm->lm_msgtype != LDAP_RES_SEARCH_REFERENCE &&
-				lm->lm_msgtype != LDAP_RES_EXTENDED_PARTIAL )
+				lm->lm_msgtype != LDAP_RES_INTERMEDIATE )
 			{
 				/* more results to return */
 				errcode = LDAP_MORE_RESULTS_TO_RETURN;
@@ -380,6 +390,9 @@ ldap_parse_result(
 	if ( freeit ) {
 		ldap_msgfree( r );
 	}
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_unlock( &ld->ld_res_mutex );
+#endif
 
 	return( errcode );
 }
