@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/extended.c,v 1.15 2002/01/04 20:17:38 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/extended.c,v 1.23.2.1 2003/05/31 19:01:39 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -30,6 +30,7 @@
 #include <ac/time.h>
 
 #include "ldap-int.h"
+#include "ldap_log.h"
 
 int
 ldap_extended_operation(
@@ -42,8 +43,13 @@ ldap_extended_operation(
 {
 	BerElement *ber;
 	int rc;
+	ber_int_t id;
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_extended_operation\n", 0,0,0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_extended_operation\n", 0, 0, 0 );
+#endif
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
@@ -56,26 +62,22 @@ ldap_extended_operation(
 		return( ld->ld_errno );
 	}
 
-	if( reqoid == NULL || *reqoid == '\0' || msgidp == NULL ) {
-		ld->ld_errno = LDAP_PARAM_ERROR;
-		return( ld->ld_errno );
-	}
-
 	/* create a message to send */
 	if ( (ber = ldap_alloc_ber_with_options( ld )) == NULL ) {
 		ld->ld_errno = LDAP_NO_MEMORY;
 		return( ld->ld_errno );
 	}
 
+	LDAP_NEXT_MSGID( ld, id );
 	if ( reqdata != NULL ) {
 		rc = ber_printf( ber, "{it{tstON}", /* '}' */
-			++ld->ld_msgid, LDAP_REQ_EXTENDED,
+			id, LDAP_REQ_EXTENDED,
 			LDAP_TAG_EXOP_REQ_OID, reqoid,
 			LDAP_TAG_EXOP_REQ_VALUE, reqdata );
 
 	} else {
 		rc = ber_printf( ber, "{it{tsN}", /* '}' */
-			++ld->ld_msgid, LDAP_REQ_EXTENDED,
+			id, LDAP_REQ_EXTENDED,
 			LDAP_TAG_EXOP_REQ_OID, reqoid );
 	}
 
@@ -98,7 +100,7 @@ ldap_extended_operation(
 	}
 
 	/* send the message */
-	*msgidp = ldap_send_initial_request( ld, LDAP_REQ_EXTENDED, NULL, ber );
+	*msgidp = ldap_send_initial_request( ld, LDAP_REQ_EXTENDED, NULL, ber, id );
 
 	return( *msgidp < 0 ? ld->ld_errno : LDAP_SUCCESS );
 }
@@ -117,18 +119,17 @@ ldap_extended_operation_s(
     int     msgid;
     LDAPMessage *res;
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_extended_operation_s\n", 0,0,0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_extended_operation_s\n", 0, 0, 0 );
+#endif
 
 	assert( ld != NULL );
 	assert( LDAP_VALID( ld ) );
 	assert( reqoid != NULL || *reqoid == '\0' );
 	assert( retoidp != NULL || retdatap != NULL );
 
-	if( retoidp == NULL || retdatap == NULL ) {
-		ld->ld_errno = LDAP_PARAM_ERROR;
-		return( ld->ld_errno );
-	}
-
     rc = ldap_extended_operation( ld, reqoid, reqdata,
 		sctrls, cctrls, &msgid );
         
@@ -140,8 +141,8 @@ ldap_extended_operation_s(
         return( ld->ld_errno );
 	}
 
-	*retoidp = NULL;
-	*retdatap = NULL;
+	if ( retoidp != NULL ) *retoidp = NULL;
+	if ( retdatap != NULL ) *retdatap = NULL;
 
 	rc = ldap_parse_extended_result( ld, res, retoidp, retdatap, 0 );
 
@@ -174,7 +175,11 @@ ldap_parse_extended_result (
 	assert( LDAP_VALID( ld ) );
 	assert( res != NULL );
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_parse_extended_result\n", 0,0,0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_parse_extended_result\n", 0, 0, 0 );
+#endif
 
 	if( ld->ld_version < LDAP_VERSION3 ) {
 		ld->ld_errno = LDAP_NOT_SUPPORTED;
@@ -278,7 +283,7 @@ ldap_parse_extended_result (
 
 /* Parse an extended partial */
 int
-ldap_parse_extended_partial (
+ldap_parse_intermediate (
 	LDAP			*ld,
 	LDAPMessage		*res,
 	char			**retoidp,
@@ -297,14 +302,18 @@ ldap_parse_extended_partial (
 	assert( LDAP_VALID( ld ) );
 	assert( res != NULL );
 
-	Debug( LDAP_DEBUG_TRACE, "ldap_parse_extended_result\n", 0, 0, 0 );
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ENTRY, "ldap_parse_intermediate\n", 0,0,0 );
+#else
+	Debug( LDAP_DEBUG_TRACE, "ldap_parse_intermediate\n", 0, 0, 0 );
+#endif
 
 	if( ld->ld_version < LDAP_VERSION3 ) {
 		ld->ld_errno = LDAP_NOT_SUPPORTED;
 		return ld->ld_errno;
 	}
 
-	if( res->lm_msgtype != LDAP_RES_EXTENDED_PARTIAL ) {
+	if( res->lm_msgtype != LDAP_RES_INTERMEDIATE ) {
 		ld->ld_errno = LDAP_PARAM_ERROR;
 		return ld->ld_errno;
 	}
@@ -386,3 +395,4 @@ free_and_return:
 
 	return LDAP_SUCCESS;
 }
+
