@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/init.c,v 1.76 2002/02/09 00:09:23 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/init.c,v 1.82 2003/05/21 22:13:00 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 #include "portable.h"
@@ -81,7 +81,6 @@ static const struct ol_attribute {
 #endif
 
 #ifdef HAVE_TLS
-  	{0, ATTR_TLS,		"TLS",			NULL,	LDAP_OPT_X_TLS},
 	{1, ATTR_TLS,		"TLS_CERT",		NULL,	LDAP_OPT_X_TLS_CERTFILE},
 	{1, ATTR_TLS,		"TLS_KEY",		NULL,	LDAP_OPT_X_TLS_KEYFILE},
   	{0, ATTR_TLS,		"TLS_CACERT",	NULL,	LDAP_OPT_X_TLS_CACERTFILE},
@@ -115,7 +114,12 @@ static void openldap_ldap_init_w_conf(
 		return;
 	}
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( CONFIG, DETAIL1, 
+		"openldap_init_w_conf: trying %s\n", file, 0, 0 );
+#else
 	Debug(LDAP_DEBUG_TRACE, "ldap_init: trying %s\n", file, 0, 0);
+#endif
 
 	fp = fopen(file, "r");
 	if(fp == NULL) {
@@ -123,7 +127,11 @@ static void openldap_ldap_init_w_conf(
 		return;
 	}
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( CONFIG, DETAIL1, "openldap_init_w_conf: using %s\n", file, 0, 0 );
+#else
 	Debug(LDAP_DEBUG_TRACE, "ldap_init: using %s\n", file, 0, 0);
+#endif
 
 	while((start = fgets(linebuf, sizeof(linebuf), fp)) != NULL) {
 		/* skip lines starting with '#' */
@@ -251,23 +259,33 @@ static void openldap_ldap_init_w_userconf(const char *file)
 	home = getenv("HOME");
 
 	if (home != NULL) {
+#ifdef NEW_LOGGING
+	LDAP_LOG ( CONFIG, ARGS, 
+		"openldap_init_w_userconf: HOME env is %s\n", home, 0, 0 );
+#else
 		Debug(LDAP_DEBUG_TRACE, "ldap_init: HOME env is %s\n",
 		      home, 0, 0);
-		path = LDAP_MALLOC(strlen(home) + strlen(file) + 3);
+#endif
+		path = LDAP_MALLOC(strlen(home) + strlen(file) + sizeof( LDAP_DIRSEP "."));
 	} else {
+#ifdef NEW_LOGGING
+	LDAP_LOG ( CONFIG, ARGS, "openldap_init_w_userconf: HOME env is NULL\n",
+		0, 0, 0 );
+#else
 		Debug(LDAP_DEBUG_TRACE, "ldap_init: HOME env is NULL\n",
 		      0, 0, 0);
+#endif
 	}
 
 	if(home != NULL && path != NULL) {
 		/* we assume UNIX path syntax is used... */
 
 		/* try ~/file */
-		sprintf(path, "%s%s%s", home, LDAP_DIRSEP, file);
+		sprintf(path, "%s" LDAP_DIRSEP "%s", home, file);
 		openldap_ldap_init_w_conf(path, 1);
 
 		/* try ~/.file */
-		sprintf(path, "%s%s.%s", home, LDAP_DIRSEP, file);
+		sprintf(path, "%s" LDAP_DIRSEP ".%s", home, file);
 		openldap_ldap_init_w_conf(path, 1);
 	}
 
@@ -464,6 +482,8 @@ void ldap_int_initialize( struct ldapoptions *gopts, int *dbglvl )
 
 	ldap_int_error_init();
 
+	ldap_int_utils_init();
+
 #ifdef HAVE_WINSOCK2
 {	WORD wVersionRequested;
 	WSADATA wsaData;
@@ -502,8 +522,6 @@ void ldap_int_initialize( struct ldapoptions *gopts, int *dbglvl )
 	|| defined(HAVE_TLS) || defined(HAVE_CYRUS_SASL)
 	ldap_int_hostname = ldap_pvt_get_fqdn( ldap_int_hostname );
 #endif
-	ldap_int_utils_init();
-
 	if ( ldap_int_tblsize == 0 )
 		ldap_int_ip_init();
 
@@ -534,26 +552,50 @@ void ldap_int_initialize( struct ldapoptions *gopts, int *dbglvl )
 		char *altfile = getenv(LDAP_ENV_PREFIX "CONF");
 
 		if( altfile != NULL ) {
+#ifdef NEW_LOGGING
+			LDAP_LOG ( CONFIG, DETAIL1, 
+				"openldap_init_w_userconf: %sCONF env is %s\n",
+				LDAP_ENV_PREFIX, altfile, 0 );
+#else
 			Debug(LDAP_DEBUG_TRACE, "ldap_init: %s env is %s\n",
 			      LDAP_ENV_PREFIX "CONF", altfile, 0);
+#endif
 			openldap_ldap_init_w_sysconf( altfile );
 		}
 		else
+#ifdef NEW_LOGGING
+			LDAP_LOG ( CONFIG, DETAIL1, 
+				"openldap_init_w_userconf: %sCONF env is NULL\n",
+				LDAP_ENV_PREFIX, 0, 0 );
+#else
 			Debug(LDAP_DEBUG_TRACE, "ldap_init: %s env is NULL\n",
 			      LDAP_ENV_PREFIX "CONF", 0, 0);
+#endif
 	}
 
 	{
 		char *altfile = getenv(LDAP_ENV_PREFIX "RC");
 
 		if( altfile != NULL ) {
+#ifdef NEW_LOGGING
+			LDAP_LOG ( CONFIG, DETAIL1, 
+				"openldap_init_w_userconf: %sRC env is %s\n",
+				LDAP_ENV_PREFIX, altfile, 0 );
+#else
 			Debug(LDAP_DEBUG_TRACE, "ldap_init: %s env is %s\n",
 			      LDAP_ENV_PREFIX "RC", altfile, 0);
+#endif
 			openldap_ldap_init_w_userconf( altfile );
 		}
 		else
+#ifdef NEW_LOGGING
+			LDAP_LOG ( CONFIG, DETAIL1, 
+				"openldap_init_w_userconf: %sRC env is NULL\n",
+				LDAP_ENV_PREFIX, 0, 0 );
+#else
 			Debug(LDAP_DEBUG_TRACE, "ldap_init: %s env is NULL\n",
 			      LDAP_ENV_PREFIX "RC", 0, 0);
+#endif
 	}
 
 	openldap_ldap_init_w_env(gopts, NULL);
