@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/open.c,v 1.85.2.1 2002/02/23 22:10:19 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldap/open.c,v 1.102 2003/04/30 12:13:58 hyc Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*  Portions
@@ -22,7 +22,10 @@
 #include <ac/string.h>
 #include <ac/time.h>
 
+#include <ac/unistd.h>
+
 #include "ldap-int.h"
+#include "ldap_log.h"
 
 int ldap_open_defconn( LDAP *ld )
 {
@@ -54,8 +57,12 @@ ldap_open( LDAP_CONST char *host, int port )
 	int rc;
 	LDAP		*ld;
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( CONNECTION, ARGS, "ldap_open(%s, %d)\n", host, port, 0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_open(%s, %d)\n",
 		host, port, 0 );
+#endif
 
 	ld = ldap_init( host, port );
 	if ( ld == NULL ) {
@@ -69,8 +76,13 @@ ldap_open( LDAP_CONST char *host, int port )
 		ld = NULL;
 	}
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( CONNECTION, RESULTS, "ldap_open: %s\n",
+		ld == NULL ? "succeeded" : "failed", 0, 0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_open: %s\n",
 		ld == NULL ? "succeeded" : "failed", 0, 0 );
+#endif
 
 	return ld;
 }
@@ -96,7 +108,11 @@ ldap_create( LDAP **ldp )
 			return LDAP_LOCAL_ERROR;
 	}
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( CONNECTION, ENTRY, "ldap_create\n", 0, 0, 0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_create\n", 0, 0, 0 );
+#endif
 
 	if ( (ld = (LDAP *) LDAP_CALLOC( 1, sizeof(LDAP) )) == NULL ) {
 		return( LDAP_NO_MEMORY );
@@ -144,6 +160,10 @@ ldap_create( LDAP **ldp )
 		return LDAP_NO_MEMORY;
 	}
 
+#ifdef LDAP_R_COMPILE
+	ldap_pvt_thread_mutex_init( &ld->ld_req_mutex );
+	ldap_pvt_thread_mutex_init( &ld->ld_res_mutex );
+#endif
 	*ldp = ld;
 	return LDAP_SUCCESS;
 }
@@ -217,24 +237,21 @@ ldap_int_open_connection(
 	int async )
 {
 	int rc = -1;
-#ifdef HAVE_CYRUS_SASL
-	char *sasl_host = NULL;
-	int sasl_ssf = 0;
-#endif
 	char *host;
 	int port, proto;
-	long addr;
 
+#ifdef NEW_LOGGING
+	LDAP_LOG ( CONNECTION, ENTRY, "ldap_int_open_connection\n", 0, 0, 0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, "ldap_int_open_connection\n", 0, 0, 0 );
+#endif
 
 	switch ( proto = ldap_pvt_url_scheme2proto( srv->lud_scheme ) ) {
 		case LDAP_PROTO_TCP:
 			port = srv->lud_port;
 
-			addr = 0;
 			if ( srv->lud_host == NULL || *srv->lud_host == 0 ) {
 				host = NULL;
-				addr = htonl( INADDR_LOOPBACK );
 			} else {
 				host = srv->lud_host;
 			}
@@ -248,7 +265,7 @@ ldap_int_open_connection(
 			}
 
 			rc = ldap_connect_to_host( ld, conn->lconn_sb,
-				proto, host, addr, port, async );
+				proto, host, port, async );
 
 			if ( rc == -1 ) return rc;
 
@@ -259,19 +276,14 @@ ldap_int_open_connection(
 			ber_sockbuf_add_io( conn->lconn_sb, &ber_sockbuf_io_tcp,
 				LBER_SBIOD_LEVEL_PROVIDER, NULL );
 
-#ifdef HAVE_CYRUS_SASL
-			sasl_host = ldap_host_connected_to( conn->lconn_sb );
-#endif
 			break;
-#ifdef LDAP_CONNECTIONLESS
 
+#ifdef LDAP_CONNECTIONLESS
 		case LDAP_PROTO_UDP:
 			port = srv->lud_port;
 
-			addr = 0;
 			if ( srv->lud_host == NULL || *srv->lud_host == 0 ) {
 				host = NULL;
-				addr = htonl( INADDR_LOOPBACK );
 			} else {
 				host = srv->lud_host;
 			}
@@ -280,7 +292,7 @@ ldap_int_open_connection(
 
 			LDAP_IS_UDP(ld) = 1;
 			rc = ldap_connect_to_host( ld, conn->lconn_sb,
-				proto, host, addr, port, async );
+				proto, host, port, async );
 
 			if ( rc == -1 ) return rc;
 #ifdef LDAP_DEBUG
@@ -289,6 +301,10 @@ ldap_int_open_connection(
 #endif
 			ber_sockbuf_add_io( conn->lconn_sb, &ber_sockbuf_io_udp,
 				LBER_SBIOD_LEVEL_PROVIDER, NULL );
+
+			ber_sockbuf_add_io( conn->lconn_sb, &ber_sockbuf_io_readahead,
+				LBER_SBIOD_LEVEL_PROVIDER, NULL );
+
 			break;
 #endif
 		case LDAP_PROTO_IPC:
@@ -304,10 +320,6 @@ ldap_int_open_connection(
 			ber_sockbuf_add_io( conn->lconn_sb, &ber_sockbuf_io_fd,
 				LBER_SBIOD_LEVEL_PROVIDER, NULL );
 
-#ifdef HAVE_CYRUS_SASL
-			sasl_host = ldap_host_connected_to( conn->lconn_sb );
-			sasl_ssf = LDAP_PVT_SASL_LOCAL_SSF;
-#endif
 			break;
 #endif /* LDAP_PF_LOCAL */
 		default:
@@ -315,26 +327,13 @@ ldap_int_open_connection(
 			break;
 	}
 
-	ber_sockbuf_add_io( conn->lconn_sb, &ber_sockbuf_io_readahead,
-		LBER_SBIOD_LEVEL_PROVIDER, NULL );
-
 #ifdef LDAP_DEBUG
 	ber_sockbuf_add_io( conn->lconn_sb, &ber_sockbuf_io_debug,
 		INT_MAX, (void *)"ldap_" );
 #endif
 
 #ifdef LDAP_CONNECTIONLESS
-	if( proto == LDAP_PROTO_UDP )
-		return 0;
-#endif
-
-#ifdef HAVE_CYRUS_SASL
-	/* establish Cyrus SASL context prior to starting TLS so
-		that SASL EXTERNAL might be used */
-	if( sasl_host != NULL ) {
-		ldap_int_sasl_open( ld, conn, sasl_host, sasl_ssf );
-		LDAP_FREE( sasl_host );
-	}
+	if( proto == LDAP_PROTO_UDP ) return 0;
 #endif
 
 #ifdef HAVE_TLS
@@ -356,7 +355,8 @@ ldap_int_open_connection(
 #ifdef LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND
 	if ( conn->lconn_krbinstance == NULL ) {
 		char *c;
-		conn->lconn_krbinstance = ldap_host_connected_to( conn->lconn_sb );
+		conn->lconn_krbinstance = ldap_host_connected_to(
+			conn->lconn_sb, host );
 
 		if( conn->lconn_krbinstance != NULL && 
 		    ( c = strchr( conn->lconn_krbinstance, '.' )) != NULL ) {
@@ -392,6 +392,7 @@ int ldap_open_internal_connection( LDAP **ldp, ber_socket_t *fdp )
 	lr->lr_msgid = 0;
 	lr->lr_status = LDAP_REQST_INPROGRESS;
 	lr->lr_res_errno = LDAP_SUCCESS;
+	/* no mutex lock needed, we just created this ld here */
 	(*ldp)->ld_requests = lr;
 
 	/* Attach the passed socket as the *LDAP's connection */
