@@ -1,5 +1,6 @@
+/* $OpenLDAP: pkg/ldap/libraries/libldap/test.c,v 1.44 2003/02/09 06:42:25 kurt Exp $ */
 /*
- * Copyright 1998-1999 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -34,23 +35,23 @@
 
 /* local functions */
 #ifndef HAVE_GETLINE
-static char *getline LDAP_P(( char *line, int len, FILE *fp, char *prompt ));
+static char *getline LDAP_P(( char *line, int len, FILE *fp, const char *prompt ));
 #endif
-static char **get_list LDAP_P(( char *prompt ));
-static int file_read LDAP_P(( char *path, struct berval *bv ));
-static LDAPMod **get_modlist LDAP_P(( char *prompt1, char *prompt2, char *prompt3 ));
+static char **get_list LDAP_P(( const char *prompt ));
+static int file_read LDAP_P(( const char *path, struct berval *bv ));
+static LDAPMod **get_modlist LDAP_P(( const char *prompt1,
+	const char *prompt2, const char *prompt3 ));
 static void handle_result LDAP_P(( LDAP *ld, LDAPMessage *lm ));
-static void print_ldap_result LDAP_P(( LDAP *ld, LDAPMessage *lm, char *s ));
+static void print_ldap_result LDAP_P(( LDAP *ld, LDAPMessage *lm,
+	const char *s ));
 static void print_search_entry LDAP_P(( LDAP *ld, LDAPMessage *res ));
 static void free_list LDAP_P(( char **list ));
 
-#define NOCACHEERRMSG	"don't compile with -DLDAP_NOCACHE if you desire local caching"
-
 static char *dnsuffix;
 
 #ifndef HAVE_GETLINE
 static char *
-getline( char *line, int len, FILE *fp, char *prompt )
+getline( char *line, int len, FILE *fp, const char *prompt )
 {
 	printf(prompt);
 
@@ -64,7 +65,7 @@ getline( char *line, int len, FILE *fp, char *prompt )
 #endif
 
 static char **
-get_list( char *prompt )
+get_list( const char *prompt )
 {
 	static char	buf[256];
 	int		num;
@@ -110,7 +111,7 @@ free_list( char **list )
 
 
 static int
-file_read( char *path, struct berval *bv )
+file_read( const char *path, struct berval *bv )
 {
 	FILE		*fp;
 	ber_slen_t	rlen;
@@ -156,7 +157,10 @@ file_read( char *path, struct berval *bv )
 
 
 static LDAPMod **
-get_modlist( char *prompt1, char *prompt2, char *prompt3 )
+get_modlist(
+	const char *prompt1,
+	const char *prompt2,
+	const char *prompt3 )
 {
 	static char	buf[256];
 	int		num;
@@ -228,38 +232,40 @@ get_modlist( char *prompt1, char *prompt2, char *prompt3 )
 
 
 static int
-bind_prompt( LDAP *ld, char **dnp, char **passwdp, int *authmethodp,
-	int freeit )
+bind_prompt( LDAP *ld,
+	LDAP_CONST char *url,
+	ber_tag_t request, ber_int_t msgid,
+	void *params )
 {
 	static char	dn[256], passwd[256];
+	int	authmethod;
 
-	if ( !freeit ) {
-#ifdef HAVE_KERBEROS
+	printf("rebind for request=%ld msgid=%ld url=%s\n",
+		request, (long) msgid, url );
+
+#ifdef LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND
 		getline( dn, sizeof(dn), stdin,
 		    "re-bind method (0->simple, 1->krbv41, 2->krbv42, 3->krbv41&2)? " );
-		if (( *authmethodp = atoi( dn )) == 3 ) {
-			*authmethodp = LDAP_AUTH_KRBV4;
+	if (( authmethod = atoi( dn )) == 3 ) {
+		authmethod = LDAP_AUTH_KRBV4;
 		} else {
-			*authmethodp |= 0x80;
+		authmethod |= 0x80;
 		}
-#else /* HAVE_KERBEROS */
-		*authmethodp = LDAP_AUTH_SIMPLE;
-#endif /* HAVE_KERBEROS */
+#else /* LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND */
+	authmethod = LDAP_AUTH_SIMPLE;
+#endif /* LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND */
 
 		getline( dn, sizeof(dn), stdin, "re-bind dn? " );
 		strcat( dn, dnsuffix );
-		*dnp = dn;
 
-		if ( *authmethodp == LDAP_AUTH_SIMPLE && dn[0] != '\0' ) {
+	if ( authmethod == LDAP_AUTH_SIMPLE && dn[0] != '\0' ) {
 			getline( passwd, sizeof(passwd), stdin,
 			    "re-bind password? " );
 		} else {
 			passwd[0] = '\0';
 		}
-		*passwdp = passwd;
-	}
 
-	return( LDAP_SUCCESS );
+	return ldap_bind_s( ld, dn, passwd, authmethod);
 }
 
 
@@ -267,7 +273,7 @@ int
 main( int argc, char **argv )
 {
 	LDAP		*ld = NULL;
-	int		i, c, port, cldapflg, errflg, method, id, msgtype;
+	int		i, c, port, errflg, method, id, msgtype;
 	char		line[256], command1, command2, command3;
 	char		passwd[64], dn[256], rdn[64], attr[64], value[256];
 	char		filter[256], *host, **types;
@@ -284,24 +290,16 @@ main( int argc, char **argv )
 	host = NULL;
 	port = LDAP_PORT;
 	dnsuffix = "";
-	cldapflg = errflg = 0;
+	errflg = 0;
 
-	while (( c = getopt( argc, argv, "uh:d:s:p:t:T:" )) != -1 ) {
+	while (( c = getopt( argc, argv, "h:d:s:p:t:T:" )) != -1 ) {
 		switch( c ) {
-		case 'u':
-#ifdef LDAP_CONNECTIONLESS
-			cldapflg++;
-#else /* LDAP_CONNECTIONLESS */
-			printf( "Compile with -DLDAP_CONNECTIONLESS for UDP support\n" );
-#endif /* LDAP_CONNECTIONLESS */
-			break;
-
 		case 'd':
 #ifdef LDAP_DEBUG
 			ldap_debug = atoi( optarg );
 #ifdef LBER_DEBUG
 			if ( ldap_debug & LDAP_DEBUG_PACKETS ) {
-				lber_debug = ldap_debug;
+				ber_set_option( NULL, LBER_OPT_DEBUG_LEVEL, &ldap_debug );
 			}
 #endif
 #else
@@ -323,12 +321,12 @@ main( int argc, char **argv )
 
 		case 't':	/* copy ber's to given file */
 			copyfname = strdup( optarg );
-			copyoptions = LBER_TO_FILE;
+/*			copyoptions = LBER_TO_FILE; */
 			break;
 
 		case 'T':	/* only output ber's to given file */
 			copyfname = strdup( optarg );
-			copyoptions = (LBER_TO_FILE | LBER_TO_FILE_ONLY);
+/*			copyoptions = (LBER_TO_FILE | LBER_TO_FILE_ONLY); */
 			break;
 
 		default:
@@ -343,33 +341,26 @@ main( int argc, char **argv )
 
 	if ( errflg || optind < argc - 1 ) {
 		fprintf( stderr, usage, argv[ 0 ] );
-		exit( 1 );
+		exit( EXIT_FAILURE );
 	}
 	
-	printf( "%s( %s, %d )\n",
-		cldapflg ? "cldap_open" : "ldap_init",
+	printf( "ldap_init( %s, %d )\n",
 		host == NULL ? "(null)" : host, port );
 
-	if ( cldapflg ) {
-#ifdef LDAP_CONNECTIONLESS
-		ld = cldap_open( host, port );
-#endif /* LDAP_CONNECTIONLESS */
-	} else {
-		ld = ldap_init( host, port );
-	}
+	ld = ldap_init( host, port );
 
 	if ( ld == NULL ) {
-		perror( cldapflg ? "cldap_open" : "ldap_init" );
-		exit(1);
+		perror( "ldap_init" );
+		exit( EXIT_FAILURE );
 	}
 
 	if ( copyfname != NULL ) {
-		if ( (ld->ld_sb.sb_fd = open( copyfname, O_WRONLY | O_CREAT,
+		if ( ( ld->ld_sb->sb_fd = open( copyfname, O_WRONLY|O_CREAT|O_EXCL,
 		    0600 ))  == -1 ) {
 			perror( copyfname );
-			exit ( 1 );
+			exit ( EXIT_FAILURE );
 		}
-		ld->ld_sb.sb_options = copyoptions;
+		ld->ld_sb->sb_options = copyoptions;
 	}
 
 	bound = 0;
@@ -412,13 +403,13 @@ main( int argc, char **argv )
 			break;
 
 		case 'b':	/* asynch bind */
-#ifdef HAVE_KERBEROS
+#ifdef LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND
 			getline( line, sizeof(line), stdin,
 			    "method (0->simple, 1->krbv41, 2->krbv42)? " );
 			method = atoi( line ) | 0x80;
-#else /* HAVE_KERBEROS */
+#else /* LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND */
 			method = LDAP_AUTH_SIMPLE;
-#endif /* HAVE_KERBEROS */
+#endif /* LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND */
 			getline( dn, sizeof(dn), stdin, "dn? " );
 			strcat( dn, dnsuffix );
 
@@ -438,7 +429,7 @@ main( int argc, char **argv )
 			break;
 
 		case 'B':	/* synch bind */
-#ifdef HAVE_KERBEROS
+#ifdef LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND
 			getline( line, sizeof(line), stdin,
 			    "method 0->simple 1->krbv41 2->krbv42 3->krb? " );
 			method = atoi( line );
@@ -446,9 +437,9 @@ main( int argc, char **argv )
 				method = LDAP_AUTH_KRBV4;
 			else
 				method = method | 0x80;
-#else /* HAVE_KERBEROS */
+#else /* LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND */
 			method = LDAP_AUTH_SIMPLE;
-#endif /* HAVE_KERBEROS */
+#endif /* LDAP_API_FEATURE_X_OPENLDAP_V2_KBIND */
 			getline( dn, sizeof(dn), stdin, "dn? " );
 			strcat( dn, dnsuffix );
 
@@ -486,7 +477,7 @@ main( int argc, char **argv )
 			ldap_debug = atoi( line );
 #ifdef LBER_DEBUG
 			if ( ldap_debug & LDAP_DEBUG_PACKETS ) {
-				lber_debug = ldap_debug;
+				ber_set_option( NULL, LBER_OPT_DEBUG_LEVEL, &ldap_debug );
 			}
 #endif
 #else
@@ -541,15 +532,8 @@ main( int argc, char **argv )
 			break;
 
 		case 'q':	/* quit */
-#ifdef LDAP_CONNECTIONLESS
-			if ( cldapflg )
-				cldap_close( ld );
-#endif /* LDAP_CONNECTIONLESS */
-
-			if ( !cldapflg ) {
-				ldap_unbind( ld );
-			}
-			exit( 0 );
+			ldap_unbind( ld );
+			exit( EXIT_SUCCESS );
 			break;
 
 		case 'r':	/* result or remove */
@@ -607,28 +591,12 @@ main( int argc, char **argv )
 			    "attrsonly (0=attrs&values, 1=attrs only)? " );
 			attrsonly = atoi( line );
 
-			if ( cldapflg ) {
-#ifdef LDAP_CONNECTIONLESS
-			    getline( line, sizeof(line), stdin,
-				"Requestor DN (for logging)? " );
-			    if ( cldap_search_s( ld, dn, scope, filter, types,
-				    attrsonly, &res, line ) != 0 ) {
-				ldap_perror( ld, "cldap_search_s" );
-			    } else {
-				printf( "\nresult: msgid %d\n",
-				    res->lm_msgid );
-				handle_result( ld, res );
-				res = NULL;
-			    }
-#endif /* LDAP_CONNECTIONLESS */
-			} else {
 			    if (( id = ldap_search( ld, dn, scope, filter,
 				    types, attrsonly  )) == -1 ) {
 				ldap_perror( ld, "ldap_search" );
 			    } else {
 				printf( "Search initiated with id %d\n", id );
 			    }
-			}
 			free_list( types );
 			break;
 
@@ -637,50 +605,6 @@ main( int argc, char **argv )
 			timeout.tv_sec = atoi( line );
 			break;
 
-		case 'U':	/* set ufn search prefix */
-			getline( line, sizeof(line), stdin, "ufn prefix? " );
-			ldap_ufn_setprefix( ld, line );
-			break;
-
-		case 'u':	/* user friendly search w/optional timeout */
-			getline( dn, sizeof(dn), stdin, "ufn? " );
-			strcat( dn, dnsuffix );
-			types = get_list( "attrs to return? " );
-			getline( line, sizeof(line), stdin,
-			    "attrsonly (0=attrs&values, 1=attrs only)? " );
-			attrsonly = atoi( line );
-
-			if ( command2 == 't' ) {
-				id = ldap_ufn_search_c( ld, dn, types,
-				    attrsonly, &res, ldap_ufn_timeout,
-				    &timeout );
-			} else {
-				id = ldap_ufn_search_s( ld, dn, types,
-				    attrsonly, &res );
-			}
-			if ( res == NULL )
-				ldap_perror( ld, "ldap_ufn_search" );
-			else {
-				printf( "\nresult: err %d\n", id );
-				handle_result( ld, res );
-				res = NULL;
-			}
-			free_list( types );
-			break;
-
-		case 'l':	/* URL search */
-			getline( line, sizeof(line), stdin,
-			    "attrsonly (0=attrs&values, 1=attrs only)? " );
-			attrsonly = atoi( line );
-			getline( line, sizeof(line), stdin, "LDAP URL? " );
-			if (( id = ldap_url_search( ld, line, attrsonly  ))
-				== -1 ) {
-			    ldap_perror( ld, "ldap_url_search" );
-			} else {
-			    printf( "URL search initiated with id %d\n", id );
-			}
-			break;
-
 		case 'p':	/* parse LDAP URL */
 			getline( line, sizeof(line), stdin, "LDAP URL? " );
 			if (( i = ldap_url_parse( line, &ludp )) != 0 ) {
@@ -720,39 +644,6 @@ main( int argc, char **argv )
 			strcpy( dnsuffix, line );
 			break;
 
-		case 'e':	/* enable cache */
-#ifdef LDAP_NOCACHE
-			printf( NOCACHEERRMSG );
-#else /* LDAP_NOCACHE */
-			getline( line, sizeof(line), stdin, "Cache timeout (secs)? " );
-			i = atoi( line );
-			getline( line, sizeof(line), stdin, "Maximum memory to use (bytes)? " );
-			if ( ldap_enable_cache( ld, i, atoi( line )) == 0 ) {
-				printf( "local cache is on\n" ); 
-			} else {
-				printf( "ldap_enable_cache failed\n" ); 
-			}
-#endif /* LDAP_NOCACHE */
-			break;
-
-		case 'x':	/* uncache entry */
-#ifdef LDAP_NOCACHE
-			printf( NOCACHEERRMSG );
-#else /* LDAP_NOCACHE */
-			getline( line, sizeof(line), stdin, "DN? " );
-			ldap_uncache_entry( ld, line );
-#endif /* LDAP_NOCACHE */
-			break;
-
-		case 'X':	/* uncache request */
-#ifdef LDAP_NOCACHE
-			printf( NOCACHEERRMSG );
-#else /* LDAP_NOCACHE */
-			getline( line, sizeof(line), stdin, "request msgid? " );
-			ldap_uncache_request( ld, atoi( line ));
-#endif /* LDAP_NOCACHE */
-			break;
-
 		case 'o':	/* set ldap options */
 			getline( line, sizeof(line), stdin, "alias deref (0=never, 1=searching, 2=finding, 3=always)?" );
 			ld->ld_deref = atoi( line );
@@ -763,33 +654,6 @@ main( int argc, char **argv )
 
 			LDAP_BOOL_ZERO(&ld->ld_options);
 
-#ifdef STR_TRANSLATION
-			getline( line, sizeof(line), stdin,
-				"Automatic translation of T.61 strings (0=no, 1=yes)?" );
-			if ( atoi( line ) == 0 ) {
-				ld->ld_lberoptions &= ~LBER_TRANSLATE_STRINGS;
-			} else {
-				ld->ld_lberoptions |= LBER_TRANSLATE_STRINGS;
-#ifdef LDAP_CHARSET_8859
-				getline( line, sizeof(line), stdin,
-					"Translate to/from ISO-8859 (0=no, 1=yes?" );
-				if ( atoi( line ) != 0 ) {
-					ldap_set_string_translators( ld,
-					    ldap_8859_to_t61,
-					    ldap_t61_to_8859 );
-				}
-#endif /* LDAP_CHARSET_8859 */
-			}
-#endif /* STR_TRANSLATION */
-
-#ifdef LDAP_API_FEATURE_X_OPENLDAP_V2_DNS
-			getline( line, sizeof(line), stdin,
-				"Use DN & DNS to determine where to send requests (0=no, 1=yes)?" );
-			if ( atoi( line ) != 0 ) {
-				LDAP_BOOL_SET(&ld->ld_options, LDAP_BOOL_DNS);
-			}
-#endif /* LDAP_API_FEATURE_X_OPENLDAP_V2_DNS */
-
 			getline( line, sizeof(line), stdin,
 				"Recognize and chase referrals (0=no, 1=yes)?" );
 			if ( atoi( line ) != 0 ) {
@@ -797,45 +661,21 @@ main( int argc, char **argv )
 				getline( line, sizeof(line), stdin,
 					"Prompt for bind credentials when chasing referrals (0=no, 1=yes)?" );
 				if ( atoi( line ) != 0 ) {
-					ldap_set_rebind_proc( ld, bind_prompt );
+					ldap_set_rebind_proc( ld, bind_prompt, NULL );
 				}
 			}
 			break;
 
-		case 'O':	/* set cache options */
-#ifdef LDAP_NOCACHE
-			printf( NOCACHEERRMSG );
-#else /* LDAP_NOCACHE */
-			getline( line, sizeof(line), stdin, "cache errors (0=smart, 1=never, 2=always)?" );
-			switch( atoi( line )) {
-			case 0:
-				ldap_set_cache_options( ld, 0 );
-				break;
-			case 1:
-				ldap_set_cache_options( ld,
-					LDAP_CACHE_OPT_CACHENOERRS );
-				break;
-			case 2:
-				ldap_set_cache_options( ld,
-					LDAP_CACHE_OPT_CACHEALLERRS );
-				break;
-			default:
-				printf( "not a valid cache option\n" );
-			}
-#endif /* LDAP_NOCACHE */
-			break;
-
 		case '?':	/* help */
-    printf( "Commands: [ad]d         [ab]andon         [b]ind\n" );
-    printf( "          [B]ind async  [c]ompare         [l]URL search\n" );
-    printf( "          [modi]fy      [modr]dn          [rem]ove\n" );
-    printf( "          [res]ult      [s]earch          [q]uit/unbind\n\n" );
-    printf( "          [u]fn search  [ut]fn search with timeout\n" );
-    printf( "          [d]ebug       [e]nable cache    set ms[g]id\n" );
-    printf( "          d[n]suffix    [t]imeout         [v]ersion\n" );
-    printf( "          [U]fn prefix  [x]uncache entry  [X]uncache request\n" );
-    printf( "          [?]help       [o]ptions         [O]cache options\n" );
-    printf( "          [E]xplode dn  [p]arse LDAP URL\n" );
+			printf(
+"Commands: [ad]d         [ab]andon         [b]ind\n"
+"          [B]ind async  [c]ompare\n"
+"          [modi]fy      [modr]dn          [rem]ove\n"
+"          [res]ult      [s]earch          [q]uit/unbind\n\n"
+"          [d]ebug       set ms[g]id\n"
+"          d[n]suffix    [t]imeout         [v]ersion\n"
+"          [?]help       [o]ptions"
+"          [E]xplode dn  [p]arse LDAP URL\n" );
 			break;
 
 		default:
@@ -889,13 +729,14 @@ handle_result( LDAP *ld, LDAPMessage *lm )
 		break;
 
 	default:
-		printf( "Unknown result type 0x%x\n", lm->lm_msgtype );
+		printf( "Unknown result type 0x%lx\n",
+		        (unsigned long) lm->lm_msgtype );
 		print_ldap_result( ld, lm, "unknown" );
 	}
 }
 
 static void
-print_ldap_result( LDAP *ld, LDAPMessage *lm, char *s )
+print_ldap_result( LDAP *ld, LDAPMessage *lm, const char *s )
 {
 	ldap_result2error( ld, lm, 1 );
 	ldap_perror( ld, s );
@@ -954,7 +795,7 @@ print_search_entry( LDAP *ld, LDAPMessage *res )
 					if ( nonascii ) {
 						printf( "\t\t\tlength (%ld) (not ascii)\n", vals[i]->bv_len );
 #ifdef BPRINT_NONASCII
-						lber_bprint( vals[i]->bv_val,
+						ber_bprint( vals[i]->bv_val,
 						    vals[i]->bv_len );
 #endif /* BPRINT_NONASCII */
 						continue;
