@@ -1,5 +1,6 @@
+/* $OpenLDAP: pkg/ldap/libraries/libldap/util-int.c,v 1.45 2003/05/25 18:50:26 hallvard Exp $ */
 /*
- * Copyright 1998-1999 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -28,40 +29,43 @@
 #include <ac/socket.h>
 #include <ac/string.h>
 #include <ac/time.h>
+#include <ac/unistd.h>
 
 #include "ldap-int.h"
 
-#if defined( LDAP_R_COMPILE )
-# include <ldap_pvt_thread.h>
-#else
-# undef HAVE_REENTRANT_FUNCTIONS
+#ifndef h_errno
+/* newer systems declare this in <netdb.h> for you, older ones don't.
+ * harmless to declare it again (unless defined by a macro).
+ */
+extern int h_errno;
 #endif
 
+#ifndef LDAP_R_COMPILE
+# undef HAVE_REENTRANT_FUNCTIONS
+# undef HAVE_CTIME_R
+# undef HAVE_GETHOSTBYNAME_R
+# undef HAVE_GETHOSTBYADDR_R
+
+#else
+# include <ldap_pvt_thread.h>
+  ldap_pvt_thread_mutex_t ldap_int_resolv_mutex;
+
 #if (defined( HAVE_CTIME_R ) || defined( HAVE_REENTRANT_FUNCTIONS)) \
 	&& defined( CTIME_R_NARGS )
-#	define USE_CTIME_R
-#endif
-
-#if defined(HAVE_GETHOSTBYNAME_R) && \
-    (GETHOSTBYNAME_R_NARGS > 6 || GETHOSTBYNAME_R_NARGS < 5)
-	/* Don't know how to handle this version, pretend it's not there */
-#	undef HAVE_GETHOSTBYNAME_R
-#endif
-#if defined(HAVE_GETHOSTBYADDR_R) && \
-    (GETHOSTBYADDR_R_NARGS > 8 || GETHOSTBYADDR_R_NARGS < 7)
-	/* Don't know how to handle this version, pretend it's not there */
-#	undef HAVE_GETHOSTBYADDR_R
-#endif
-
-#ifdef LDAP_R_COMPILE
-# ifndef USE_CTIME_R
+# define USE_CTIME_R
+# else
 	static ldap_pvt_thread_mutex_t ldap_int_ctime_mutex;
 # endif
-# ifndef HAVE_GETHOSTBYNAME_R
-	static ldap_pvt_thread_mutex_t ldap_int_gethostbyname_mutex;
+
+# if defined(HAVE_GETHOSTBYNAME_R) && \
+	(GETHOSTBYNAME_R_NARGS < 5) || (6 < GETHOSTBYNAME_R_NARGS)
+	/* Don't know how to handle this version, pretend it's not there */
+#	undef HAVE_GETHOSTBYNAME_R
 # endif
-# ifndef HAVE_GETHOSTBYADDR_R
-	static ldap_pvt_thread_mutex_t ldap_int_gethostbyaddr_mutex;
+# if defined(HAVE_GETHOSTBYADDR_R) && \
+	(GETHOSTBYADDR_R_NARGS < 7) || (8 < GETHOSTBYADDR_R_NARGS)
+	/* Don't know how to handle this version, pretend it's not there */
+#	undef HAVE_GETHOSTBYADDR_R
 # endif
 #endif /* LDAP_R_COMPILE */
 
@@ -79,22 +83,30 @@ char *ldap_pvt_ctime( const time_t *tp, char *buf )
 # endif	  
 
 #else
+
 # ifdef LDAP_R_COMPILE
 	ldap_pvt_thread_mutex_lock( &ldap_int_ctime_mutex );
 # endif
-	memcpy( buf, ctime(tp), 26 );
+
+	AC_MEMCPY( buf, ctime(tp), 26 );
+
 # ifdef LDAP_R_COMPILE
 	ldap_pvt_thread_mutex_unlock( &ldap_int_ctime_mutex );
 # endif
+
 	return buf;
 #endif	
 }
 
-#define BUFSTART 1024
-#define BUFMAX (32*1024)
+#define BUFSTART (1024-32)
+#define BUFMAX (32*1024-32)
 
 static char *safe_realloc( char **buf, int len );
-static int copy_hostent( struct hostent *res, char **buf, struct hostent * src );
+
+#if !defined(HAVE_GETHOSTBYNAME_R) && defined(LDAP_R_COMPILE)
+static int copy_hostent( struct hostent *res,
+	char **buf, struct hostent * src );
+#endif
 
 int ldap_pvt_gethostbyname_a(
 	const char *name, 
@@ -114,9 +126,8 @@ int ldap_pvt_gethostbyname_a(
 			return r;
 
 #if (GETHOSTBYNAME_R_NARGS < 6)
-		r = ((*result=gethostbyname_r( name, resbuf, *buf,
-					       buflen, herrno_ptr ))== NULL) ?
-		    -1 : 0;
+		*result=gethostbyname_r( name, resbuf, *buf, buflen, herrno_ptr );
+		r = (*result == NULL) ?  -1 : 0;
 #else
 		r = gethostbyname_r( name, resbuf, *buf,
 			buflen, result, herrno_ptr );
@@ -141,8 +152,9 @@ int ldap_pvt_gethostbyname_a(
 # define NEED_COPY_HOSTENT   
 	struct hostent *he;
 	int	retval;
+	*buf = NULL;
 	
-	ldap_pvt_thread_mutex_lock( &ldap_int_gethostbyname_mutex );
+	ldap_pvt_thread_mutex_lock( &ldap_int_resolv_mutex );
 	
 	he = gethostbyname( name );
 	
@@ -157,10 +169,11 @@ int ldap_pvt_gethostbyname_a(
 		retval = 0;
 	}
 	
-	ldap_pvt_thread_mutex_unlock( &ldap_int_gethostbyname_mutex );
+	ldap_pvt_thread_mutex_unlock( &ldap_int_resolv_mutex );
 	
 	return retval;
 #else	
+	*buf = NULL;
 	*result = gethostbyname( name );
 
 	if (*result!=NULL) {
@@ -172,7 +185,124 @@ int ldap_pvt_gethostbyname_a(
 	return -1;
 #endif	
 }
-	 
+
+#ifndef GETNAMEINFO
+static const char *
+hp_strerror( int err )
+{
+	switch (err) {
+	case HOST_NOT_FOUND:	return _("Host not found (authoritative)");
+	case TRY_AGAIN:			return _("Host not found (server fail?)");
+	case NO_RECOVERY:		return _("Non-recoverable failure");
+	case NO_DATA:			return _("No data of requested type");
+#ifdef NETDB_INTERNAL
+	case NETDB_INTERNAL:	return STRERROR( errno );
+#endif
+	}
+	return _("Unknown resolver error");
+}
+#endif
+
+int ldap_pvt_get_hname(
+	const struct sockaddr *sa,
+	int len,
+	char *name,
+	int namelen,
+	char **err )
+{
+	int rc;
+#if defined( HAVE_GETNAMEINFO )
+
+#if defined( LDAP_R_COMPILE )
+	ldap_pvt_thread_mutex_lock( &ldap_int_resolv_mutex );
+#endif
+	rc = getnameinfo( sa, len, name, namelen, NULL, 0, 0 );
+#if defined( LDAP_R_COMPILE )
+	ldap_pvt_thread_mutex_unlock( &ldap_int_resolv_mutex );
+#endif
+	if ( rc ) *err = (char *)AC_GAI_STRERROR( rc );
+	return rc;
+
+#else /* !HAVE_GETNAMEINFO */
+	char *addr;
+	int alen;
+	struct hostent *hp = NULL;
+#ifdef HAVE_GETHOSTBYADDR_R
+	struct hostent hb;
+	int buflen=BUFSTART, h_errno;
+	char *buf=NULL;
+#endif
+
+#ifdef LDAP_PF_INET6
+	if (sa->sa_family == AF_INET6) {
+		struct sockaddr_in6 *sin = (struct sockaddr_in6 *)sa;
+		addr = (char *)&sin->sin6_addr;
+		alen = sizeof(sin->sin6_addr);
+	} else
+#endif
+	if (sa->sa_family == AF_INET) {
+		struct sockaddr_in *sin = (struct sockaddr_in *)sa;
+		addr = (char *)&sin->sin_addr;
+		alen = sizeof(sin->sin_addr);
+	} else {
+		rc = NO_RECOVERY;
+		*err = (char *)hp_strerror( rc );
+		return rc;
+	}
+#if defined( HAVE_GETHOSTBYADDR_R )
+	for(;buflen<BUFMAX;) {
+		if (safe_realloc( &buf, buflen )==NULL) {
+			*err = (char *)STRERROR( ENOMEM );
+			return ENOMEM;
+		}
+#if (GETHOSTBYADDR_R_NARGS < 8)
+		hp=gethostbyaddr_r( addr, alen, sa->sa_family,
+			&hb, buf, buflen, &h_errno );
+		rc = (hp == NULL) ? -1 : 0;
+#else
+		rc = gethostbyaddr_r( addr, alen, sa->sa_family,
+			&hb, buf, buflen, 
+			&hp, &h_errno );
+#endif
+#ifdef NETDB_INTERNAL
+		if ((rc<0) &&
+			(h_errno==NETDB_INTERNAL) &&
+			(errno==ERANGE))
+		{
+			buflen*=2;
+			continue;
+		}
+#endif
+		break;
+	}
+	if (hp) {
+		strncpy( name, hp->h_name, namelen );
+	} else {
+		*err = (char *)hp_strerror( h_errno );
+	}
+	LDAP_FREE(buf);
+#else /* HAVE_GETHOSTBYADDR_R */
+
+#if defined( LDAP_R_COMPILE )
+	ldap_pvt_thread_mutex_lock( &ldap_int_resolv_mutex );
+#endif
+	hp = gethostbyaddr( addr, alen, sa->sa_family );
+	if (hp) {
+		strncpy( name, hp->h_name, namelen );
+		rc = 0;
+	} else {
+		rc = h_errno;
+		*err = (char *)hp_strerror( h_errno );
+	}
+#if defined( LDAP_R_COMPILE )
+	ldap_pvt_thread_mutex_unlock( &ldap_int_resolv_mutex );
+#endif
+
+#endif	/* !HAVE_GETHOSTBYADDR_R */
+	return rc;
+#endif	/* !HAVE_GETNAMEINFO */
+}
+
 int ldap_pvt_gethostbyaddr_a(
 	const char *addr,
 	int len,
@@ -193,10 +323,9 @@ int ldap_pvt_gethostbyaddr_a(
 		if (safe_realloc( buf, buflen )==NULL)
 			return r;
 #if (GETHOSTBYADDR_R_NARGS < 8)
-		r = ((*result=gethostbyaddr_r( addr, len, type,
-					       resbuf, *buf, buflen, 
-					       herrno_ptr )) == NULL) ?
-		    -1 : 0;
+		*result=gethostbyaddr_r( addr, len, type,
+			resbuf, *buf, buflen, herrno_ptr );
+		r = (*result == NULL) ? -1 : 0;
 #else
 		r = gethostbyaddr_r( addr, len, type,
 			resbuf, *buf, buflen, 
@@ -220,8 +349,9 @@ int ldap_pvt_gethostbyaddr_a(
 # define NEED_COPY_HOSTENT   
 	struct hostent *he;
 	int	retval;
+	*buf = NULL;   
 	
-	ldap_pvt_thread_mutex_lock( &ldap_int_gethostbyaddr_mutex );
+	ldap_pvt_thread_mutex_lock( &ldap_int_resolv_mutex );
 	
 	he = gethostbyaddr( addr, len, type );
 	
@@ -236,10 +366,12 @@ int ldap_pvt_gethostbyaddr_a(
 		retval = 0;
 	}
 	
-	ldap_pvt_thread_mutex_unlock( &ldap_int_gethostbyaddr_mutex );
+	ldap_pvt_thread_mutex_unlock( &ldap_int_resolv_mutex );
 	
-	return retval;   
+	return retval;
+
 #else /* gethostbyaddr() */
+	*buf = NULL;   
 	*result = gethostbyaddr( addr, len, type );
 
 	if (*result!=NULL) {
@@ -260,34 +392,29 @@ void ldap_int_utils_init( void )
 	done=1;
 
 #ifdef LDAP_R_COMPILE
-
 #if !defined( USE_CTIME_R ) && !defined( HAVE_REENTRANT_FUNCTIONS )
 	ldap_pvt_thread_mutex_init( &ldap_int_ctime_mutex );
 #endif
-
-#if !defined( HAVE_GETHOSTBYNAME_R )
-	ldap_pvt_thread_mutex_init( &ldap_int_gethostbyname_mutex );
-#endif
-
-#if !defined( HAVE_GETHOSTBYADDR_R )
-	ldap_pvt_thread_mutex_init( &ldap_int_gethostbyaddr_mutex );
+	ldap_pvt_thread_mutex_init( &ldap_int_resolv_mutex );
 #endif
 
 	/* call other module init functions here... */
-#endif
 }
 
 #if defined( NEED_COPY_HOSTENT )
 # undef NEED_SAFE_REALLOC
 #define NEED_SAFE_REALLOC
 
-static char *cpy_aliases( char ***tgtio, char *buf, char **src )
+static char *cpy_aliases(
+	char ***tgtio,
+	char *buf,
+	char **src )
 {
 	int len;
 	char **tgt=*tgtio;
 	for( ; (*src) ; src++ ) {
 		len = strlen( *src ) + 1;
-		memcpy( buf, *src, len );
+		AC_MEMCPY( buf, *src, len );
 		*tgt++=buf;
 		buf+=len;
 	}
@@ -295,11 +422,15 @@ static char *cpy_aliases( char ***tgtio, char *buf, char **src )
 	return buf;
 }
 
-static char *cpy_addresses( char ***tgtio, char *buf, char **src, int len )
+static char *cpy_addresses(
+	char ***tgtio,
+	char *buf,
+	char **src,
+	int len )
 {
    	char **tgt=*tgtio;
 	for( ; (*src) ; src++ ) {
-		memcpy( buf, *src, len );
+		AC_MEMCPY( buf, *src, len );
 		*tgt++=buf;
 		buf+=len;
 	}
@@ -307,30 +438,37 @@ static char *cpy_addresses( char ***tgtio, char *buf, char **src, int len )
 	return buf;
 }
 
-static int copy_hostent( struct hostent *res, char **buf, struct hostent * src )
+static int copy_hostent(
+	struct hostent *res,
+	char **buf,
+	struct hostent * src )
 {
 	char	**p;
 	char	**tp;
 	char	*tbuf;
 	int	name_len;
-	int	n_alias;
-	int	total_alias_len;
-	int	n_addr;
+	int	n_alias=0;
+	int	total_alias_len=0;
+	int	n_addr=0;
 	int	total_addr_len;
 	int	total_len;
 	  
 	/* calculate the size needed for the buffer */
 	name_len = strlen( src->h_name ) + 1;
 	
-	for( n_alias=total_alias_len=0, p=src->h_aliases; (*p) ; p++ ) {
-		total_alias_len += strlen( *p ) + 1;
-		n_alias++; 
+	if( src->h_aliases != NULL ) {
+		for( p = src->h_aliases; (*p) != NULL; p++ ) {
+			total_alias_len += strlen( *p ) + 1;
+			n_alias++; 
+		}
 	}
 
-	for( n_addr=0, p=src->h_addr_list; (*p) ; p++ ) {
-		n_addr++;
+	if( src->h_addr_list != NULL ) {
+		for( p = src->h_addr_list; (*p) != NULL; p++ ) {
+			n_addr++;
+		}
+		total_addr_len = n_addr * src->h_length;
 	}
-	total_addr_len = n_addr * src->h_length;
 	
 	total_len = (n_alias + n_addr + 2) * sizeof( char * ) +
 		total_addr_len + total_alias_len + name_len;
@@ -338,17 +476,21 @@ static int copy_hostent( struct hostent *res, char **buf, struct hostent * src )
 	if (safe_realloc( buf, total_len )) {			 
 		tp = (char **) *buf;
 		tbuf = *buf + (n_alias + n_addr + 2) * sizeof( char * );
-		memcpy( res, src, sizeof( struct hostent ) );
+		AC_MEMCPY( res, src, sizeof( struct hostent ) );
 		/* first the name... */
-		memcpy( tbuf, src->h_name, name_len );
+		AC_MEMCPY( tbuf, src->h_name, name_len );
 		res->h_name = tbuf; tbuf+=name_len;
 		/* now the aliases */
 		res->h_aliases = tp;
-		tbuf = cpy_aliases( &tp, tbuf, src->h_aliases );
+		if ( src->h_aliases != NULL ) {
+			tbuf = cpy_aliases( &tp, tbuf, src->h_aliases );
+		}
 		*tp++=NULL;
 		/* finally the addresses */
 		res->h_addr_list = tp;
-		tbuf = cpy_addresses( &tp, tbuf, src->h_addr_list, src->h_length );
+		if ( src->h_addr_list != NULL ) {
+			tbuf = cpy_addresses( &tp, tbuf, src->h_addr_list, src->h_length );
+		}
 		*tp++=NULL;
 		return 0;
 	}
@@ -368,4 +510,68 @@ static char *safe_realloc( char **buf, int len )
 }
 #endif
 
+char * ldap_pvt_get_fqdn( char *name )
+{
+	char *fqdn, *ha_buf;
+	char hostbuf[MAXHOSTNAMELEN+1];
+	struct hostent *hp, he_buf;
+	int rc, local_h_errno;
 
+	if( name == NULL ) {
+		if( gethostname( hostbuf, MAXHOSTNAMELEN ) == 0 ) {
+			hostbuf[MAXHOSTNAMELEN] = '\0';
+			name = hostbuf;
+		} else {
+			name = "localhost";
+		}
+	}
+
+	rc = ldap_pvt_gethostbyname_a( name,
+		&he_buf, &ha_buf, &hp, &local_h_errno );
+
+	if( rc < 0 || hp == NULL || hp->h_name == NULL ) {
+		fqdn = LDAP_STRDUP( name );
+	} else {
+		fqdn = LDAP_STRDUP( hp->h_name );
+	}
+
+	LDAP_FREE( ha_buf );
+	return fqdn;
+}
+
+#if ( defined( HAVE_GETADDRINFO ) || defined( HAVE_GETNAMEINFO ) ) \
+	&& !defined( HAVE_GAI_STRERROR )
+char *ldap_pvt_gai_strerror (int code) {
+	static struct {
+		int code;
+		const char *msg;
+	} values[] = {
+#ifdef EAI_ADDRFAMILY
+		{ EAI_ADDRFAMILY, N_("Address family for hostname not supported") },
+#endif
+		{ EAI_AGAIN, N_("Temporary failure in name resolution") },
+		{ EAI_BADFLAGS, N_("Bad value for ai_flags") },
+		{ EAI_FAIL, N_("Non-recoverable failure in name resolution") },
+		{ EAI_FAMILY, N_("ai_family not supported") },
+		{ EAI_MEMORY, N_("Memory allocation failure") },
+#ifdef EAI_NODATA
+		{ EAI_NODATA, N_("No address associated with hostname") },
+#endif    
+		{ EAI_NONAME, N_("Name or service not known") },
+		{ EAI_SERVICE, N_("Servname not supported for ai_socktype") },
+		{ EAI_SOCKTYPE, N_("ai_socktype not supported") },
+		{ EAI_SYSTEM, N_("System error") },
+		{ 0, NULL }
+	};
+
+	int i;
+
+	for ( i = 0; values[i].msg != NULL; i++ ) {
+		if ( values[i].code == code ) {
+			return (char *) _(values[i].msg);
+		}
+	}
+	
+	return _("Unknown error");
+}
+#endif
