@@ -1,7 +1,7 @@
 /* line64.c - routines for dealing with the slapd line format */
-/* $OpenLDAP: pkg/ldap/libraries/libldif/line64.c,v 1.45 2002/01/04 20:17:42 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/libraries/libldif/line64.c,v 1.52 2003/04/06 04:47:31 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -89,7 +89,7 @@ ldif_parse_line(
 
 	if( freeme == NULL ) {
 		ber_pvt_log_printf( LDAP_DEBUG_ANY, ldif_debug,
-			"ldif_parse_line: line malloc failed\n");
+			_("ldif_parse_line: line malloc failed\n"));
 		return( -1 );
 	}
 
@@ -99,7 +99,7 @@ ldif_parse_line(
 
 	if ( s == NULL ) {
 		ber_pvt_log_printf( LDAP_DEBUG_PARSE, ldif_debug,
-			"ldif_parse_line: missing ':' after %s\n",
+			_("ldif_parse_line: missing ':' after %s\n"),
 			type );
 		ber_memfree( freeme );
 		return( -1 );
@@ -114,16 +114,10 @@ ldif_parse_line(
 	url = 0;
 	b64 = 0;
 
-	if ( *s == '\0' ) {
-		/* no value */
-		value = "";
-		vlen = 0;
-		goto done;
-	}
-		
 	if ( *s == '<' ) {
 		s++;
 		url = 1;
+
 	} else if ( *s == ':' ) {
 		/* base 64 encoded value */
 		s++;
@@ -142,20 +136,18 @@ ldif_parse_line(
 	}
 	*d = '\0';
 
-	/* if no value is present, error out */
-	if ( *s == '\0' ) {
-		ber_pvt_log_printf( LDAP_DEBUG_PARSE, ldif_debug,
-			"ldif_parse_line: %s missing %svalue\n", type,
-				url ? "URL " : b64 ? "base64 " : "" );
-		value = NULL;
-		vlen = 0;
-		goto done;
-	}
-
 	if ( b64 ) {
 		char *byte = s;
 
-		value = s;
+		if ( *s == '\0' ) {
+			/* no value is present, error out */
+			ber_pvt_log_printf( LDAP_DEBUG_PARSE, ldif_debug,
+				_("ldif_parse_line: %s missing base64 value\n"), type );
+			ber_memfree( freeme );
+			return( -1 );
+		}
+
+		byte = value = s;
 
 		for ( p = s, vlen = 0; p < d; p += 4, vlen += 3 ) {
 			int i;
@@ -163,8 +155,8 @@ ldif_parse_line(
 				if ( p[i] != '=' && (p[i] & 0x80 ||
 				    b642nib[ p[i] & 0x7f ] > 0x3f) ) {
 					ber_pvt_log_printf( LDAP_DEBUG_ANY, ldif_debug,
-						"ldif_parse_line: %s: invalid base64 encoding"
-						" char (%c) 0x%x\n",
+						_("ldif_parse_line: %s: invalid base64 encoding"
+						" char (%c) 0x%x\n"),
 					    type, p[i], p[i] );
 					ber_memfree( freeme );
 					return( -1 );
@@ -199,9 +191,17 @@ ldif_parse_line(
 		s[ vlen ] = '\0';
 
 	} else if ( url ) {
+		if ( *s == '\0' ) {
+			/* no value is present, error out */
+			ber_pvt_log_printf( LDAP_DEBUG_PARSE, ldif_debug,
+				_("ldif_parse_line: %s missing URL value\n"), type );
+			ber_memfree( freeme );
+			return( -1 );
+		}
+
 		if( ldif_fetch_url( s, &value, &vlen ) ) {
 			ber_pvt_log_printf( LDAP_DEBUG_ANY, ldif_debug,
-				"ldif_parse_line: %s: URL \"%s\" fetch failed\n",
+				_("ldif_parse_line: %s: URL \"%s\" fetch failed\n"),
 				type, s );
 			ber_memfree( freeme );
 			return( -1 );
@@ -212,21 +212,21 @@ ldif_parse_line(
 		vlen = (int) (d - s);
 	}
 
-done:
 	type = ber_strdup( type );
 
 	if( type == NULL ) {
 		ber_pvt_log_printf( LDAP_DEBUG_ANY, ldif_debug,
-			"ldif_parse_line: type malloc failed\n");
+			_("ldif_parse_line: type malloc failed\n"));
+		if( url ) ber_memfree( value );
 		ber_memfree( freeme );
 		return( -1 );
 	}
 
-	if( !url && value != NULL ) {
+	if( !url ) {
 		p = ber_memalloc( vlen + 1 );
 		if( p == NULL ) {
 			ber_pvt_log_printf( LDAP_DEBUG_ANY, ldif_debug,
-				"ldif_parse_line: value malloc failed\n");
+				_("ldif_parse_line: value malloc failed\n"));
 			ber_memfree( type );
 			ber_memfree( freeme );
 			return( -1 );
@@ -411,8 +411,8 @@ ldif_sput(
 	stop = (const unsigned char *) (val + vlen);
 
 	if ( type == LDIF_PUT_VALUE
-		&& isgraph( val[0] ) && val[0] != ':' && val[0] != '<'
-		&& isgraph( val[vlen-1] )
+		&& isgraph( (unsigned char) val[0] ) && val[0] != ':' && val[0] != '<'
+		&& isgraph( (unsigned char) val[vlen-1] )
 #ifndef LDAP_BINARY_DEBUG
 		&& strstr( name, ";binary" ) == NULL
 #endif
@@ -523,7 +523,7 @@ ldif_put(
 
     if ( buf == NULL ) {
 		ber_pvt_log_printf( LDAP_DEBUG_ANY, ldif_debug,
-			"ldif_type_and_value: malloc failed!" );
+			_("ldif_type_and_value: malloc failed!"));
 		return NULL;
     }
 
@@ -542,8 +542,8 @@ int ldif_is_not_printable(
 		return -1;
 	}
 
-	if( isgraph( val[0] ) && val[0] != ':' && val[0] != '<' &&
-		isgraph( val[vlen-1] ) )
+	if( isgraph( (unsigned char) val[0] ) && val[0] != ':' && val[0] != '<' &&
+		isgraph( (unsigned char) val[vlen-1] ) )
 	{
 		ber_len_t i;
 
