@@ -1,7 +1,7 @@
 /* abandon.c - decode and handle an ldap abandon operation */
-/* $OpenLDAP: pkg/ldap/servers/slapd/abandon.c,v 1.26 2002/01/04 20:17:44 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/abandon.c,v 1.36 2003/04/15 18:40:02 julius Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -25,18 +25,14 @@
 #include "slap.h"
 
 int
-do_abandon(
-    Connection	*conn,
-    Operation	*op
-)
+do_abandon( Operation *op, SlapReply *rs )
 {
-	ber_int_t		id;
+	ber_int_t	id;
 	Operation	*o;
-	int rc;
+	int		i;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_ENTRY, "conn: %d do_abandon\n",
-		conn->c_connid));
+	LDAP_LOG( OPERATION, ENTRY, "conn: %d do_abandon\n", op->o_connid, 0, 0);
 #else
 	Debug( LDAP_DEBUG_TRACE, "do_abandon\n", 0, 0, 0 );
 #endif
@@ -49,33 +45,33 @@ do_abandon(
 
 	if ( ber_scanf( op->o_ber, "i", &id ) == LBER_ERROR ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_ERR, 
-		       "conn: %d do_abandon: ber_scanf failed\n",
-		       conn->c_connid ));
+		LDAP_LOG( OPERATION, ERR, 
+			"conn: %d do_abandon: ber_scanf failed\n", op->o_connid, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "do_abandon: ber_scanf failed\n", 0, 0 ,0 );
 #endif
-		send_ldap_disconnect( conn, op,
+		send_ldap_discon( op, rs,
 			LDAP_PROTOCOL_ERROR, "decoding error" );
 		return -1;
 	}
 
-	if( (rc = get_ctrls( conn, op, 0 )) != LDAP_SUCCESS ) {
+	if( get_ctrls( op, rs, 0 ) != LDAP_SUCCESS ) {
 		Debug( LDAP_DEBUG_ANY, "do_abandon: get_ctrls failed\n", 0, 0 ,0 );
-		return rc;
+		return rs->sr_err;
 	} 
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_ARGS,
-		"do_abandon: conn: %d  id=%ld\n", conn->c_connid, (long) id ));
+	LDAP_LOG( OPERATION, ARGS, "do_abandon: conn: %d  id=%ld\n", 
+		op->o_connid, (long) id, 0 );
 #else
 	Debug( LDAP_DEBUG_ARGS, "do_abandon: id=%ld\n", (long) id, 0 ,0 );
 #endif
 
 	if( id <= 0 ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_ERR,
-		    "do_abandon: conn: %d bad msgid %ld\n", conn->c_connid, (long) id ));
+		LDAP_LOG( OPERATION, ERR, 
+			"do_abandon: conn: %d bad msgid %ld\n", 
+			op->o_connid, (long) id, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"do_abandon: bad msgid %ld\n", (long) id, 0, 0 );
@@ -83,37 +79,43 @@ do_abandon(
 		return LDAP_SUCCESS;
 	}
 
-	ldap_pvt_thread_mutex_lock( &conn->c_mutex );
+	ldap_pvt_thread_mutex_lock( &op->o_conn->c_mutex );
 	/*
 	 * find the operation being abandoned and set the o_abandon
 	 * flag.  It's up to the backend to periodically check this
 	 * flag and abort the operation at a convenient time.
 	 */
 
-	LDAP_STAILQ_FOREACH( o, &conn->c_ops, o_next ) {
+	LDAP_STAILQ_FOREACH( o, &op->o_conn->c_ops, o_next ) {
 		if ( o->o_msgid == id ) {
-			ldap_pvt_thread_mutex_lock( &o->o_abandonmutex );
 			o->o_abandon = 1;
-			ldap_pvt_thread_mutex_unlock( &o->o_abandonmutex );
 			goto done;
 		}
 	}
 
-	LDAP_STAILQ_FOREACH( o, &conn->c_pending_ops, o_next ) {
+	LDAP_STAILQ_FOREACH( o, &op->o_conn->c_pending_ops, o_next ) {
 		if ( o->o_msgid == id ) {
-			LDAP_STAILQ_REMOVE( &conn->c_pending_ops, o, slap_op, o_next );
+			LDAP_STAILQ_REMOVE( &op->o_conn->c_pending_ops, o, slap_op, o_next );
 			slap_op_free( o );
 			goto done;
 		}
 	}
 
 done:
-	ldap_pvt_thread_mutex_unlock( &conn->c_mutex );
+
+	op->orn_msgid = id;
+	for ( i = 0; i < nbackends; i++ ) {
+		op->o_bd = &backends[i];
+
+		if( op->o_bd->be_abandon ) op->o_bd->be_abandon( op, rs );
+	}
+
+	ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_ENTRY,
+	LDAP_LOG( OPERATION, ENTRY, 
 		"do_abandon: conn: %d op=%ld %sfound\n",
-		conn->c_connid, (long)id, o ? "" : "not " ));
+		op->o_connid, (long)id, o ? "" : "not " );
 #else
 	Debug( LDAP_DEBUG_TRACE, "do_abandon: op=%ld %sfound\n",
 		(long) id, o ? "" : "not ", 0 );
