@@ -1,5 +1,5 @@
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  *
  * Copyright 2001, Pierangelo Masarati, All rights reserved. <ando@sys-net.it>
@@ -76,102 +76,74 @@
 #include "back-meta.h"
 
 int
-meta_back_modify(
-		Backend	*be,
-		Connection	*conn,
-		Operation	*op,
-		struct berval	*dn,
-		struct berval	*ndn,
-		Modifications	*modlist
-)
+meta_back_modify( Operation *op, SlapReply *rs )
 {
-	struct metainfo	*li = ( struct metainfo * )be->be_private;
-	struct metaconn *lc;
-	LDAPMod **modv;
-	LDAPMod *mods;
-	Modifications *ml;
-	int candidate = -1, i;
-	char *mdn;
-	struct berval mapped;
+	struct metainfo		*li = ( struct metainfo * )op->o_bd->be_private;
+	struct metaconn 	*lc;
+	int			rc = 0;
+	LDAPMod			**modv = NULL;
+	LDAPMod			*mods = NULL;
+	Modifications		*ml;
+	int			candidate = -1, i;
+	struct berval		mdn = { 0, NULL };
+	struct berval		mapped;
+	dncookie		dc;
 
-	lc = meta_back_getconn( li, conn, op, META_OP_REQUIRE_SINGLE,
-			ndn, &candidate );
-	if ( !lc || !meta_back_dobind( lc, op ) || !meta_back_is_valid( lc, candidate ) ) {
-		send_ldap_result( conn, op, LDAP_OPERATIONS_ERROR,
-				NULL, NULL, NULL, NULL );
-		return -1;
+	lc = meta_back_getconn( op, rs, META_OP_REQUIRE_SINGLE,
+			&op->o_req_ndn, &candidate );
+	if ( !lc ) {
+		rc = -1;
+		goto cleanup;
+	}
+	
+	if ( !meta_back_dobind( lc, op )
+			|| !meta_back_is_valid( lc, candidate ) ) {
+		rs->sr_err = LDAP_OTHER;
+		rc = -1;
+		goto cleanup;
 	}
 
 	/*
 	 * Rewrite the modify dn, if needed
 	 */
-	switch ( rewrite_session( li->targets[ candidate ]->rwinfo,
-				"modifyDn", dn->bv_val, conn, &mdn ) ) {
-	case REWRITE_REGEXEC_OK:
-		if ( mdn == NULL ) {
-			mdn = ( char * )dn->bv_val;
-		}
-#ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_DETAIL1,
-				"[rw] modifyDn: \"%s\" -> \"%s\"\n",
-				dn->bv_val, mdn ));
-#else /* !NEW_LOGGING */
-		Debug( LDAP_DEBUG_ARGS, "rw> modifyDn: \"%s\" -> \"%s\"\n%s",
-				dn->bv_val, mdn, "" );
-#endif /* !NEW_LOGGING */
-		break;
-		
-	case REWRITE_REGEXEC_UNWILLING:
-		send_ldap_result( conn, op, LDAP_UNWILLING_TO_PERFORM,
-				NULL, NULL, NULL, NULL );
-		return -1;
+	dc.rwmap = &li->targets[ candidate ]->rwmap;
+	dc.conn = op->o_conn;
+	dc.rs = rs;
+	dc.ctx = "modifyDn";
 
-	case REWRITE_REGEXEC_ERR:
-		send_ldap_result( conn, op, LDAP_OPERATIONS_ERROR,
-				NULL, NULL, NULL, NULL );
-		return -1;
+	if ( ldap_back_dn_massage( &dc, &op->o_req_dn, &mdn ) ) {
+		rc = -1;
+		goto cleanup;
 	}
 
-	for ( i = 0, ml = modlist; ml; i++ ,ml = ml->sml_next )
+	for ( i = 0, ml = op->oq_modify.rs_modlist; ml; i++ ,ml = ml->sml_next )
 		;
 
 	mods = ch_malloc( sizeof( LDAPMod )*i );
 	if ( mods == NULL ) {
-		if ( mdn != dn->bv_val ) {
-			free( mdn );
-		}
-		return -1;
+		rs->sr_err = LDAP_NO_MEMORY;
+		rc = -1;
+		goto cleanup;
 	}
 	modv = ( LDAPMod ** )ch_malloc( ( i + 1 )*sizeof( LDAPMod * ) );
 	if ( modv == NULL ) {
-		free( mods );
-		if ( mdn != dn->bv_val ) {
-			free( mdn );
-		}
-		return -1;
+		rs->sr_err = LDAP_NO_MEMORY;
+		rc = -1;
+		goto cleanup;
 	}
 
-	for ( i = 0, ml = modlist; ml; ml = ml->sml_next ) {
+	dc.ctx = "modifyAttrDN";
+	for ( i = 0, ml = op->oq_modify.rs_modlist; ml; ml = ml->sml_next ) {
 		int j;
-		/*
-		 * lastmod should always be <off>
-		 */
-#if 0
-		if ( !strcasecmp( a->a_desc->ad_cname.bv_val,
-			slap_schema.si_ad_creatorsName->ad_cname.bv_val )
-			|| !strcasecmp( a->a_desc->ad_cname.bv_val,
-			slap_schema.si_ad_createTimestamp->ad_cname.bv_val )
-			|| !strcasecmp( a->a_desc->ad_cname.bv_val,
-			slap_schema.si_ad_modifiersName->ad_cname.bv_val )
-			|| !strcasecmp( a->a_desc->ad_cname.bv_val,
-			slap_schema.si_ad_modifyTimestamp->ad_cname.bv_val ) ) {
+
+		if ( ml->sml_desc->ad_type->sat_no_user_mod  ) {
 			continue;
 		}
-#endif
 
-		ldap_back_map( &li->targets[ candidate ]->at_map,
-				&ml->sml_desc->ad_cname, &mapped, 0 );
-		if ( mapped.bv_val == NULL ) {
+		ldap_back_map( &li->targets[ candidate ]->rwmap.rwm_at,
+				&ml->sml_desc->ad_cname, &mapped,
+				BACKLDAP_MAP );
+		if ( mapped.bv_val == NULL || mapped.bv_val[0] == '\0' ) {
 			continue;
 		}
 
@@ -186,30 +158,46 @@ meta_back_modify(
 		 */
 		if ( strcmp( ml->sml_desc->ad_type->sat_syntax->ssyn_oid,
 					SLAPD_DN_SYNTAX ) == 0 ) {
-			ldap_dnattr_rewrite(
-				li->targets[ candidate ]->rwinfo,
-				ml->sml_bvalues, conn );
+			( void )ldap_dnattr_rewrite( &dc, ml->sml_bvalues );
+		}
+
+		if ( ml->sml_bvalues != NULL ){
+			for (j = 0; ml->sml_bvalues[ j ].bv_val; j++);
+			mods[ i ].mod_bvalues = (struct berval **)ch_malloc((j+1) *
+				sizeof(struct berval *));
+			for (j = 0; ml->sml_bvalues[ j ].bv_val; j++)
+				mods[ i ].mod_bvalues[ j ] = &ml->sml_bvalues[j];
+			mods[ i ].mod_bvalues[ j ] = NULL;
+
+		} else {
+			mods[ i ].mod_bvalues = NULL;
 		}
 
-		for (j = 0; ml->sml_bvalues[ j ].bv_val; j++);
-		mods[ i ].mod_bvalues = (struct berval **)ch_malloc((j+1) *
-			sizeof(struct berval *));
-		for (j = 0; ml->sml_bvalues[ j ].bv_val; j++)
-			mods[ i ].mod_bvalues[ j ] = &ml->sml_bvalues[j];
-		mods[ i ].mod_bvalues[ j ] = NULL;
 		i++;
 	}
 	modv[ i ] = 0;
 
-	ldap_modify_s( lc->conns[ candidate ]->ld, mdn, modv );
+	ldap_modify_s( lc->conns[ candidate ].ld, mdn.bv_val, modv );
 
-	if ( mdn != dn->bv_val ) {
-		free( mdn );
+cleanup:;
+	if ( mdn.bv_val != op->o_req_dn.bv_val ) {
+		free( mdn.bv_val );
+	}
+	if ( modv != NULL ) {
+		for ( i = 0; modv[ i ]; i++) {
+			free( modv[ i ]->mod_bvalues );
+		}
 	}
-	for ( i=0; modv[ i ]; i++)
-		free( modv[ i ]->mod_bvalues );
 	free( mods );
 	free( modv );
-	return meta_back_op_result( lc, op );
+	
+	if ( rc == 0 ) {
+		return meta_back_op_result( lc, op, rs ) == LDAP_SUCCESS
+			? 0 : 1;
+	} /* else */
+
+	send_ldap_result( op, rs );
+
+	return rc;
 }
 
