@@ -1,6 +1,6 @@
 /* database.c - deals with database subsystem */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /*
@@ -34,10 +34,16 @@
 #include "portable.h"
 
 #include <stdio.h>
+#include <ac/string.h>
 
 #include "slap.h"
 #include "back-monitor.h"
 
+#if defined(LDAP_SLAPI)
+#include "slapi.h"
+static int monitor_back_add_plugin( Backend *be, Entry *e );
+#endif /* defined(LDAP_SLAPI) */
+
 int
 monitor_subsys_database_init(
 	BackendDB	*be
@@ -47,12 +53,8 @@ monitor_subsys_database_init(
 	Entry			*e, *e_database, *e_tmp;
 	int			i;
 	struct monitorentrypriv	*mp;
-	AttributeDescription 	*ad_nc = slap_schema.si_ad_namingContexts;
-	AttributeDescription 	*ad_seeAlso = NULL;
-	const char		*text = NULL;
 
 	assert( be != NULL );
-	assert( monitor_ad_desc != NULL );
 
 	mi = ( struct monitorinfo * )be->be_private;
 
@@ -60,10 +62,10 @@ monitor_subsys_database_init(
 				&monitor_subsys[SLAPD_MONITOR_DATABASE].mss_ndn, 
 				&e_database ) ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
+		LDAP_LOG( OPERATION, CRIT,
 			"monitor_subsys_database_init: "
 			"unable to get entry '%s'\n",
-			monitor_subsys[SLAPD_MONITOR_DATABASE].mss_ndn.bv_val ));
+			monitor_subsys[SLAPD_MONITOR_DATABASE].mss_ndn.bv_val, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"monitor_subsys_database_init: "
@@ -74,35 +76,44 @@ monitor_subsys_database_init(
 		return( -1 );
 	}
 
-	if ( slap_str2ad( "seeAlso", &ad_seeAlso, &text ) != LDAP_SUCCESS ) {
-		return( -1 );
-	}
-
 	e_tmp = NULL;
 	for ( i = nBackendDB; i--; ) {
-		char buf[1024];
+		char buf[ BACKMONITOR_BUFSIZE ];
 		int j;
 
 		be = &backendDB[i];
 
+		/* Subordinates are not exposed as their own naming context */
+		if ( SLAP_GLUE_SUBORDINATE( be ) ) {
+			continue;
+		}
+
 		snprintf( buf, sizeof( buf ),
 				"dn: cn=Database %d,%s\n"
-				SLAPD_MONITOR_OBJECTCLASSES
+				"objectClass: %s\n"
+				"structuralObjectClass: %s\n"
 				"cn: Database %d\n"
-				"description: %s",
+				"description: This object contains the type of the database.\n"
+				"%s: %s\n"
+				"createTimestamp: %s\n"
+				"modifyTimestamp: %s\n",
 				i,
 				monitor_subsys[SLAPD_MONITOR_DATABASE].mss_dn.bv_val,
+				mi->mi_oc_monitoredObject->soc_cname.bv_val,
+				mi->mi_oc_monitoredObject->soc_cname.bv_val,
 				i,
-				be->bd_info->bi_type );
+				mi->mi_ad_monitoredInfo->ad_cname.bv_val,
+				be->bd_info->bi_type,
+				mi->mi_startTime.bv_val,
+				mi->mi_startTime.bv_val );
 		
 		e = str2entry( buf );
 		if ( e == NULL ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
+			LDAP_LOG( OPERATION, CRIT,
 				"monitor_subsys_database_init: "
 				"unable to create entry 'cn=Database %d,%s'\n",
-				i, 
-				monitor_subsys[SLAPD_MONITOR_DATABASE].mss_ndn.bv_val ));
+				i, monitor_subsys[SLAPD_MONITOR_DATABASE].mss_ndn.bv_val, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_subsys_database_init: "
@@ -114,32 +125,29 @@ monitor_subsys_database_init(
 			return( -1 );
 		}
 		
-		for ( j = 0; be->be_suffix[j]; j++ ) {
-			struct berval 		bv[ 2 ];
-
-			bv[ 0 ] = *be->be_suffix[ j ];
-			bv[ 1 ].bv_val = NULL;
-
-			attr_merge( e, ad_nc, bv );
-			attr_merge( e_database, ad_nc, bv );
+		if ( be->be_flags & SLAP_BFLAG_MONITOR ) {
+			attr_merge( e, slap_schema.si_ad_monitorContext,
+					be->be_suffix, be->be_nsuffix );
+			attr_merge( e_database, slap_schema.si_ad_monitorContext,
+					be->be_suffix, be->be_nsuffix );
+		} else {
+			attr_merge( e, slap_schema.si_ad_namingContexts,
+					be->be_suffix, be->be_nsuffix );
+			attr_merge( e_database, slap_schema.si_ad_namingContexts,
+					be->be_suffix, be->be_nsuffix );
 		}
 
 		for ( j = nBackendInfo; j--; ) {
-			if ( &backendInfo[ j ] == be->bd_info ) {
-				struct berval 		bv[ 2 ];
-
-				/* we check the pointer; the test on the
-				 * string should be more reliable */
-				assert( strcasecmp( backendInfo[ j ].bi_type,
-					be->bd_info->bi_type ) == 0 );
+			if ( backendInfo[ j ].bi_type == be->bd_info->bi_type ) {
+				struct berval 		bv;
 
 				snprintf( buf, sizeof( buf ), 
 					"cn=Backend %d,%s", 
 					j, monitor_subsys[SLAPD_MONITOR_BACKEND].mss_dn.bv_val );
-				bv[ 0 ].bv_val = buf;
-				bv[ 0 ].bv_len = strlen( buf );
-				bv[ 1 ].bv_val = NULL;
-				attr_merge( e, ad_seeAlso, bv );
+				bv.bv_val = buf;
+				bv.bv_len = strlen( buf );
+				attr_merge_normalize_one( e, mi->mi_ad_seeAlso,
+						&bv, NULL );
 				break;
 			}
 		}
@@ -156,11 +164,10 @@ monitor_subsys_database_init(
 
 		if ( monitor_cache_add( mi, e ) ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
+			LDAP_LOG( OPERATION, CRIT,
 				"monitor_subsys_database_init: "
 				"unable to add entry 'cn=Database %d,%s'\n",
-				i, 
-				monitor_subsys[SLAPD_MONITOR_DATABASE].mss_ndn.bv_val ));
+				i, monitor_subsys[SLAPD_MONITOR_DATABASE].mss_ndn.bv_val, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"monitor_subsys_database_init: "
@@ -172,6 +179,10 @@ monitor_subsys_database_init(
 			return( -1 );
 		}
 
+#if defined(LDAP_SLAPI)
+		monitor_back_add_plugin( be, e );
+#endif /* defined(LDAP_SLAPI) */
+
 		e_tmp = e;
 	}
 	
@@ -183,3 +194,56 @@ monitor_subsys_database_init(
 	return( 0 );
 }
 
+#if defined(LDAP_SLAPI)
+static int
+monitor_back_add_plugin( Backend *be, Entry *e_database )
+{
+	Slapi_PBlock		*pCurrentPB; 
+	int			i, rc = LDAP_SUCCESS;
+	struct monitorinfo	*mi = ( struct monitorinfo * )be->be_private;
+
+	if ( slapi_x_pblock_get_first( be, &pCurrentPB ) != LDAP_SUCCESS ) {
+		/*
+		 * LDAP_OTHER is returned if no plugins are installed
+		 */
+		rc = LDAP_OTHER;
+		goto done;
+	}
+
+	i = 0;
+	do {
+		Slapi_PluginDesc	*srchdesc;
+		char			buf[ BACKMONITOR_BUFSIZE ];
+		struct berval		bv;
+
+		rc = slapi_pblock_get( pCurrentPB, SLAPI_PLUGIN_DESCRIPTION,
+				&srchdesc );
+		if ( rc != LDAP_SUCCESS ) {
+			goto done;
+		}
+
+		snprintf( buf, sizeof(buf),
+				"plugin %d name: %s; "
+				"vendor: %s; "
+				"version: %s; "
+				"description: %s", 
+				i,
+				srchdesc->spd_id,
+				srchdesc->spd_vendor,
+				srchdesc->spd_version,
+				srchdesc->spd_description );
+
+		bv.bv_val = buf;
+		bv.bv_len = strlen( buf );
+		attr_merge_normalize_one( e_database,
+				mi->mi_ad_monitoredInfo, &bv, NULL );
+
+		i++;
+
+	} while ( ( slapi_x_pblock_get_next( &pCurrentPB ) == LDAP_SUCCESS )
+			&& ( pCurrentPB != NULL ) );
+
+done:
+	return rc;
+}
+#endif /* defined(LDAP_SLAPI) */
