@@ -1,5 +1,11 @@
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/add.c,v 1.11 2003/05/25 01:56:59 kurt Exp $ */
+/*
+ * Copyright 1999-2003 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 /*
  *	 Copyright 1999, John C. Quillan, All rights reserved.
+ *	 Portions Copyright 2002, myinternet Limited. All rights reserved.
  *
  *	 Redistribution and use in source and binary forms are permitted only
  *	 as authorized by the OpenLDAP Public License.	A copy of this
@@ -10,65 +16,60 @@
 #include "portable.h"
 
 #include <stdio.h>
-/*	#include <ac/types.h>
-	#include <ac/socket.h>
-*/
+
+#include "slap.h"
+#ifdef HAVE_WIN32_ASPERL
+#include "asperl_undefs.h"
+#endif
 
 #include <EXTERN.h>
 #include <perl.h>
 
-#include "slap.h"
 #include "perl_back.h"
 
 int
 perl_back_add(
-	Backend	*be,
-	Connection	*conn,
 	Operation	*op,
-	Entry	*e
-)
+	SlapReply	*rs )
 {
+	PerlBackend *perl_back = (PerlBackend *) op->o_bd->be_private;
 	int len;
 	int count;
-	int return_code;
 
-	PerlBackend *perl_back = (PerlBackend *) be->be_private;
-
-	ldap_pvt_thread_mutex_lock( &perl_interpreter_mutex );	
+	ldap_pvt_thread_mutex_lock( &perl_interpreter_mutex );
+	ldap_pvt_thread_mutex_lock( &entry2str_mutex );
 
 	{
 		dSP; ENTER; SAVETMPS;
 
 		PUSHMARK(sp);
 		XPUSHs( perl_back->pb_obj_ref );
-		XPUSHs(sv_2mortal(newSVpv( entry2str( e, &len, 0 ), 0 )));
+		XPUSHs(sv_2mortal(newSVpv( entry2str( op->ora_e, &len ), 0 )));
 
 		PUTBACK;
 
+#ifdef PERL_IS_5_6
+		count = call_method("add", G_SCALAR);
+#else
 		count = perl_call_method("add", G_SCALAR);
+#endif
 
 		SPAGAIN;
 
 		if (count != 1) {
-			croak("Big trouble in back_search\n");
+			croak("Big trouble in back_add\n");
 		}
 							 
-		return_code = POPi;
+		rs->sr_err = POPi;
 
 		PUTBACK; FREETMPS; LEAVE;
 	}
 
+	ldap_pvt_thread_mutex_unlock( &entry2str_mutex );
 	ldap_pvt_thread_mutex_unlock( &perl_interpreter_mutex );	
 
-	if( return_code != 0 ) {
-		send_ldap_result( conn, op, LDAP_OPERATIONS_ERROR,
-			NULL, NULL, NULL, NULL );
+	send_ldap_result( op, rs );
 
-	} else {
-		send_ldap_result( conn, op, LDAP_SUCCESS,
-			NULL, NULL, NULL, NULL );
-	}
-
-	Debug( LDAP_DEBUG_ANY, "Here ADD\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_ANY, "Perl ADD\n", 0, 0, 0 );
 	return( 0 );
 }
