@@ -1,6 +1,11 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/close.c,v 1.4 1999/09/08 17:06:38 kdz Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/close.c,v 1.9 2003/05/25 01:56:59 kurt Exp $ */
+/*
+ * Copyright 1999-2003 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 /*
  *	 Copyright 1999, John C. Quillan, All rights reserved.
+ *	 Portions Copyright 2002, myinternet Limited. All rights reserved.
  *
  *	 Redistribution and use in source and binary forms are permitted only
  *	 as authorized by the OpenLDAP Public License.	A copy of this
@@ -12,14 +17,15 @@
 /* init.c - initialize shell backend */
 	
 #include <stdio.h>
-/*	#include <ac/types.h>
-	#include <ac/socket.h>
-*/
+
+#include "slap.h"
+#ifdef HAVE_WIN32_ASPERL
+#include "asperl_undefs.h"
+#endif
 
 #include <EXTERN.h>
 #include <perl.h>
 
-#include "slap.h"
 #include "perl_back.h"
 
 /**********************************************************
@@ -35,7 +41,7 @@ perl_back_close(
 {
 	ldap_pvt_thread_mutex_lock( &perl_interpreter_mutex );	
 
-	perl_destruct(perl_interpreter);
+	perl_destruct(PERL_INTERPRETER);
 
 	ldap_pvt_thread_mutex_unlock( &perl_interpreter_mutex );	
 
@@ -47,8 +53,8 @@ perl_back_destroy(
 	BackendInfo *bd
 )
 {
-	perl_free(perl_interpreter);
-	perl_interpreter = NULL;
+	perl_free(PERL_INTERPRETER);
+	PERL_INTERPRETER = NULL;
 
 	ldap_pvt_thread_mutex_destroy( &perl_interpreter_mutex );	
 
@@ -62,4 +68,6 @@ perl_back_db_destroy(
 {
 	free( be->be_private );
 	be->be_private = NULL;
+
+	return 0;
 }
