@@ -1,6 +1,11 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/init.c,v 1.23 2002/02/02 09:10:35 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/init.c,v 1.30 2003/05/25 01:56:59 kurt Exp $ */
+/*
+ * Copyright 1999-2003 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 /*
  *	 Copyright 1999, John C. Quillan, All rights reserved.
+ *	 Portions Copyright 2002, myinternet Limited. All rights reserved.
  *
  *	 Redistribution and use in source and binary forms are permitted only
  *	 as authorized by the OpenLDAP Public License.	A copy of this
@@ -12,36 +17,36 @@
  /* init.c - initialize shell backend */
 	
 #include <stdio.h>
-/* #include <ac/types.h>
-	#include <ac/socket.h>
-*/
-
 
+#include "slap.h"
+#ifdef HAVE_WIN32_ASPERL
+#include "asperl_undefs.h"
+#endif
 
 #include <EXTERN.h>
 #include <perl.h>
 
-#include "slap.h"
 #include "perl_back.h"
 
 
-static void perl_back_xs_init LDAP_P((void));
-EXT void boot_DynaLoader LDAP_P((CV* cv));
+static void perl_back_xs_init LDAP_P((PERL_BACK_XS_INIT_PARAMS));
+EXT void boot_DynaLoader LDAP_P((PERL_BACK_BOOT_DYNALOADER_PARAMS));
 
-PerlInterpreter *perl_interpreter = NULL;
+PerlInterpreter *PERL_INTERPRETER = NULL;
 ldap_pvt_thread_mutex_t	perl_interpreter_mutex;
 
 #ifdef SLAPD_PERL_DYNAMIC
 
-int back_perl_LTX_init_module(int argc, char *argv[]) {
-    BackendInfo bi;
+int back_perl_LTX_init_module(int argc, char *argv[])
+{
+	BackendInfo bi;
 
-    memset( &bi, '\0', sizeof(bi) );
-    bi.bi_type = "perl";
-    bi.bi_init = perl_back_initialize;
+	memset( &bi, '\0', sizeof(bi) );
+	bi.bi_type = "perl";
+	bi.bi_init = perl_back_initialize;
 
-    backend_add(&bi);
-    return 0;
+	backend_add(&bi);
+	return 0;
 }
 
 #endif /* SLAPD_PERL_DYNAMIC */
@@ -62,16 +67,16 @@ perl_back_initialize(
 
 	Debug( LDAP_DEBUG_TRACE, "perl backend open\n", 0, 0, 0 );
 
-	if( perl_interpreter != NULL ) {
+	if( PERL_INTERPRETER != NULL ) {
 		Debug( LDAP_DEBUG_ANY, "perl backend open: already opened\n",
 			0, 0, 0 );
 		return 1;
 	}
 	
-	perl_interpreter = perl_alloc();
-	perl_construct(perl_interpreter);
-	perl_parse(perl_interpreter, perl_back_xs_init, 3, embedding, (char **)NULL);
-	perl_run(perl_interpreter);
+	PERL_INTERPRETER = perl_alloc();
+	perl_construct(PERL_INTERPRETER);
+	perl_parse(PERL_INTERPRETER, perl_back_xs_init, 3, embedding, (char **)NULL);
+	perl_run(PERL_INTERPRETER);
 
 	bi->bi_open = perl_back_open;
 	bi->bi_config = 0;
@@ -80,7 +85,7 @@ perl_back_initialize(
 
 	bi->bi_db_init = perl_back_db_init;
 	bi->bi_db_config = perl_back_db_config;
-	bi->bi_db_open = 0;
+	bi->bi_db_open = perl_back_db_open;
 	bi->bi_db_close = 0;
 	bi->bi_db_destroy = perl_back_db_destroy;
 
@@ -96,8 +101,6 @@ perl_back_initialize(
 
 	bi->bi_extended = 0;
 
-	bi->bi_acl_group = 0;
-	bi->bi_acl_attribute = 0;
 	bi->bi_chk_referrals = 0;
 
 	bi->bi_connection_init = 0;
@@ -117,22 +120,66 @@ perl_back_open(
 
 int
 perl_back_db_init(
-	Backend	*be
+	BackendDB	*be
 )
 {
 	be->be_private = (PerlBackend *) ch_malloc( sizeof(PerlBackend) );
 	memset( be->be_private, '\0', sizeof(PerlBackend));
 
+	((PerlBackend *)be->be_private)->pb_filter_search_results = 0;
+
 	Debug( LDAP_DEBUG_TRACE, "perl backend db init\n", 0, 0, 0 );
 
 	return 0;
 }
 
+int
+perl_back_db_open(
+	BackendDB	*be
+)
+{
+	int count;
+	int return_code;
+
+	PerlBackend *perl_back = (PerlBackend *) be->be_private;
+
+	ldap_pvt_thread_mutex_lock( &perl_interpreter_mutex );
+
+	{
+		dSP; ENTER; SAVETMPS;
+
+		PUSHMARK(sp);
+		XPUSHs( perl_back->pb_obj_ref );
+
+		PUTBACK;
+
+#ifdef PERL_IS_5_6
+		count = call_method("init", G_SCALAR);
+#else
+		count = perl_call_method("init", G_SCALAR);
+#endif
+
+		SPAGAIN;
+
+		if (count != 1) {
+			croak("Big trouble in perl_back_db_open\n");
+		}
+
+		return_code = POPi;
+
+		PUTBACK; FREETMPS; LEAVE;
+	}
+
+	ldap_pvt_thread_mutex_unlock( &perl_interpreter_mutex );
+
+	return return_code;
+}
+
 
 static void
-perl_back_xs_init()
+perl_back_xs_init(PERL_BACK_XS_INIT_PARAMS)
 {
-    char *file = __FILE__;
-    dXSUB_SYS;
-        newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
+	char *file = __FILE__;
+	dXSUB_SYS;
+	newXS("DynaLoader::boot_DynaLoader", boot_DynaLoader, file);
 }
