@@ -1,6 +1,11 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/modify.c,v 1.8 2002/02/02 09:10:35 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-perl/modify.c,v 1.15 2003/05/25 01:56:59 kurt Exp $ */
+/*
+ * Copyright 1999-2003 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 /*
  *	 Copyright 1999, John C. Quillan, All rights reserved.
+ *	 Portions Copyright 2002, myinternet Limited. All rights reserved.
  *
  *	 Redistribution and use in source and binary forms are permitted only
  *	 as authorized by the OpenLDAP Public License.	A copy of this
@@ -11,34 +16,27 @@
 #include "portable.h"
 
 #include <stdio.h>
-/* #include <ac/types.h>
-	#include <ac/socket.h>
-*/
+
+#include "slap.h"
+#ifdef HAVE_WIN32_ASPERL
+#include "asperl_undefs.h"
+#endif
 
 #include <EXTERN.h>
 #include <perl.h>
 
-#include "slap.h"
 #include "perl_back.h"
 
 int
 perl_back_modify(
-	Backend	*be,
-	Connection	*conn,
 	Operation	*op,
-	struct berval 	*dn,
-	struct berval 	*ndn,
-	Modifications	*modlist
-)
+	SlapReply	*rs )
 {
-	char test[500];
-	int return_code;
+	PerlBackend *perl_back = (PerlBackend *)op->o_bd->be_private;
+	Modifications *modlist = op->orm_modlist;
 	int count;
 	int i;
-	int err = 0;
-	char *matched = NULL, *info = NULL;
 
-	PerlBackend *perl_back = (PerlBackend *)be->be_private;
 
 	ldap_pvt_thread_mutex_lock( &perl_interpreter_mutex );	
 
@@ -47,7 +45,7 @@ perl_back_modify(
 		
 		PUSHMARK(sp);
 		XPUSHs( perl_back->pb_obj_ref );
-		XPUSHs(sv_2mortal(newSVpv( dn->bv_val , 0)));
+		XPUSHs(sv_2mortal(newSVpv( op->o_req_dn.bv_val , 0)));
 
 		for (; modlist != NULL; modlist = modlist->sml_next ) {
 			Modification *mods = &modlist->sml_mod;
@@ -67,7 +65,7 @@ perl_back_modify(
 			}
 
 			
-			XPUSHs(sv_2mortal(newSVpv( mods->sm_type.bv_val, 0 )));
+			XPUSHs(sv_2mortal(newSVpv( mods->sm_desc->ad_cname.bv_val, 0 )));
 
 			for ( i = 0;
 				mods->sm_bvalues != NULL && mods->sm_bvalues[i].bv_val != NULL;
@@ -79,7 +77,11 @@ perl_back_modify(
 
 		PUTBACK;
 
+#ifdef PERL_IS_5_6
+		count = call_method("modify", G_SCALAR);
+#else
 		count = perl_call_method("modify", G_SCALAR);
+#endif
 
 		SPAGAIN;
 
@@ -87,21 +89,14 @@ perl_back_modify(
 			croak("Big trouble in back_modify\n");
 		}
 							 
-		return_code = POPi;
+		rs->sr_err = POPi;
 
 		PUTBACK; FREETMPS; LEAVE;
 	}
 
 	ldap_pvt_thread_mutex_unlock( &perl_interpreter_mutex );
 
-	if( return_code != 0 ) {
-		send_ldap_result( conn, op, LDAP_OPERATIONS_ERROR,
-			NULL, NULL, NULL, NULL );
-
-	} else {
-		send_ldap_result( conn, op, LDAP_SUCCESS,
-			NULL, NULL, NULL, NULL );
-	}
+	send_ldap_result( op, rs );
 
 	Debug( LDAP_DEBUG_ANY, "Perl MODIFY\n", 0, 0, 0 );
 	return( 0 );
