@@ -1,4 +1,9 @@
 /* init.c - initialize shell backend */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-shell/init.c,v 1.27 2003/03/26 10:50:05 hyc Exp $ */
+/*
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 
 #include "portable.h"
 
@@ -10,15 +15,16 @@
 #include "shell.h"
 
 #ifdef SLAPD_SHELL_DYNAMIC
-#include <gmodule.h>
 
-G_MODULE_EXPORT void init_module(int argc, char *argv[]) {
-   BackendInfo bi;
+int back_shell_LTX_init_module(int argc, char *argv[]) {
+    BackendInfo bi;
 
-   bi.bi_type = "shell";
-   bi.bi_init = shell_back_initialize;
+    memset( &bi, '\0', sizeof(bi) );
+    bi.bi_type = "shell";
+    bi.bi_init = shell_back_initialize;
 
-   backend_add(&bi);
+    backend_add(&bi);
+    return 0;
 }
 
 #endif /* SLAPD_SHELL_DYNAMIC */
@@ -49,7 +55,9 @@ shell_back_initialize(
 	bi->bi_op_delete = shell_back_delete;
 	bi->bi_op_abandon = shell_back_abandon;
 
-	bi->bi_acl_group = 0;
+	bi->bi_extended = 0;
+
+	bi->bi_chk_referrals = 0;
 
 	bi->bi_connection_init = 0;
 	bi->bi_connection_destroy = 0;
