@@ -1,6 +1,6 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.150 2002/02/12 03:50:40 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/backend.c,v 1.206 2003/05/22 06:57:13 hyc Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 /* backend.c - routines for dealing with back-end databases */
@@ -12,11 +12,17 @@
 
 #include <ac/string.h>
 #include <ac/socket.h>
-
 #include <sys/stat.h>
 
 #include "slap.h"
 #include "lutil.h"
+#include "lber_pvt.h"
+
+#include "ldap_rq.h"
+
+#ifdef LDAP_SLAPI
+#include "slapi.h"
+#endif
 
 /*
  * If a module is configured as dynamic, its header should not
@@ -32,6 +38,9 @@
 #if defined(SLAPD_DNSSRV) && !defined(SLAPD_DNSSRV_DYNAMIC)
 #include "back-dnssrv/external.h"
 #endif
+#if defined(SLAPD_HDB) && !defined(SLAPD_HDB_DYNAMIC)
+#include "back-hdb/external.h"
+#endif
 #if defined(SLAPD_LDAP) && !defined(SLAPD_LDAP_DYNAMIC)
 #include "back-ldap/external.h"
 #endif
@@ -44,6 +53,9 @@
 #if defined(SLAPD_MONITOR) && !defined(SLAPD_MONITOR_DYNAMIC)
 #include "back-monitor/external.h"
 #endif
+#if defined(SLAPD_NULL) && !defined(SLAPD_NULL_DYNAMIC)
+#include "back-null/external.h"
+#endif
 #if defined(SLAPD_PASSWD) && !defined(SLAPD_PASSWD_DYNAMIC)
 #include "back-passwd/external.h"
 #endif
@@ -70,6 +82,9 @@ static BackendInfo binfo[] = {
 #if defined(SLAPD_DNSSRV) && !defined(SLAPD_DNSSRV_DYNAMIC)
 	{"dnssrv",	dnssrv_back_initialize},
 #endif
+#if defined(SLAPD_HDB) && !defined(SLAPD_HDB_DYNAMIC)
+	{"hdb",	hdb_initialize},
+#endif
 #if defined(SLAPD_LDAP) && !defined(SLAPD_LDAP_DYNAMIC)
 	{"ldap",	ldap_back_initialize},
 #endif
@@ -82,6 +97,9 @@ static BackendInfo binfo[] = {
 #if defined(SLAPD_MONITOR) && !defined(SLAPD_MONITOR_DYNAMIC)
 	{"monitor",	monitor_back_initialize},
 #endif
+#if defined(SLAPD_NULL) && !defined(SLAPD_NULL_DYNAMIC)
+	{"null",	null_back_initialize},
+#endif
 #if defined(SLAPD_PASSWD) && !defined(SLAPD_PASSWD_DYNAMIC)
 	{"passwd",	passwd_back_initialize},
 #endif
@@ -110,15 +128,24 @@ BackendInfo	*backendInfo = NULL;
 int			nBackendDB = 0; 
 BackendDB	*backendDB = NULL;
 
+#ifdef LDAP_SYNCREPL
+ldap_pvt_thread_pool_t	syncrepl_pool;
+int			syncrepl_pool_max = SLAP_MAX_SYNCREPL_THREADS;
+#endif
+
 int backend_init(void)
 {
 	int rc = -1;
 
+#ifdef LDAP_SYNCREPL
+        ldap_pvt_thread_pool_init( &syncrepl_pool, syncrepl_pool_max, 0 );
+#endif
+
 	if((nBackendInfo != 0) || (backendInfo != NULL)) {
 		/* already initialized */
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-			   "backend_init:  backend already initialized\n" ));
+		LDAP_LOG( BACKEND, ERR, 
+			"backend_init:  backend already initialized\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"backend_init: already initialized.\n", 0, 0, 0 );
@@ -134,9 +161,9 @@ int backend_init(void)
 
 		if(rc != 0) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
+			LDAP_LOG( BACKEND, INFO, 
 				"backend_init:  initialized for type \"%s\"\n",
-				binfo[nBackendInfo].bi_type ));
+				binfo[nBackendInfo].bi_type, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"backend_init: initialized for type \"%s\"\n",
@@ -166,8 +193,7 @@ int backend_init(void)
 #else
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
-		"backend_init: failed\n" ));
+	LDAP_LOG( BACKEND, ERR, "backend_init: failed\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY,
 		"backend_init: failed\n",
@@ -184,9 +210,9 @@ int backend_add(BackendInfo *aBackendInfo)
 
    if ((rc = aBackendInfo->bi_init(aBackendInfo)) != 0) {
 #ifdef NEW_LOGGING
-       LDAP_LOG(( "backend", LDAP_LEVEL_ERR,
+       	LDAP_LOG( BACKEND, ERR, 
                   "backend_add:  initialization for type \"%s\" failed\n",
-                  aBackendInfo->bi_type ));
+                  aBackendInfo->bi_type, 0, 0 );
 #else
       Debug( LDAP_DEBUG_ANY,
 	     "backend_add: initialization for type \"%s\" failed\n",
@@ -222,12 +248,15 @@ int backend_startup(Backend *be)
 	int i;
 	int rc = 0;
 
+#ifdef LDAP_SYNCREPL
+	init_syncrepl();
+#endif
+
 	if( ! ( nBackendDB > 0 ) ) {
 		/* no databases */
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_INFO,
-			   "backend_startup: %d databases to startup. \n",
-			   nBackendDB ));
+		LDAP_LOG( BACKEND, INFO, 
+			"backend_startup: %d databases to startup. \n", nBackendDB, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 			"backend_startup: %d databases to startup.\n",
@@ -239,46 +268,42 @@ int backend_startup(Backend *be)
 	if(be != NULL) {
 		/* startup a specific backend database */
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "backend", LDAP_LEVEL_DETAIL1,
-			   "backend_startup:  starting \"%s\"\n",
-			   be->be_suffix[0]->bv_val ));
+		LDAP_LOG( BACKEND, DETAIL1, "backend_startup:  starting \"%s\"\n",
+			   be->be_suffix[0].bv_val, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_TRACE,
 			"backend_startup: starting \"%s\"\n",
-			be->be_suffix[0]->bv_val, 0, 0 );
+			be->be_suffix[0].bv_val, 0, 0 );
 #endif
 
 		if ( be->bd_info->bi_open ) {
 			rc = be->bd_info->bi_open( be->bd_info );
-		}
-
-		if(rc != 0) {
+			if ( rc != 0 ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_CRIT,
-				   "backend_startup: bi_open failed!\n" ));
+				LDAP_LOG( BACKEND, CRIT, "backend_startup: bi_open failed!\n", 0, 0, 0 );
 #else
-			Debug( LDAP_DEBUG_ANY,
-				"backend_startup: bi_open failed!\n",
-				0, 0, 0 );
+				Debug( LDAP_DEBUG_ANY,
+					"backend_startup: bi_open failed!\n",
+					0, 0, 0 );
 #endif
 
-			return rc;
+				return rc;
+			}
 		}
 
 		if ( be->bd_info->bi_db_open ) {
 			rc = be->bd_info->bi_db_open( be );
-		}
-
-		if(rc != 0) {
+			if ( rc != 0 ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_CRIT,
-				"backend_startup: bi_db_open failed! (%d)\n", rc ));
+				LDAP_LOG( BACKEND, CRIT, 
+					"backend_startup: bi_db_open failed! (%d)\n", rc, 0, 0 );
 #else
-			Debug( LDAP_DEBUG_ANY,
-				"backend_startup: bi_db_open failed! (%d)\n",
-				rc, 0, 0 );
+				Debug( LDAP_DEBUG_ANY,
+					"backend_startup: bi_db_open failed! (%d)\n",
+					rc, 0, 0 );
 #endif
-			return rc;
+				return rc;
+			}
 		}
 
 		return rc;
@@ -294,21 +319,26 @@ int backend_startup(Backend *be)
 		if( backendInfo[i].bi_open ) {
 			rc = backendInfo[i].bi_open(
 				&backendInfo[i] );
-		}
-
-		if(rc != 0) {
+			if ( rc != 0 ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_CRIT,
-				   "backend_startup: bi_open %d failed!\n", i ));
+				LDAP_LOG( BACKEND, CRIT, 
+					"backend_startup: bi_open %d failed!\n", i, 0, 0 );
 #else
-			Debug( LDAP_DEBUG_ANY,
-				"backend_startup: bi_open %d failed!\n",
-				i, 0, 0 );
+				Debug( LDAP_DEBUG_ANY,
+					"backend_startup: bi_open %d failed!\n",
+					i, 0, 0 );
 #endif
-			return rc;
+				return rc;
+			}
 		}
 	}
 
+#ifdef LDAP_SYNCREPL
+	ldap_pvt_thread_mutex_init( &syncrepl_rq.rq_mutex );
+	LDAP_STAILQ_INIT( &syncrepl_rq.task_list );
+	LDAP_STAILQ_INIT( &syncrepl_rq.run_list );
+#endif
+
 	/* open each backend database */
 	for( i = 0; i < nBackendDB; i++ ) {
 		/* append global access controls */
@@ -317,20 +347,28 @@ int backend_startup(Backend *be)
 		if ( backendDB[i].bd_info->bi_db_open ) {
 			rc = backendDB[i].bd_info->bi_db_open(
 				&backendDB[i] );
+			if ( rc != 0 ) {
+#ifdef NEW_LOGGING
+				LDAP_LOG( BACKEND, CRIT, 
+					"backend_startup: bi_db_open(%d) failed! (%d)\n", i, rc, 0 );
+#else
+				Debug( LDAP_DEBUG_ANY,
+					"backend_startup: bi_db_open(%d) failed! (%d)\n",
+					i, rc, 0 );
+#endif
+				return rc;
+			}
 		}
 
-		if(rc != 0) {
-#ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_CRIT,
-				"backend_startup: bi_db_open(%d) failed! (%d)\n",
-				i, rc ));
-#else
-			Debug( LDAP_DEBUG_ANY,
-				"backend_startup: bi_db_open(%d) failed! (%d)\n",
-				i, rc, 0 );
-#endif
-			return rc;
+#ifdef LDAP_SYNCREPL
+		if ( backendDB[i].syncinfo != NULL ) {
+			syncinfo_t *si = ( syncinfo_t * ) backendDB[i].syncinfo;
+			ldap_pvt_thread_mutex_lock( &syncrepl_rq.rq_mutex );
+			ldap_pvt_runqueue_insert( &syncrepl_rq, si->interval,
+							do_syncrepl, (void *) &backendDB[i] );
+			ldap_pvt_thread_mutex_unlock( &syncrepl_rq.rq_mutex );
 		}
+#endif
 	}
 
 	return rc;
@@ -381,9 +419,9 @@ int backend_shutdown( Backend *be )
 
 		if(rc != 0) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "backend", LDAP_LEVEL_NOTICE,
-				   "backend_shutdown: bi_close %s failed!\n",
-				   backendDB[i].be_type ));
+			LDAP_LOG( BACKEND, NOTICE, 
+				"backend_shutdown: bi_close %s failed!\n",
+				backendDB[i].be_type, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 				"backend_close: bi_close %s failed!\n",
@@ -413,13 +451,17 @@ int backend_destroy(void)
 	int i;
 	BackendDB *bd;
 
+#ifdef LDAP_SYNCREPL
+        ldap_pvt_thread_pool_destroy( &syncrepl_pool, 1 );
+#endif
+
 	/* destroy each backend database */
 	for( i = 0, bd = backendDB; i < nBackendDB; i++, bd++ ) {
 		if ( bd->bd_info->bi_db_destroy ) {
 			bd->bd_info->bi_db_destroy( bd );
 		}
-		ber_bvecfree( bd->be_suffix );
-		ber_bvecfree( bd->be_nsuffix );
+		ber_bvarray_free( bd->be_suffix );
+		ber_bvarray_free( bd->be_nsuffix );
 		if ( bd->be_rootdn.bv_val ) free( bd->be_rootdn.bv_val );
 		if ( bd->be_rootndn.bv_val ) free( bd->be_rootndn.bv_val );
 		if ( bd->be_rootpw.bv_val ) free( bd->be_rootpw.bv_val );
@@ -492,6 +534,10 @@ backend_db_init(
 	be->be_requires = global_requires;
 	be->be_ssf_set = global_ssf_set;
 
+#ifdef LDAP_SYNCREPL
+        be->syncinfo = NULL;
+#endif
+
  	/* assign a default depth limit for alias deref */
 	be->be_max_deref_depth = SLAPD_DEFAULT_MAXDEREFDEPTH; 
 
@@ -533,15 +579,15 @@ select_backend(
 
 	for ( i = 0; i < nbackends; i++ ) {
 		for ( j = 0; backends[i].be_nsuffix != NULL &&
-		    backends[i].be_nsuffix[j] != NULL; j++ )
+		    backends[i].be_nsuffix[j].bv_val != NULL; j++ )
 		{
-			if (( backends[i].be_flags & SLAP_BFLAG_GLUE_SUBORDINATE )
+			if ( ( SLAP_GLUE_SUBORDINATE( &backends[i] ) )
 				&& noSubs )
 			{
 			  	continue;
 			}
 
-			len = backends[i].be_nsuffix[j]->bv_len;
+			len = backends[i].be_nsuffix[j].bv_len;
 
 			if ( len > dnlen ) {
 				/* suffix is longer than DN */
@@ -558,7 +604,7 @@ select_backend(
 				continue;
 			}
 
-			if ( strcmp( backends[i].be_nsuffix[j]->bv_val,
+			if ( strcmp( backends[i].be_nsuffix[j].bv_val,
 				&dn->bv_val[dnlen-len] ) == 0 )
 			{
 				if( be == NULL ) {
@@ -586,8 +632,8 @@ be_issuffix(
 {
 	int	i;
 
-	for ( i = 0; be->be_nsuffix != NULL && be->be_nsuffix[i] != NULL; i++ ) {
-		if ( ber_bvcmp( be->be_nsuffix[i], bvsuffix ) == 0 ) {
+	for ( i = 0; be->be_nsuffix != NULL && be->be_nsuffix[i].bv_val != NULL; i++ ) {
+		if ( bvmatch( &be->be_nsuffix[i], bvsuffix ) ) {
 			return( 1 );
 		}
 	}
@@ -630,29 +676,27 @@ be_root_dn( Backend *be )
 }
 
 int
-be_isroot_pw( Backend *be,
-	Connection *conn,
-	struct berval *ndn,
-	struct berval *cred )
+be_isroot_pw( Operation *op )
 {
 	int result;
+	char *errmsg;
 
-	if ( ! be_isroot( be, ndn ) ) {
+	if ( ! be_isroot( op->o_bd, &op->o_req_ndn ) ) {
 		return 0;
 	}
 
-	if( be->be_rootpw.bv_len == 0 ) {
+	if( op->o_bd->be_rootpw.bv_len == 0 ) {
 		return 0;
 	}
 
 #if defined( SLAPD_CRYPT ) || defined( SLAPD_SPASSWD )
 	ldap_pvt_thread_mutex_lock( &passwd_mutex );
 #ifdef SLAPD_SPASSWD
-	lutil_passwd_sasl_conn = conn->c_sasl_context;
+	lutil_passwd_sasl_conn = op->o_conn->c_sasl_authctx;
 #endif
 #endif
 
-	result = lutil_passwd( &be->be_rootpw, cred, NULL );
+	result = lutil_passwd( &op->o_bd->be_rootpw, &op->orb_cred, NULL, NULL );
 
 #if defined( SLAPD_CRYPT ) || defined( SLAPD_SPASSWD )
 #ifdef SLAPD_SPASSWD
@@ -666,15 +710,13 @@ be_isroot_pw( Backend *be,
 
 int
 be_entry_release_rw(
-	BackendDB *be,
-	Connection *conn,
 	Operation *op,
 	Entry *e,
 	int rw )
 {
-	if ( be->be_release ) {
+	if ( op->o_bd->be_release ) {
 		/* free and release entry from backend */
-		return be->be_release( be, conn, op, e, rw );
+		return op->o_bd->be_release( op, e, rw );
 	} else {
 		/* free entry */
 		entry_free( e );
@@ -683,17 +725,54 @@ be_entry_release_rw(
 }
 
 int
-backend_unbind(
-	Connection   *conn,
-	Operation    *op
-)
+backend_unbind( Operation *op, SlapReply *rs )
 {
-	int	i;
+	int		i;
+#if defined( LDAP_SLAPI )
+	Slapi_PBlock *pb = op->o_pb;
+
+	int     rc;
+	slapi_x_pblock_set_operation( pb, op );
+#endif /* defined( LDAP_SLAPI ) */
 
 	for ( i = 0; i < nbackends; i++ ) {
-		if ( backends[i].be_unbind ) {
-			(*backends[i].be_unbind)( &backends[i], conn, op );
+#if defined( LDAP_SLAPI )
+		slapi_pblock_set( pb, SLAPI_BACKEND, (void *)&backends[i] );
+		rc = doPluginFNs( &backends[i], SLAPI_PLUGIN_PRE_UNBIND_FN,
+				(Slapi_PBlock *)pb );
+		if ( rc < 0 ) {
+			/*
+			 * A preoperation plugin failure will abort the
+			 * entire operation.
+			 */
+#ifdef NEW_LOGGING
+			LDAP_LOG( OPERATION, INFO, "do_bind: Unbind preoperation plugin "
+					"failed\n", 0, 0, 0);
+#else
+			Debug(LDAP_DEBUG_TRACE, "do_bind: Unbind preoperation plugin "
+					"failed.\n", 0, 0, 0);
+#endif
+			return 0;
 		}
+#endif /* defined( LDAP_SLAPI ) */
+
+		if ( backends[i].be_unbind ) {
+			op->o_bd = &backends[i];
+			(*backends[i].be_unbind)( op, rs );
+		}
+
+#if defined( LDAP_SLAPI )
+		if ( doPluginFNs( &backends[i], SLAPI_PLUGIN_POST_UNBIND_FN,
+				(Slapi_PBlock *)pb ) < 0 ) {
+#ifdef NEW_LOGGING
+			LDAP_LOG( OPERATION, INFO, "do_unbind: Unbind postoperation plugins "
+					"failed\n", 0, 0, 0);
+#else
+			Debug(LDAP_DEBUG_TRACE, "do_unbind: Unbind postoperation plugins "
+					"failed.\n", 0, 0, 0);
+#endif
+		}
+#endif /* defined( LDAP_SLAPI ) */
 	}
 
 	return 0;
@@ -733,54 +812,49 @@ backend_connection_destroy(
 
 static int
 backend_check_controls(
-	Backend *be,
-	Connection *conn,
 	Operation *op,
-	const char **text )
+	SlapReply *rs )
 {
-	LDAPControl **ctrls;
-	ctrls = op->o_ctrls;
-	if( ctrls == NULL ) {
-		return LDAP_SUCCESS;
-	}
+	LDAPControl **ctrls = op->o_ctrls;
+	rs->sr_err = LDAP_SUCCESS;
 
-	for( ; *ctrls != NULL ; ctrls++ ) {
-		if( (*ctrls)->ldctl_iscritical &&
-			!charray_inlist( be->be_controls, (*ctrls)->ldctl_oid ) )
-		{
-			*text = "control unavailable in context";
-			return LDAP_UNAVAILABLE_CRITICAL_EXTENSION;
+	if( ctrls ) {
+		for( ; *ctrls != NULL ; ctrls++ ) {
+			if( (*ctrls)->ldctl_iscritical &&
+				!ldap_charray_inlist( op->o_bd->be_controls, (*ctrls)->ldctl_oid ) )
+			{
+				rs->sr_text = "control unavailable in context";
+				rs->sr_err = LDAP_UNAVAILABLE_CRITICAL_EXTENSION;
+				break;
+			}
 		}
 	}
 
-	return LDAP_SUCCESS;
+	return rs->sr_err;
 }
 
 int
 backend_check_restrictions(
-	Backend *be,
-	Connection *conn,
 	Operation *op,
-	const void *opdata,
-	const char **text )
+	SlapReply *rs,
+	struct berval *opdata )
 {
-	int rc;
 	slap_mask_t restrictops;
 	slap_mask_t requires;
 	slap_mask_t opflag;
 	slap_ssf_set_t *ssf;
 	int updateop = 0;
+	int starttls = 0;
+	int session = 0;
 
-	if( be ) {
-		rc = backend_check_controls( be, conn, op, text );
-
-		if( rc != LDAP_SUCCESS ) {
-			return rc;
+	if( op->o_bd ) {
+		if ( backend_check_controls( op, rs ) != LDAP_SUCCESS ) {
+			return rs->sr_err;
 		}
 
-		restrictops = be->be_restrictops;
-		requires = be->be_requires;
-		ssf = &be->be_ssf_set;
+		restrictops = op->o_bd->be_restrictops;
+		requires = op->o_bd->be_requires;
+		ssf = &op->o_bd->be_ssf_set;
 
 	} else {
 		restrictops = global_restrictops;
@@ -795,6 +869,7 @@ backend_check_restrictions(
 		break;
 	case LDAP_REQ_BIND:
 		opflag = SLAP_RESTRICT_OP_BIND;
+		session++;
 		break;
 	case LDAP_REQ_COMPARE:
 		opflag = SLAP_RESTRICT_OP_COMPARE;
@@ -805,7 +880,41 @@ backend_check_restrictions(
 		break;
 	case LDAP_REQ_EXTENDED:
 		opflag = SLAP_RESTRICT_OP_EXTENDED;
+
+		if( !opdata ) {
+			/* treat unspecified as a modify */
+			opflag = SLAP_RESTRICT_OP_MODIFY;
+			updateop++;
+			break;
+		}
+
+		{
+			if( bvmatch( opdata, &slap_EXOP_START_TLS ) ) {
+				session++;
+				starttls++;
+				break;
+			}
+		}
+
+		{
+			if( bvmatch( opdata, &slap_EXOP_WHOAMI ) ) {
+				break;
+			}
+		}
+
+#ifdef LDAP_EXOP_X_CANCEL
+		{
+			if ( bvmatch( opdata, &slap_EXOP_CANCEL ) ) {
+				break;
+			}
+		}
+#endif
+
+		/* treat everything else as a modify */
+		opflag = SLAP_RESTRICT_OP_MODIFY;
+		updateop++;
 		break;
+
 	case LDAP_REQ_MODIFY:
 		updateop++;
 		opflag = SLAP_RESTRICT_OP_MODIFY;
@@ -818,166 +927,222 @@ backend_check_restrictions(
 		opflag = SLAP_RESTRICT_OP_SEARCH;
 		break;
 	case LDAP_REQ_UNBIND:
+		session++;
 		opflag = 0;
 		break;
 	default:
-		*text = "restrict operations internal error";
-		return LDAP_OTHER;
+		rs->sr_text = "restrict operations internal error";
+		rs->sr_err = LDAP_OTHER;
+		return rs->sr_err;
 	}
 
-	if ( op->o_tag != LDAP_REQ_EXTENDED
-		|| strcmp( (const char *) opdata, LDAP_EXOP_START_TLS ) )
-	{
+	if ( !starttls ) {
 		/* these checks don't apply to StartTLS */
 
-		if( op->o_tag == LDAP_REQ_EXTENDED ) {
-			/* threat other extended operations as update ops */
-			updateop++;
-		}
-
+		rs->sr_err = LDAP_CONFIDENTIALITY_REQUIRED;
 		if( op->o_transport_ssf < ssf->sss_transport ) {
-			*text = "transport confidentiality required";
-			return LDAP_CONFIDENTIALITY_REQUIRED;
+			rs->sr_text = "transport confidentiality required";
+			return rs->sr_err;
 		}
 
 		if( op->o_tls_ssf < ssf->sss_tls ) {
-			*text = "TLS confidentiality required";
-			return LDAP_CONFIDENTIALITY_REQUIRED;
+			rs->sr_text = "TLS confidentiality required";
+			return rs->sr_err;
+		}
+
+
+		if( op->o_tag == LDAP_REQ_BIND && opdata == NULL ) {
+			/* simple bind specific check */
+			if( op->o_ssf < ssf->sss_simple_bind ) {
+				rs->sr_text = "confidentiality required";
+				return rs->sr_err;
+			}
 		}
 
 		if( op->o_tag != LDAP_REQ_BIND || opdata == NULL ) {
 			/* these checks don't apply to SASL bind */
 
 			if( op->o_sasl_ssf < ssf->sss_sasl ) {
-				*text = "SASL confidentiality required";
-				return LDAP_CONFIDENTIALITY_REQUIRED;
+				rs->sr_text = "SASL confidentiality required";
+				return rs->sr_err;
 			}
 
 			if( op->o_ssf < ssf->sss_ssf ) {
-				*text = "confidentiality required";
-				return LDAP_CONFIDENTIALITY_REQUIRED;
+				rs->sr_text = "confidentiality required";
+				return rs->sr_err;
 			}
 		}
 
 		if( updateop ) {
 			if( op->o_transport_ssf < ssf->sss_update_transport ) {
-				*text = "transport update confidentiality required";
-				return LDAP_CONFIDENTIALITY_REQUIRED;
+				rs->sr_text = "transport update confidentiality required";
+				return rs->sr_err;
 			}
 
 			if( op->o_tls_ssf < ssf->sss_update_tls ) {
-				*text = "TLS update confidentiality required";
-				return LDAP_CONFIDENTIALITY_REQUIRED;
+				rs->sr_text = "TLS update confidentiality required";
+				return rs->sr_err;
 			}
 
 			if( op->o_sasl_ssf < ssf->sss_update_sasl ) {
-				*text = "SASL update confidentiality required";
-				return LDAP_CONFIDENTIALITY_REQUIRED;
+				rs->sr_text = "SASL update confidentiality required";
+				return rs->sr_err;
 			}
 
 			if( op->o_ssf < ssf->sss_update_ssf ) {
-				*text = "update confidentiality required";
-				return LDAP_CONFIDENTIALITY_REQUIRED;
+				rs->sr_text = "update confidentiality required";
+				return rs->sr_err;
 			}
 
-			if( op->o_ndn.bv_len == 0 ) {
-				*text = "modifications require authentication";
-				return LDAP_OPERATIONS_ERROR;
+			if( !( global_allows & SLAP_ALLOW_UPDATE_ANON ) &&
+				op->o_ndn.bv_len == 0 )
+			{
+				rs->sr_text = "modifications require authentication";
+				rs->sr_err = LDAP_STRONG_AUTH_REQUIRED;
+				return rs->sr_err;
 			}
+
+#ifdef SLAP_X_LISTENER_MOD
+			if ( op->o_conn->c_listener && ! ( op->o_conn->c_listener->sl_perms & ( op->o_ndn.bv_len > 0 ? S_IWUSR : S_IWOTH ) ) ) {
+				/* no "w" mode means readonly */
+				rs->sr_text = "modifications not allowed on this listener";
+				rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
+				return rs->sr_err;
+			}
+#endif /* SLAP_X_LISTENER_MOD */
 		}
 	}
 
-	if ( op->o_tag != LDAP_REQ_BIND && ( op->o_tag != LDAP_REQ_EXTENDED ||
-		strcmp( (const char *) opdata, LDAP_EXOP_START_TLS ) ) )
-	{
-		/* these checks don't apply to Bind or StartTLS */
+	if ( !session ) {
+		/* these checks don't apply to Bind, StartTLS, or Unbind */
 
 		if( requires & SLAP_REQUIRE_STRONG ) {
 			/* should check mechanism */
-			if( op->o_authmech.bv_len == 0 || op->o_dn.bv_len == 0 )
+			if( ( op->o_transport_ssf < ssf->sss_transport
+				&& op->o_authmech.bv_len == 0 ) || op->o_dn.bv_len == 0 )
 			{
-				*text = "strong authentication required";
-				return LDAP_STRONG_AUTH_REQUIRED;
+				rs->sr_text = "strong authentication required";
+				rs->sr_err = LDAP_STRONG_AUTH_REQUIRED;
+				return rs->sr_err;
 			}
 		}
 
 		if( requires & SLAP_REQUIRE_SASL ) {
-			if( op->o_authmech.bv_len == 0 || op->o_dn.bv_len == 0 )
-			{
-				*text = "SASL authentication required";
-				return LDAP_STRONG_AUTH_REQUIRED;
+			if( op->o_authmech.bv_len == 0 || op->o_dn.bv_len == 0 ) {
+				rs->sr_text = "SASL authentication required";
+				rs->sr_err = LDAP_STRONG_AUTH_REQUIRED;
+				return rs->sr_err;
 			}
 		}
 			
 		if( requires & SLAP_REQUIRE_AUTHC ) {
 			if( op->o_dn.bv_len == 0 ) {
-				*text = "authentication required";
-				return LDAP_UNWILLING_TO_PERFORM;
+				rs->sr_text = "authentication required";
+				rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
+				return rs->sr_err;
 			}
 		}
 
 		if( requires & SLAP_REQUIRE_BIND ) {
 			int version;
-			ldap_pvt_thread_mutex_lock( &conn->c_mutex );
-			version = conn->c_protocol;
-			ldap_pvt_thread_mutex_unlock( &conn->c_mutex );
+			ldap_pvt_thread_mutex_lock( &op->o_conn->c_mutex );
+			version = op->o_conn->c_protocol;
+			ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
 
 			if( !version ) {
 				/* no bind has occurred */
-				*text = "BIND required";
-				return LDAP_OPERATIONS_ERROR;
+				rs->sr_text = "BIND required";
+				rs->sr_err = LDAP_OPERATIONS_ERROR;
+				return rs->sr_err;
 			}
 		}
 
 		if( requires & SLAP_REQUIRE_LDAP_V3 ) {
 			if( op->o_protocol < LDAP_VERSION3 ) {
 				/* no bind has occurred */
-				*text = "operation restricted to LDAPv3 clients";
-				return LDAP_OPERATIONS_ERROR;
+				rs->sr_text = "operation restricted to LDAPv3 clients";
+				rs->sr_err = LDAP_OPERATIONS_ERROR;
+				return rs->sr_err;
 			}
 		}
+
+#ifdef SLAP_X_LISTENER_MOD
+		if ( !starttls && op->o_dn.bv_len == 0 ) {
+			if ( op->o_conn->c_listener && ! ( op->o_conn->c_listener->sl_perms & S_IXOTH ) ) {
+				/* no "x" mode means bind required */
+				rs->sr_text = "bind required on this listener";
+				rs->sr_err = LDAP_STRONG_AUTH_REQUIRED;
+				return rs->sr_err;
+			}
+		}
+
+		if ( !starttls && !updateop ) {
+			if ( op->o_conn->c_listener && ! ( op->o_conn->c_listener->sl_perms & ( op->o_dn.bv_len > 0 ? S_IRUSR : S_IROTH ) ) ) {
+				/* no "r" mode means no read */
+				rs->sr_text = "read not allowed on this listener";
+				rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
+				return rs->sr_err;
+			}
+		}
+#endif /* SLAP_X_LISTENER_MOD */
+
 	}
 
 	if( restrictops & opflag ) {
 		if( restrictops == SLAP_RESTRICT_OP_READS ) {
-			*text = "read operations restricted";
+			rs->sr_text = "read operations restricted";
 		} else {
-			*text = "operation restricted";
+			rs->sr_text = "operation restricted";
 		}
-		return LDAP_UNWILLING_TO_PERFORM;
+		rs->sr_err = LDAP_UNWILLING_TO_PERFORM;
+		return rs->sr_err;
  	}
 
-	return LDAP_SUCCESS;
+	rs->sr_err = LDAP_SUCCESS;
+	return rs->sr_err;
 }
 
-int backend_check_referrals(
-	Backend *be,
-	Connection *conn,
-	Operation *op,
-	struct berval *dn,
-	struct berval *ndn )
+int backend_check_referrals( Operation *op, SlapReply *rs )
 {
-	int rc = LDAP_SUCCESS;
+	rs->sr_err = LDAP_SUCCESS;
 
-	if( be->be_chk_referrals ) {
-		const char *text;
+	if( op->o_bd->be_chk_referrals ) {
+		rs->sr_err = op->o_bd->be_chk_referrals( op, rs );
 
-		rc = be->be_chk_referrals( be,
-			conn, op, dn, ndn, &text );
-
-		if( rc != LDAP_SUCCESS && rc != LDAP_REFERRAL ) {
-			send_ldap_result( conn, op, rc,
-				NULL, text, NULL, NULL );
+		if( rs->sr_err != LDAP_SUCCESS && rs->sr_err != LDAP_REFERRAL ) {
+			send_ldap_result( op, rs );
 		}
 	}
 
+	return rs->sr_err;
+}
+
+int
+be_entry_get_rw(
+	Operation *op,
+	struct berval *ndn,
+	ObjectClass *oc,
+	AttributeDescription *at,
+	int rw,
+	Entry **e )
+{
+	int rc;
+
+	*e = NULL;
+
+	if (op->o_bd == NULL) {
+		rc = LDAP_NO_SUCH_OBJECT;
+	} else if ( op->o_bd->be_fetch ) {
+		rc = ( op->o_bd->be_fetch )( op, ndn,
+			oc, at, rw, e );
+	} else {
+		rc = LDAP_UNWILLING_TO_PERFORM;
+	}
 	return rc;
 }
 
 int 
 backend_group(
-	Backend	*be,
-	Connection *conn,
 	Operation *op,
 	Entry	*target,
 	struct berval *gr_ndn,
@@ -986,67 +1151,75 @@ backend_group(
 	AttributeDescription *group_at
 )
 {
+	Entry *e;
+	Attribute *a;
+	int rc;
 	GroupAssertion *g;
-	int i;
+	Backend *be = op->o_bd;
 
-	ldap_pvt_thread_mutex_lock( &op->o_abandonmutex );
-	i = op->o_abandon;
-	ldap_pvt_thread_mutex_unlock( &op->o_abandonmutex );
-	if (i) return SLAPD_ABANDON;
+	if ( op->o_abandon ) return SLAPD_ABANDON;
 
-	if ( !dn_match( &target->e_nname, gr_ndn ) ) {
-		/* we won't attempt to send it to a different backend */
-		
-		be = select_backend( gr_ndn, 0,
-			(be->be_flags & SLAP_BFLAG_GLUE_INSTANCE));
+	op->o_bd = select_backend( gr_ndn, 0, 0 );
 
-		if (be == NULL) {
-			return LDAP_NO_SUCH_OBJECT;
-		}
-	} 
+	ldap_pvt_thread_mutex_lock( &op->o_conn->c_mutex );
 
-	ldap_pvt_thread_mutex_lock( &conn->c_mutex );
-	for (g = conn->c_groups; g; g=g->ga_next) {
-		if (g->ga_be != be || g->ga_oc != group_oc ||
+	for (g = op->o_conn->c_groups; g; g=g->ga_next) {
+		if (g->ga_be != op->o_bd || g->ga_oc != group_oc ||
 			g->ga_at != group_at || g->ga_len != gr_ndn->bv_len)
 			continue;
 		if (strcmp( g->ga_ndn, gr_ndn->bv_val ) == 0)
 			break;
 	}
-	ldap_pvt_thread_mutex_unlock( &conn->c_mutex );
+
+	ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
+
 	if (g) {
-		return g->ga_res;
+		rc = g->ga_res;
+		goto done;
 	}
 
-	if( be->be_group ) {
-		int res = be->be_group( be, conn, op,
-			target, gr_ndn, op_ndn,
-			group_oc, group_at );
-		
-		if (op->o_tag != LDAP_REQ_BIND) {
-			g = ch_malloc(sizeof(GroupAssertion) + gr_ndn->bv_len);
-			g->ga_be = be;
-			g->ga_oc = group_oc;
-			g->ga_at = group_at;
-			g->ga_res = res;
-			g->ga_len = gr_ndn->bv_len;
-			strcpy(g->ga_ndn, gr_ndn->bv_val);
-			ldap_pvt_thread_mutex_lock( &conn->c_mutex );
-			g->ga_next = conn->c_groups;
-			conn->c_groups = g;
-			ldap_pvt_thread_mutex_unlock( &conn->c_mutex );
+	if ( target && dn_match( &target->e_nname, gr_ndn ) ) {
+		e = target;
+	} else {
+		rc = be_entry_get_rw(op, gr_ndn, group_oc, group_at, 0, &e );
+	}
+	if ( e ) {
+		a = attr_find( e->e_attrs, group_at );
+		if ( a ) {
+			rc = value_find_ex( group_at,
+				SLAP_MR_ATTRIBUTE_VALUE_NORMALIZED_MATCH |
+				SLAP_MR_ASSERTED_VALUE_NORMALIZED_MATCH,
+				a->a_nvals, op_ndn, op->o_tmpmemctx );
+		} else {
+			rc = LDAP_NO_SUCH_ATTRIBUTE;
 		}
-
-		return res;
+		if (e != target ) {
+			be_entry_release_r( op, e );
+		}
+	} else {
+		rc = LDAP_NO_SUCH_OBJECT;
 	}
 
-	return LDAP_UNWILLING_TO_PERFORM;
+	if ( op->o_tag != LDAP_REQ_BIND && !op->o_do_not_cache ) {
+		g = ch_malloc(sizeof(GroupAssertion) + gr_ndn->bv_len);
+		g->ga_be = op->o_bd;
+		g->ga_oc = group_oc;
+		g->ga_at = group_at;
+		g->ga_res = rc;
+		g->ga_len = gr_ndn->bv_len;
+		strcpy(g->ga_ndn, gr_ndn->bv_val);
+		ldap_pvt_thread_mutex_lock( &op->o_conn->c_mutex );
+		g->ga_next = op->o_conn->c_groups;
+		op->o_conn->c_groups = g;
+		ldap_pvt_thread_mutex_unlock( &op->o_conn->c_mutex );
+	}
+done:
+	op->o_bd = be;
+	return rc;
 }
 
 int 
 backend_attribute(
-	Backend	*be,
-	Connection *conn,
 	Operation *op,
 	Entry	*target,
 	struct berval	*edn,
@@ -1054,47 +1227,86 @@ backend_attribute(
 	BerVarray *vals
 )
 {
-	if ( target == NULL || !dn_match( &target->e_nname, edn ) ) {
-		/* we won't attempt to send it to a different backend */
-		
-		be = select_backend( edn, 0,
-			(be->be_flags & SLAP_BFLAG_GLUE_INSTANCE));
+	Entry *e;
+	Attribute *a;
+	int i, j, rc = LDAP_SUCCESS;
+	AccessControlState acl_state = ACL_STATE_INIT;
+	Backend *be = op->o_bd;
 
-		if (be == NULL) {
-			return LDAP_NO_SUCH_OBJECT;
-		}
+	op->o_bd = select_backend( edn, 0, 0 );
+
+	if ( target && dn_match( &target->e_nname, edn ) ) {
+		e = target;
+	} else {
+		rc = be_entry_get_rw(op, edn, NULL, entry_at, 0, &e );
 	} 
 
-	if( be->be_attribute ) {
-		return be->be_attribute( be, conn, op, target, edn,
-			entry_at, vals );
+	if ( e ) {
+		a = attr_find( e->e_attrs, entry_at );
+		if ( a ) {
+			BerVarray v;
+
+			if ( op->o_conn && access_allowed( op,
+				e, entry_at, NULL, ACL_AUTH,
+				&acl_state ) == 0 ) {
+				rc = LDAP_INSUFFICIENT_ACCESS;
+				goto freeit;
+			}
+
+			for ( i=0; a->a_vals[i].bv_val; i++ ) ;
+			
+			v = op->o_tmpalloc( sizeof(struct berval) * (i+1), op->o_tmpmemctx );
+			for ( i=0,j=0; a->a_vals[i].bv_val; i++ ) {
+				if ( op->o_conn && access_allowed( op,
+					e, entry_at,
+					&a->a_nvals[i],
+					ACL_AUTH, &acl_state ) == 0 ) {
+					continue;
+				}
+				ber_dupbv_x( &v[j],
+					&a->a_nvals[i], op->o_tmpmemctx );
+				if (v[j].bv_val ) j++;
+			}
+			if (j == 0) {
+				op->o_tmpfree( v, op->o_tmpmemctx );
+				*vals = NULL;
+				rc = LDAP_INSUFFICIENT_ACCESS;
+			} else {
+				v[j].bv_val = NULL;
+				v[j].bv_len = 0;
+				*vals = v;
+				rc = LDAP_SUCCESS;
+			}
+		}
+freeit:		if (e != target ) {
+			be_entry_release_r( op, e );
+		}
 	}
 
-	return LDAP_UNWILLING_TO_PERFORM;
+	op->o_bd = be;
+	return rc;
 }
 
 Attribute *backend_operational(
-	Backend *be,
-	Connection *conn,
 	Operation *op,
-	Entry *e,
-	AttributeName *attrs,
+	SlapReply *rs,
 	int opattrs	)
 {
 	Attribute *a = NULL, **ap = &a;
 
-#ifdef SLAPD_SCHEMA_DN
-	*ap = slap_operational_subschemaSubentry();
-	ap = &(*ap)->a_next;
-#endif
-
 	/*
 	 * If operational attributes (allegedly) are required, 
 	 * and the backend supports specific operational attributes, 
 	 * add them to the attribute list
 	 */
-	if ( ( opattrs || attrs ) && be && be->be_operational != NULL ) {
-		( void )be->be_operational( be, conn, op, e, attrs, opattrs, ap );
+	if ( opattrs || ( op->ors_attrs &&
+		ad_inlist( slap_schema.si_ad_subschemaSubentry, op->ors_attrs )) ) {
+		*ap = slap_operational_subschemaSubentry( op->o_bd );
+		ap = &(*ap)->a_next;
+	}
+
+	if ( ( opattrs || op->ors_attrs ) && op->o_bd && op->o_bd->be_operational != NULL ) {
+		( void )op->o_bd->be_operational( op, rs, opattrs, ap );
 	}
 
 	return a;
