@@ -1,7 +1,7 @@
 /* dn.c - routines for dealing with distinguished names */
-/* $OpenLDAP: pkg/ldap/servers/slapd/dn.c,v 1.124 2002/02/14 15:01:48 ando Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/dn.c,v 1.144 2003/05/28 20:15:44 hyc Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -18,11 +18,7 @@
 
 #include "slap.h"
 
-const struct berval slap_empty_bv = { 0, "" };
-
-#define SLAP_LDAPDN_PRETTY 0x1
-
-#define SLAP_LDAPDN_MAXLEN 8192
+#include "lutil.h"
 
 /*
  * The DN syntax-related functions take advantage of the dn representation
@@ -46,21 +42,21 @@ const struct berval slap_empty_bv = { 0, "" };
  * structural representation of a distinguished name.
  */
 static int
-LDAPDN_validate( LDAPDN *dn )
+LDAPDN_validate( LDAPDN dn )
 {
 	int 		iRDN;
 	int 		rc;
 
 	assert( dn );
 
-	for ( iRDN = 0; dn[ 0 ][ iRDN ]; iRDN++ ) {
-		LDAPRDN		*rdn = dn[ 0 ][ iRDN ];
+	for ( iRDN = 0; dn[ iRDN ]; iRDN++ ) {
+		LDAPRDN		rdn = dn[ iRDN ];
 		int		iAVA;
 
 		assert( rdn );
 
-		for ( iAVA = 0; rdn[ 0 ][ iAVA ]; iAVA++ ) {
-			LDAPAVA			*ava = rdn[ 0 ][ iAVA ];
+		for ( iAVA = 0; rdn[ iAVA ]; iAVA++ ) {
+			LDAPAVA			*ava = rdn[ iAVA ];
 			AttributeDescription	*ad;
 			slap_syntax_validate_func *validate = NULL;
 
@@ -110,7 +106,7 @@ dnValidate(
 	struct berval *in )
 {
 	int		rc;
-	LDAPDN		*dn = NULL;
+	LDAPDN		dn = NULL;
 
 	assert( in );
 
@@ -156,16 +152,16 @@ dnValidate(
  * a RDN should be limited.
  */
 static void
-AVA_Sort( LDAPRDN *rdn, int iAVA )
+AVA_Sort( LDAPRDN rdn, int iAVA )
 {
 	int		i;
-	LDAPAVA		*ava_in = rdn[ 0 ][ iAVA ];
+	LDAPAVA		*ava_in = rdn[ iAVA ];
 
 	assert( rdn );
 	assert( ava_in );
 	
 	for ( i = 0; i < iAVA; i++ ) {
-		LDAPAVA		*ava = rdn[ 0 ][ i ];
+		LDAPAVA		*ava = rdn[ i ];
 		int		a, j;
 
 		assert( ava );
@@ -204,7 +200,7 @@ AVA_Sort( LDAPRDN *rdn, int iAVA )
 				return;
 			}
 
-			ava = rdn[ 0 ][ i ];
+			ava = rdn[ i ];
 			a = strcmp( ava_in->la_attr.bv_val, 
 					ava->la_attr.bv_val );
 		}
@@ -213,9 +209,9 @@ AVA_Sort( LDAPRDN *rdn, int iAVA )
 		 * move ahead
 		 */
 		for ( j = iAVA; j > i; j-- ) {
-			rdn[ 0 ][ j ] = rdn[ 0 ][ j - 1 ];
+			rdn[ j ] = rdn[ j - 1 ];
 		}
-		rdn[ 0 ][ i ] = ava_in;
+		rdn[ i ] = ava_in;
 
 		return;
 	}
@@ -226,24 +222,26 @@ AVA_Sort( LDAPRDN *rdn, int iAVA )
  * structural representation of a distinguished name.
  */
 static int
-LDAPDN_rewrite( LDAPDN *dn, unsigned flags )
+LDAPDN_rewrite( LDAPDN dn, unsigned flags, void *ctx )
 {
 	int 		iRDN;
 	int 		rc;
 
 	assert( dn );
 
-	for ( iRDN = 0; dn[ 0 ][ iRDN ]; iRDN++ ) {
-		LDAPRDN		*rdn = dn[ 0 ][ iRDN ];
+	for ( iRDN = 0; dn[ iRDN ]; iRDN++ ) {
+		LDAPRDN		rdn = dn[ iRDN ];
 		int		iAVA;
 
 		assert( rdn );
 
-		for ( iAVA = 0; rdn[ 0 ][ iAVA ]; iAVA++ ) {
-			LDAPAVA			*ava = rdn[ 0 ][ iAVA ];
+		for ( iAVA = 0; rdn[ iAVA ]; iAVA++ ) {
+			LDAPAVA			*ava = rdn[ iAVA ];
 			AttributeDescription	*ad;
+			slap_syntax_validate_func *validf = NULL;
+			slap_mr_normalize_func *normf = NULL;
 			slap_syntax_transform_func *transf = NULL;
-			MatchingRule *mr;
+			MatchingRule *mr = NULL;
 			struct berval		bv = { 0, NULL };
 			int			do_sort = 0;
 
@@ -267,46 +265,73 @@ LDAPDN_rewrite( LDAPDN *dn, unsigned flags )
 			ava->la_attr = ad->ad_cname;
 
 			if( ava->la_flags & LDAP_AVA_BINARY ) {
-				/* AVA is binary encoded, don't muck with it */
-				transf = NULL;
-				mr = NULL;
+				if( ava->la_value.bv_len == 0 ) {
+					/* BER encoding is empty */
+					return LDAP_INVALID_SYNTAX;
+				}
 
+				/* AVA is binary encoded, don't muck with it */
 			} else if( flags & SLAP_LDAPDN_PRETTY ) {
 				transf = ad->ad_type->sat_syntax->ssyn_pretty;
-				mr = NULL;
-			} else {
-				transf = ad->ad_type->sat_syntax->ssyn_normalize;
+				if( !transf ) {
+					validf = ad->ad_type->sat_syntax->ssyn_validate;
+				}
+			} else { /* normalization */
+				validf = ad->ad_type->sat_syntax->ssyn_validate;
 				mr = ad->ad_type->sat_equality;
+				if( mr ) normf = mr->smr_normalize;
+			}
+
+			if ( validf ) {
+				/* validate value before normalization */
+				rc = ( *validf )( ad->ad_type->sat_syntax,
+					ava->la_value.bv_len
+						? &ava->la_value
+						: (struct berval *) &slap_empty_bv );
+
+				if ( rc != LDAP_SUCCESS ) {
+					return LDAP_INVALID_SYNTAX;
+				}
 			}
 
 			if ( transf ) {
 				/*
-			 	 * transform value by normalize/pretty function
+			 	 * transform value by pretty function
 				 *	if value is empty, use empty_bv
 				 */
 				rc = ( *transf )( ad->ad_type->sat_syntax,
 					ava->la_value.bv_len
 						? &ava->la_value
 						: (struct berval *) &slap_empty_bv,
-					&bv );
+					&bv, ctx );
 			
 				if ( rc != LDAP_SUCCESS ) {
 					return LDAP_INVALID_SYNTAX;
 				}
 			}
 
-			if( mr && ( mr->smr_usage & SLAP_MR_DN_FOLD ) ) {
-				char *s = bv.bv_val;
-
-				if ( UTF8bvnormalize( &bv, &bv, 
-						LDAP_UTF8_CASEFOLD ) == NULL ) {
+			if ( normf ) {
+				/*
+			 	 * normalize value
+				 *	if value is empty, use empty_bv
+				 */
+				rc = ( *normf )(
+					0,
+					ad->ad_type->sat_syntax,
+					mr,
+					ava->la_value.bv_len
+						? &ava->la_value
+						: (struct berval *) &slap_empty_bv,
+					&bv, ctx );
+			
+				if ( rc != LDAP_SUCCESS ) {
 					return LDAP_INVALID_SYNTAX;
 				}
-				free( s );
 			}
 
+
 			if( bv.bv_val ) {
-				free( ava->la_value.bv_val );
+				ber_memfree_x( ava->la_value.bv_val, ctx );
 				ava->la_value = bv;
 			}
 
@@ -317,34 +342,14 @@ LDAPDN_rewrite( LDAPDN *dn, unsigned flags )
 	return LDAP_SUCCESS;
 }
 
-/*
- * dn normalize routine
- */
 int
 dnNormalize(
-	Syntax *syntax,
-	struct berval *val,
-	struct berval **normalized )
-{
-	struct berval *out;
-	int rc;
-
-	assert( normalized && *normalized == NULL );
-
-	out = ch_malloc( sizeof( struct berval ) );
-	rc = dnNormalize2( syntax, val, out );
-	if ( rc != LDAP_SUCCESS )
-		free( out );
-	else
-		*normalized = out;
-	return rc;
-}
-
-int
-dnNormalize2(
-	Syntax *syntax,
-	struct berval *val,
-	struct berval *out )
+    slap_mask_t use,
+    Syntax *syntax,
+    MatchingRule *mr,
+    struct berval *val,
+    struct berval *out,
+    void *ctx)
 {
 	assert( val );
 	assert( out );
@@ -352,13 +357,13 @@ dnNormalize2(
 	Debug( LDAP_DEBUG_TRACE, ">>> dnNormalize: <%s>\n", val->bv_val, 0, 0 );
 
 	if ( val->bv_len != 0 ) {
-		LDAPDN		*dn = NULL;
+		LDAPDN		dn = NULL;
 		int		rc;
 
 		/*
 		 * Go to structural representation
 		 */
-		rc = ldap_bv2dn( val, &dn, LDAP_DN_FORMAT_LDAP );
+		rc = ldap_bv2dn_x( val, &dn, LDAP_DN_FORMAT_LDAP, ctx );
 		if ( rc != LDAP_SUCCESS ) {
 			return LDAP_INVALID_SYNTAX;
 		}
@@ -368,23 +373,24 @@ dnNormalize2(
 		/*
 		 * Schema-aware rewrite
 		 */
-		if ( LDAPDN_rewrite( dn, 0 ) != LDAP_SUCCESS ) {
-			ldap_dnfree( dn );
+		if ( LDAPDN_rewrite( dn, 0, ctx ) != LDAP_SUCCESS ) {
+			ldap_dnfree_x( dn, ctx );
 			return LDAP_INVALID_SYNTAX;
 		}
 
 		/*
 		 * Back to string representation
 		 */
-		rc = ldap_dn2bv( dn, out, LDAP_DN_FORMAT_LDAPV3 );
+		rc = ldap_dn2bv_x( dn, out,
+			LDAP_DN_FORMAT_LDAPV3 | LDAP_DN_PRETTY, ctx );
 
-		ldap_dnfree( dn );
+		ldap_dnfree_x( dn, ctx );
 
 		if ( rc != LDAP_SUCCESS ) {
 			return LDAP_INVALID_SYNTAX;
 		}
 	} else {
-		ber_dupbv( out, val );
+		ber_dupbv_x( out, val, ctx );
 	}
 
 	Debug( LDAP_DEBUG_TRACE, "<<< dnNormalize: <%s>\n", out->bv_val, 0, 0 );
@@ -392,52 +398,34 @@ dnNormalize2(
 	return LDAP_SUCCESS;
 }
 
-/*
- * dn "pretty"ing routine
- */
 int
 dnPretty(
 	Syntax *syntax,
 	struct berval *val,
-	struct berval **pretty)
-{
-	struct berval *out;
-	int rc;
-
-	assert( pretty && *pretty == NULL );
-
-	out = ch_malloc( sizeof( struct berval ) );
-	rc = dnPretty2( syntax, val, out );
-	if ( rc != LDAP_SUCCESS )
-		free( out );
-	else
-		*pretty = out;
-	return rc;
-}
-
-int
-dnPretty2(
-	Syntax *syntax,
-	struct berval *val,
-	struct berval *out)
+	struct berval *out,
+	void *ctx)
 {
 	assert( val );
 	assert( out );
 
+#ifdef NEW_LOGGING
+	LDAP_LOG( OPERATION, ARGS, ">>> dnPretty: <%s>\n", val->bv_val, 0, 0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, ">>> dnPretty: <%s>\n", val->bv_val, 0, 0 );
+#endif
 
 	if ( val->bv_len == 0 ) {
-		ber_dupbv( out, val );
+		ber_dupbv_x( out, val, ctx );
 
 	} else if ( val->bv_len > SLAP_LDAPDN_MAXLEN ) {
 		return LDAP_INVALID_SYNTAX;
 
 	} else {
-		LDAPDN		*dn = NULL;
+		LDAPDN		dn = NULL;
 		int		rc;
 
 		/* FIXME: should be liberal in what we accept */
-		rc = ldap_bv2dn( val, &dn, LDAP_DN_FORMAT_LDAP );
+		rc = ldap_bv2dn_x( val, &dn, LDAP_DN_FORMAT_LDAP, ctx );
 		if ( rc != LDAP_SUCCESS ) {
 			return LDAP_INVALID_SYNTAX;
 		}
@@ -447,8 +435,8 @@ dnPretty2(
 		/*
 		 * Schema-aware rewrite
 		 */
-		if ( LDAPDN_rewrite( dn, SLAP_LDAPDN_PRETTY ) != LDAP_SUCCESS ) {
-			ldap_dnfree( dn );
+		if ( LDAPDN_rewrite( dn, SLAP_LDAPDN_PRETTY, ctx ) != LDAP_SUCCESS ) {
+			ldap_dnfree_x( dn, ctx );
 			return LDAP_INVALID_SYNTAX;
 		}
 
@@ -456,10 +444,10 @@ dnPretty2(
 		/* RE: the default is the form that is used as
 		 * an internal representation; the pretty form
 		 * is a variant */
-		rc = ldap_dn2bv( dn, out,
-			LDAP_DN_FORMAT_LDAPV3 | LDAP_DN_PRETTY );
+		rc = ldap_dn2bv_x( dn, out,
+			LDAP_DN_FORMAT_LDAPV3 | LDAP_DN_PRETTY, ctx );
 
-		ldap_dnfree( dn );
+		ldap_dnfree_x( dn, ctx );
 
 		if ( rc != LDAP_SUCCESS ) {
 			return LDAP_INVALID_SYNTAX;
@@ -471,6 +459,61 @@ dnPretty2(
 	return LDAP_SUCCESS;
 }
 
+int
+dnPrettyNormalDN(
+	Syntax *syntax,
+	struct berval *val,
+	LDAPDN *dn,
+	int flags,
+	void *ctx )
+{
+	assert( val );
+	assert( dn );
+
+#ifdef NEW_LOGGING
+	LDAP_LOG( OPERATION, ARGS, ">>> dn%sDN: <%s>\n", 
+			flags == SLAP_LDAPDN_PRETTY ? "Pretty" : "Normal", 
+			val->bv_val, 0 );
+#else
+	Debug( LDAP_DEBUG_TRACE, ">>> dn%sDN: <%s>\n", 
+			flags == SLAP_LDAPDN_PRETTY ? "Pretty" : "Normal", 
+			val->bv_val, 0 );
+#endif
+
+	if ( val->bv_len == 0 ) {
+		return LDAP_SUCCESS;
+
+	} else if ( val->bv_len > SLAP_LDAPDN_MAXLEN ) {
+		return LDAP_INVALID_SYNTAX;
+
+	} else {
+		int		rc;
+
+		/* FIXME: should be liberal in what we accept */
+		rc = ldap_bv2dn_x( val, dn, LDAP_DN_FORMAT_LDAP, ctx );
+		if ( rc != LDAP_SUCCESS ) {
+			return LDAP_INVALID_SYNTAX;
+		}
+
+		assert( strlen( val->bv_val ) == val->bv_len );
+
+		/*
+		 * Schema-aware rewrite
+		 */
+		if ( LDAPDN_rewrite( *dn, flags, ctx ) != LDAP_SUCCESS ) {
+			ldap_dnfree_x( *dn, ctx );
+			*dn = NULL;
+			return LDAP_INVALID_SYNTAX;
+		}
+	}
+
+	Debug( LDAP_DEBUG_TRACE, "<<< dn%sDN\n", 
+			flags == SLAP_LDAPDN_PRETTY ? "Pretty" : "Normal",
+			0, 0 );
+
+	return LDAP_SUCCESS;
+}
+
 /*
  * Combination of both dnPretty and dnNormalize
  */
@@ -479,24 +522,29 @@ dnPrettyNormal(
 	Syntax *syntax,
 	struct berval *val,
 	struct berval *pretty,
-	struct berval *normal)
+	struct berval *normal,
+	void *ctx)
 {
+#ifdef NEW_LOGGING
+	LDAP_LOG ( OPERATION, ENTRY, ">>> dnPrettyNormal: <%s>\n", val->bv_val, 0, 0 );
+#else
 	Debug( LDAP_DEBUG_TRACE, ">>> dnPrettyNormal: <%s>\n", val->bv_val, 0, 0 );
+#endif
 
 	assert( val );
 	assert( pretty );
 	assert( normal );
 
 	if ( val->bv_len == 0 ) {
-		ber_dupbv( pretty, val );
-		ber_dupbv( normal, val );
+		ber_dupbv_x( pretty, val, ctx );
+		ber_dupbv_x( normal, val, ctx );
 
 	} else if ( val->bv_len > SLAP_LDAPDN_MAXLEN ) {
 		/* too big */
 		return LDAP_INVALID_SYNTAX;
 
 	} else {
-		LDAPDN		*dn = NULL;
+		LDAPDN		dn = NULL;
 		int		rc;
 
 		pretty->bv_val = NULL;
@@ -505,7 +553,7 @@ dnPrettyNormal(
 		normal->bv_len = 0;
 
 		/* FIXME: should be liberal in what we accept */
-		rc = ldap_bv2dn( val, &dn, LDAP_DN_FORMAT_LDAP );
+		rc = ldap_bv2dn_x( val, &dn, LDAP_DN_FORMAT_LDAP, ctx );
 		if ( rc != LDAP_SUCCESS ) {
 			return LDAP_INVALID_SYNTAX;
 		}
@@ -515,40 +563,46 @@ dnPrettyNormal(
 		/*
 		 * Schema-aware rewrite
 		 */
-		if ( LDAPDN_rewrite( dn, SLAP_LDAPDN_PRETTY ) != LDAP_SUCCESS ) {
-			ldap_dnfree( dn );
+		if ( LDAPDN_rewrite( dn, SLAP_LDAPDN_PRETTY, ctx ) != LDAP_SUCCESS ) {
+			ldap_dnfree_x( dn, ctx );
 			return LDAP_INVALID_SYNTAX;
 		}
 
-		rc = ldap_dn2bv( dn, pretty,
-			LDAP_DN_FORMAT_LDAPV3 | LDAP_DN_PRETTY );
+		rc = ldap_dn2bv_x( dn, pretty,
+			LDAP_DN_FORMAT_LDAPV3 | LDAP_DN_PRETTY, ctx );
 
 		if ( rc != LDAP_SUCCESS ) {
-			ldap_dnfree( dn );
+			ldap_dnfree_x( dn, ctx );
 			return LDAP_INVALID_SYNTAX;
 		}
 
-		if ( LDAPDN_rewrite( dn, 0 ) != LDAP_SUCCESS ) {
-			ldap_dnfree( dn );
-			free( pretty->bv_val );
+		if ( LDAPDN_rewrite( dn, 0, ctx ) != LDAP_SUCCESS ) {
+			ldap_dnfree_x( dn, ctx );
+			ber_memfree_x( pretty->bv_val, ctx );
 			pretty->bv_val = NULL;
 			pretty->bv_len = 0;
 			return LDAP_INVALID_SYNTAX;
 		}
 
-		rc = ldap_dn2bv( dn, normal, LDAP_DN_FORMAT_LDAPV3 );
+		rc = ldap_dn2bv_x( dn, normal,
+			LDAP_DN_FORMAT_LDAPV3 | LDAP_DN_PRETTY, ctx );
 
-		ldap_dnfree( dn );
+		ldap_dnfree_x( dn, ctx );
 		if ( rc != LDAP_SUCCESS ) {
-			free( pretty->bv_val );
+			ber_memfree_x( pretty->bv_val, ctx );
 			pretty->bv_val = NULL;
 			pretty->bv_len = 0;
 			return LDAP_INVALID_SYNTAX;
 		}
 	}
 
+#ifdef NEW_LOGGING
+	LDAP_LOG (OPERATION, RESULTS, "<<< dnPrettyNormal: <%s>, <%s>\n",
+		pretty->bv_val, normal->bv_val, 0  );
+#else
 	Debug( LDAP_DEBUG_TRACE, "<<< dnPrettyNormal: <%s>, <%s>\n",
 		pretty->bv_val, normal->bv_val, 0 );
+#endif
 
 	return LDAP_SUCCESS;
 }
@@ -580,9 +634,8 @@ dnMatch(
 	}
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "schema", LDAP_LEVEL_ENTRY,
-		"dnMatch: %d\n    %s\n    %s\n", match,
-		value->bv_val, asserted->bv_val ));
+	LDAP_LOG( CONFIG, ENTRY, "dnMatch: %d\n    %s\n    %s\n", 
+		match, value->bv_val, asserted->bv_val  );
 #else
 	Debug( LDAP_DEBUG_ARGS, "dnMatch %d\n\t\"%s\"\n\t\"%s\"\n",
 		match, value->bv_val, asserted->bv_val );
@@ -609,7 +662,8 @@ dnParent(
 
 	/* one-level dn */
 	if ( p == NULL ) {
-		*pdn = slap_empty_bv;
+		pdn->bv_len = 0;
+		pdn->bv_val = dn->bv_val + dn->bv_len;
 		return;
 	}
 
@@ -626,9 +680,10 @@ dnParent(
 int
 dnExtractRdn( 
 	struct berval	*dn, 
-	struct berval 	*rdn )
+	struct berval 	*rdn,
+	void *ctx )
 {
-	LDAPRDN		*tmpRDN;
+	LDAPRDN		tmpRDN;
 	const char	*p;
 	int		rc;
 
@@ -639,13 +694,14 @@ dnExtractRdn(
 		return LDAP_OTHER;
 	}
 
-	rc = ldap_bv2rdn( dn, &tmpRDN, (char **)&p, LDAP_DN_FORMAT_LDAP );
+	rc = ldap_bv2rdn_x( dn, &tmpRDN, (char **)&p, LDAP_DN_FORMAT_LDAP, ctx );
 	if ( rc != LDAP_SUCCESS ) {
 		return rc;
 	}
 
-	rc = ldap_rdn2bv( tmpRDN, rdn, LDAP_DN_FORMAT_LDAPV3 );
-	ldap_rdnfree( tmpRDN );
+	rc = ldap_rdn2bv_x( tmpRDN, rdn, LDAP_DN_FORMAT_LDAPV3 | LDAP_DN_PRETTY, ctx );
+
+	ldap_rdnfree_x( tmpRDN, ctx );
 	if ( rc != LDAP_SUCCESS ) {
 		return rc;
 	}
@@ -771,7 +827,7 @@ build_new_dn( struct berval * new_dn,
 	new_dn->bv_len = parent_dn->bv_len + newrdn->bv_len + 1;
 	new_dn->bv_val = (char *) ch_malloc( new_dn->bv_len + 1 );
 
-	ptr = slap_strcopy( new_dn->bv_val, newrdn->bv_val );
+	ptr = lutil_strcopy( new_dn->bv_val, newrdn->bv_val );
 	*ptr++ = ',';
 	strcpy( ptr, parent_dn->bv_val );
 }
@@ -814,3 +870,25 @@ dnIsSuffix(
 	/* compare */
 	return( strcmp( dn->bv_val + d, suffix->bv_val ) == 0 );
 }
+
+#ifdef HAVE_TLS
+/*
+ * Convert an X.509 DN into a normalized LDAP DN
+ */
+int
+dnX509normalize( void *x509_name, struct berval *out )
+{
+	/* Invoke the LDAP library's converter with our schema-rewriter */
+	return ldap_X509dn2bv( x509_name, out, LDAPDN_rewrite, 0 );
+}
+
+/*
+ * Get the TLS session's peer's DN into a normalized LDAP DN
+ */
+int
+dnX509peerNormalize( void *ssl, struct berval *dn )
+{
+
+	return ldap_pvt_tls_get_peer_dn( ssl, dn, (LDAPDN_rewrite_dummy *)LDAPDN_rewrite, 0 );
+}
+#endif
