@@ -1,7 +1,7 @@
 /* entry.c - routines for dealing with entries */
-/* $OpenLDAP: pkg/ldap/servers/slapd/entry.c,v 1.90.2.1 2002/02/23 23:24:44 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/entry.c,v 1.115 2003/05/02 11:29:29 hallvard Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -24,7 +24,9 @@ static int		emaxsize;/* max size of ebuf			 */
 /*
  * Empty root entry
  */
-const Entry slap_entry_root = { NOID, { 0, "" }, { 0, "" }, NULL, 0, { 0, "" }, NULL };
+const Entry slap_entry_root = {
+	NOID, { 0, "" }, { 0, "" }, NULL, 0, { 0, "" }, NULL
+};
 
 int entry_destroy(void)
 {
@@ -43,6 +45,7 @@ str2entry( char *s )
 	Entry		*e;
 	char		*type;
 	struct berval	vals[2];
+	struct berval	nvals[2], *nvalsp;
 	AttributeDescription *ad;
 	const char *text;
 	char	*next;
@@ -63,8 +66,7 @@ str2entry( char *s )
 	 */
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1,
-		"str2entry: \"%s\"\n", s ? s : "NULL" ));
+	LDAP_LOG( OPERATION, DETAIL1, "str2entry: \"%s\"\n", s ? s : "NULL", 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "=> str2entry\n",
 		s ? s : "NULL", 0, 0 );
@@ -75,8 +77,7 @@ str2entry( char *s )
 
 	if( e == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_ERR,
-			"str2entry: entry allocation failed.\n" ));
+		LDAP_LOG( OPERATION, ERR, "str2entry: entry allocation failed.\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "<= str2entry NULL (entry allocation failed)\n",
@@ -89,6 +90,7 @@ str2entry( char *s )
 	e->e_id = NOID;
 
 	/* dn + attributes */
+	vals[1].bv_len = 0;
 	vals[1].bv_val = NULL;
 
 	next = s;
@@ -99,8 +101,7 @@ str2entry( char *s )
 
 		if ( ldif_parse_line( s, &type, &vals[0].bv_val, &vals[0].bv_len ) != 0 ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1,
-				   "str2entry:  NULL (parse_line)\n" ));
+			LDAP_LOG( OPERATION, DETAIL1, "str2entry:  NULL (parse_line)\n",0, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 			    "<= str2entry NULL (parse_line)\n", 0, 0, 0 );
@@ -109,44 +110,39 @@ str2entry( char *s )
 		}
 
 		if ( strcasecmp( type, "dn" ) == 0 ) {
-			struct berval *pdn = NULL;
-
 			free( type );
 
 			if ( e->e_dn != NULL ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1, "str2entry: "
+				LDAP_LOG( OPERATION, DETAIL1, "str2entry: "
 					"entry %ld has multiple DNs \"%s\" and \"%s\"\n",
-					(long) e->e_id, e->e_dn,
-					vals[0].bv_val != NULL ? vals[0].bv_val : "" ));
+					(long) e->e_id, e->e_dn, vals[0].bv_val );
 #else
 				Debug( LDAP_DEBUG_ANY, "str2entry: "
 					"entry %ld has multiple DNs \"%s\" and \"%s\"\n",
-				    (long) e->e_id, e->e_dn,
-					vals[0].bv_val != NULL ? vals[0].bv_val : "" );
+				    (long) e->e_id, e->e_dn, vals[0].bv_val );
 #endif
-				if( vals[0].bv_val != NULL ) free( vals[0].bv_val );
+				free( vals[0].bv_val );
 				entry_free( e );
 				return NULL;
 			}
 
-			rc = dnPrettyNormal( NULL, &vals[0], &e->e_name, &e->e_nname );
-			free( vals[0].bv_val );
+			rc = dnPrettyNormal( NULL, &vals[0], &e->e_name, &e->e_nname, NULL );
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1, "str2entry: "
-					"entry %ld has invalid DN \"%s\"\n",
-					(long) e->e_id,
-					pdn->bv_val ? pdn->bv_val : "" ));
+				LDAP_LOG( OPERATION, DETAIL1, 
+					"str2entry: entry %ld has invalid DN \"%s\"\n",
+					(long) e->e_id, vals[0].bv_val, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY, "str2entry: "
 					"entry %ld has invalid DN \"%s\"\n",
-					(long) e->e_id,
-					pdn->bv_val ? pdn->bv_val : "", 0 );
+					(long) e->e_id, vals[0].bv_val, 0 );
 #endif
 				entry_free( e );
+				free( vals[0].bv_val );
 				return NULL;
 			}
+			free( vals[0].bv_val );
 			continue;
 		}
 
@@ -155,8 +151,8 @@ str2entry( char *s )
 
 		if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1,
-				"str2entry:  str2ad(%s): %s\n", type, text ));
+			LDAP_LOG( OPERATION, DETAIL1, 
+				"str2entry:  str2ad(%s): %s\n", type, text, 0 );
 #else
 			Debug( slapMode & SLAP_TOOL_MODE
 				? LDAP_DEBUG_ANY : LDAP_DEBUG_TRACE,
@@ -172,8 +168,8 @@ str2entry( char *s )
 			rc = slap_str2undef_ad( type, &ad, &text );
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1,
-					"str2entry:  str2undef_ad(%s): %s\n", type, text ));
+				LDAP_LOG( OPERATION, DETAIL1, 
+					"str2entry: str2undef_ad(%s): %s\n", type, text, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
 					"<= str2entry: str2undef_ad(%s): %s\n",
@@ -195,7 +191,7 @@ str2entry( char *s )
 
 			if( pretty ) {
 				rc = pretty( ad->ad_type->sat_syntax,
-					&vals[0], &pval );
+					&vals[0], &pval, NULL );
 
 			} else if( validate ) {
 				/*
@@ -205,9 +201,9 @@ str2entry( char *s )
 
 			} else {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
+				LDAP_LOG( OPERATION, INFO, 
 					"str2entry: no validator for syntax %s\n", 
-					ad->ad_type->sat_syntax->ssyn_oid ));
+					ad->ad_type->sat_syntax->ssyn_oid, 0, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
 					"str2entry: no validator for syntax %s\n",
@@ -221,13 +217,13 @@ str2entry( char *s )
 
 			if( rc != 0 ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "operation", LDAP_LEVEL_ERR,
-					"str2entry:  invalid value for syntax %s\n",
-					ad->ad_type->sat_syntax->ssyn_oid ));
+				LDAP_LOG( OPERATION, ERR, 
+					"str2entry:  invalid value for attribute %s (syntax %s)\n",
+					ad->ad_cname.bv_val, ad->ad_type->sat_syntax->ssyn_oid, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
-					"str2entry: invalid value for syntax %s\n",
-					ad->ad_type->sat_syntax->ssyn_oid, 0, 0 );
+					"str2entry: invalid value for attribute %s (syntax %s)\n",
+					ad->ad_cname.bv_val, ad->ad_type->sat_syntax->ssyn_oid, 0 );
 #endif
 				entry_free( e );
 				free( vals[0].bv_val );
@@ -241,11 +237,44 @@ str2entry( char *s )
 			}
 		}
 
-		rc = attr_merge( e, ad, vals );
+		nvalsp = NULL;
+		nvals[0].bv_val = NULL;
+
+		if( ad->ad_type->sat_equality &&
+			ad->ad_type->sat_equality->smr_normalize )
+		{
+			rc = ad->ad_type->sat_equality->smr_normalize(
+				SLAP_MR_VALUE_OF_ATTRIBUTE_SYNTAX,
+				ad->ad_type->sat_syntax,
+				ad->ad_type->sat_equality,
+				&vals[0], &nvals[0], NULL );
+
+			if( rc ) {
+#ifdef NEW_LOGGING
+				LDAP_LOG( OPERATION, DETAIL1,
+					"str2entry:  NULL (smr_normalize %d)\n" , rc, 0, 0 );
+#else
+				Debug( LDAP_DEBUG_ANY,
+			   		"<= str2entry NULL (smr_normalize %d)\n", rc, 0, 0 );
+#endif
+
+				entry_free( e );
+				free( vals[0].bv_val );
+				free( type );
+				return NULL;
+			}
+
+			nvals[1].bv_len = 0;
+			nvals[1].bv_val = NULL;
+
+			nvalsp = &nvals[0];
+		}
+
+		rc = attr_merge( e, ad, vals, nvalsp );
 		if( rc != 0 ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1,
-				"str2entry:  NULL (attr_merge)\n" ));
+			LDAP_LOG( OPERATION, DETAIL1,
+				"str2entry:  NULL (attr_merge)\n" , 0, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY,
 			    "<= str2entry NULL (attr_merge)\n", 0, 0, 0 );
@@ -258,25 +287,25 @@ str2entry( char *s )
 
 		free( type );
 		free( vals[0].bv_val );
+		free( nvals[0].bv_val );
 	}
 
 	/* check to make sure there was a dn: line */
 	if ( e->e_dn == NULL ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-			"str2entry:  entry %ld has no dn.\n",
-			(long) e->e_id ));
+		LDAP_LOG( OPERATION, INFO, 
+			"str2entry:  entry %ld has no dn.\n", (long) e->e_id, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY, "str2entry: entry %ld has no dn\n",
 		    (long) e->e_id, 0, 0 );
 #endif
 		entry_free( e );
-		return( NULL );
+		return NULL;
 	}
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL2,
-		"str2entry(%s) -> 0x%lx\n", e->e_dn, (unsigned long)e ));
+	LDAP_LOG( OPERATION, DETAIL2,
+		"str2entry(%s) -> 0x%lx\n", e->e_dn, (unsigned long)e, 0 );
 #else
 	Debug(LDAP_DEBUG_TRACE, "<= str2entry(%s) -> 0x%lx\n",
 		e->e_dn, (unsigned long) e, 0 );
@@ -308,6 +337,8 @@ entry2str(
 	int		i;
 	ber_len_t tmplen;
 
+	assert( e != NULL );
+
 	/*
 	 * In string format, an entry looks like this:
 	 *	dn: <dn>\n
@@ -391,26 +422,28 @@ entry_free( Entry *e )
 int
 entry_cmp( Entry *e1, Entry *e2 )
 {
-	return( e1 < e2 ? -1 : (e1 > e2 ? 1 : 0) );
+	return SLAP_PTRCMP( e1, e2 );
 }
 
 int
-entry_dn_cmp( Entry *e1, Entry *e2 )
+entry_dn_cmp( const void *v_e1, const void *v_e2 )
 {
 	/* compare their normalized UPPERCASED dn's */
-	int rc = e1->e_nname.bv_len - e2->e_nname.bv_len;
-	if (rc) return rc;
-	return( strcmp( e1->e_ndn, e2->e_ndn ) );
+	const Entry *e1 = v_e1, *e2 = v_e2;
+
+	return ber_bvcmp( &e1->e_nname, &e2->e_nname );
 }
 
 int
-entry_id_cmp( Entry *e1, Entry *e2 )
+entry_id_cmp( const void *v_e1, const void *v_e2 )
 {
+	const Entry *e1 = v_e1, *e2 = v_e2;
 	return( e1->e_id < e2->e_id ? -1 : (e1->e_id > e2->e_id ? 1 : 0) );
 }
 
-#ifdef SLAPD_BDB
-
+#define entry_lenlen(l)	((l) < 0x80) ? 1 : ((l) < 0x100) ? 2 : \
+	((l) < 0x10000) ? 3 : ((l) < 0x1000000) ? 4 : 5
+#if 0
 /* This is like a ber_len */
 static ber_len_t
 entry_lenlen(ber_len_t len)
@@ -425,6 +458,7 @@ entry_lenlen(ber_len_t len)
 		return 4;
 	return 5;
 }
+#endif
 
 static void
 entry_putlen(unsigned char **buf, ber_len_t len)
@@ -462,6 +496,52 @@ entry_getlen(unsigned char **buf)
 	return len;
 }
 
+/* Add up the size of the entry for a flattened buffer */
+void entry_flatsize(Entry *e, ber_len_t *psiz, ber_len_t *plen, int norm)
+{
+	ber_len_t siz = sizeof(Entry);
+	ber_len_t len, dnlen, ndnlen;
+	int i;
+	Attribute *a;
+
+	dnlen = e->e_name.bv_len;
+	len = dnlen + 1;	/* trailing NUL byte */
+	len += entry_lenlen(dnlen);
+	if (norm) {
+		ndnlen = e->e_nname.bv_len;
+		len += ndnlen + 1;
+		len += entry_lenlen(ndnlen);
+	}
+	for (a=e->e_attrs; a; a=a->a_next) {
+		/* For AttributeDesc, we only store the attr name */
+		siz += sizeof(Attribute);
+		len += a->a_desc->ad_cname.bv_len+1;
+		len += entry_lenlen(a->a_desc->ad_cname.bv_len);
+		for (i=0; a->a_vals[i].bv_val; i++) {
+			siz += sizeof(struct berval);
+			len += a->a_vals[i].bv_len + 1;
+			len += entry_lenlen(a->a_vals[i].bv_len);
+		}
+		len += entry_lenlen(i);
+		siz += sizeof(struct berval);	/* empty berval at end */
+		if (norm && a->a_nvals != a->a_vals) {
+			for (i=0; a->a_nvals[i].bv_val; i++) {
+				siz += sizeof(struct berval);
+				len += a->a_nvals[i].bv_len + 1;
+				len += entry_lenlen(a->a_nvals[i].bv_len);
+			}
+			len += entry_lenlen(i);	/* i nvals */
+			siz += sizeof(struct berval);
+		} else {
+			len += entry_lenlen(0);	/* 0 nvals */
+		}
+	}
+	len += 1;	/* NUL byte at end */
+	len += entry_lenlen(siz);
+	*psiz = siz;
+	*plen = len;
+}
+
 /* Flatten an Entry into a buffer. The buffer is filled with just the
  * strings/bervals of all the entry components. Each field is preceded
  * by its length, encoded the way ber_put_len works. Every field is NUL
@@ -478,33 +558,17 @@ int entry_encode(Entry *e, struct berval *bv)
 	unsigned char *ptr;
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1,
-		"entry_encode: id: 0x%08lx  \"%s\"\n",
-		(long) e->e_id, e->e_dn ));
+	LDAP_LOG( OPERATION, DETAIL1, "entry_encode: id: 0x%08lx  \"%s\"\n",
+		(long) e->e_id, e->e_dn, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "=> entry_encode(0x%08lx): %s\n",
 		(long) e->e_id, e->e_dn, 0 );
 #endif
 	dnlen = e->e_name.bv_len;
 	ndnlen = e->e_nname.bv_len;
-	len = dnlen + ndnlen + 2;	/* two trailing NUL bytes */
-	len += entry_lenlen(dnlen);
-	len += entry_lenlen(ndnlen);
-	for (a=e->e_attrs; a; a=a->a_next) {
-		/* For AttributeDesc, we only store the attr name */
-		siz += sizeof(Attribute);
-		len += a->a_desc->ad_cname.bv_len+1;
-		len += entry_lenlen(a->a_desc->ad_cname.bv_len);
-		for (i=0; a->a_vals[i].bv_val; i++) {
-			siz += sizeof(struct berval);
-			len += a->a_vals[i].bv_len + 1;
-			len += entry_lenlen(a->a_vals[i].bv_len);
-		}
-		len += entry_lenlen(i);
-		siz += sizeof(struct berval);	/* empty berval at end */
-	}
-	len += 1;	/* NUL byte at end */
-	len += entry_lenlen(siz);
+
+	entry_flatsize( e, &siz, &len, 1 );
+
 	bv->bv_len = len;
 	bv->bv_val = ch_malloc(len);
 	ptr = (unsigned char *)bv->bv_val;
@@ -529,11 +593,23 @@ int entry_encode(Entry *e, struct berval *bv)
 		    entry_putlen(&ptr, i);
 		    for (i=0; a->a_vals[i].bv_val; i++) {
 			entry_putlen(&ptr, a->a_vals[i].bv_len);
-			memcpy(ptr, a->a_vals[i].bv_val,
+			AC_MEMCPY(ptr, a->a_vals[i].bv_val,
 				a->a_vals[i].bv_len);
 			ptr += a->a_vals[i].bv_len;
 			*ptr++ = '\0';
 		    }
+		    if (a->a_nvals != a->a_vals) {
+		    	entry_putlen(&ptr, i);
+			for (i=0; a->a_nvals[i].bv_val; i++) {
+			    entry_putlen(&ptr, a->a_nvals[i].bv_len);
+			    AC_MEMCPY(ptr, a->a_nvals[i].bv_val,
+				a->a_nvals[i].bv_len);
+			    ptr += a->a_nvals[i].bv_len;
+			    *ptr++ = '\0';
+			}
+		    } else {
+		    	entry_putlen(&ptr, 0);
+		    }
 		}
 	}
 	*ptr = '\0';
@@ -542,7 +618,7 @@ int entry_encode(Entry *e, struct berval *bv)
 
 /* Retrieve an Entry that was stored using entry_encode above.
  * We malloc a single block with the size stored above for the Entry
- * and all if its Attributes. We also must lookup the stored
+ * and all of its Attributes. We also must lookup the stored
  * attribute names to get AttributeDescriptions. To detect if the
  * attributes of an Entry are later modified, we note that e->e_attr
  * is always a constant offset from (e).
@@ -553,7 +629,7 @@ int entry_encode(Entry *e, struct berval *bv)
  */
 int entry_decode(struct berval *bv, Entry **e)
 {
-	int i, j;
+	int i, j, count;
 	int rc;
 	Attribute *a;
 	Entry *x;
@@ -565,16 +641,15 @@ int entry_decode(struct berval *bv, Entry **e)
 	i = entry_getlen(&ptr);
 	x = ch_calloc(1, i);
 	i = entry_getlen(&ptr);
-	x->e_name.bv_val = ptr;
+	x->e_name.bv_val = (char *) ptr;
 	x->e_name.bv_len = i;
 	ptr += i+1;
 	i = entry_getlen(&ptr);
-	x->e_nname.bv_val = ptr;
+	x->e_nname.bv_val = (char *) ptr;
 	x->e_nname.bv_len = i;
 	ptr += i+1;
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL2,
-		"entry_decode: \"%s\"\n", x->e_dn ));
+	LDAP_LOG( OPERATION, DETAIL2, "entry_decode: \"%s\"\n", x->e_dn, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE,
 	    "entry_decode: \"%s\"\n",
@@ -589,10 +664,10 @@ int entry_decode(struct berval *bv, Entry **e)
 	bptr = (BerVarray)x->e_attrs;
 	a = NULL;
 
-	while (i = entry_getlen(&ptr)) {
+	while ((i = entry_getlen(&ptr))) {
 		struct berval bv;
 		bv.bv_len = i;
-		bv.bv_val = ptr;
+		bv.bv_val = (char *) ptr;
 		if (a) {
 			a->a_next = (Attribute *)bptr;
 		}
@@ -602,8 +677,8 @@ int entry_decode(struct berval *bv, Entry **e)
 
 		if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-				"entry_decode: str2ad(%s): %s\n", ptr, text ));
+			LDAP_LOG( OPERATION, INFO, 
+				"entry_decode: str2ad(%s): %s\n", ptr, text, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE,
 				"<= entry_decode: str2ad(%s): %s\n", ptr, text, 0 );
@@ -612,8 +687,8 @@ int entry_decode(struct berval *bv, Entry **e)
 
 			if( rc != LDAP_SUCCESS ) {
 #ifdef NEW_LOGGING
-				LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-					"entry_decode:  str2undef_ad(%s): %s\n", ptr, text));
+				LDAP_LOG( OPERATION, INFO, 
+					"entry_decode:  str2undef_ad(%s): %s\n", ptr, text, 0 );
 #else
 				Debug( LDAP_DEBUG_ANY,
 					"<= entry_decode: str2undef_ad(%s): %s\n",
@@ -627,7 +702,7 @@ int entry_decode(struct berval *bv, Entry **e)
 		bptr = (BerVarray)(a+1);
 		a->a_vals = bptr;
 		a->a_flags = 0;
-		j = entry_getlen(&ptr);
+		count = j = entry_getlen(&ptr);
 
 		while (j) {
 			i = entry_getlen(&ptr);
@@ -640,12 +715,29 @@ int entry_decode(struct berval *bv, Entry **e)
 		bptr->bv_val = NULL;
 		bptr->bv_len = 0;
 		bptr++;
+
+		j = entry_getlen(&ptr);
+		if (j) {
+			a->a_nvals = bptr;
+			while (j) {
+				i = entry_getlen(&ptr);
+				bptr->bv_len = i;
+				bptr->bv_val = (char *)ptr;
+				ptr += i+1;
+				bptr++;
+				j--;
+			}
+			bptr->bv_val = NULL;
+			bptr->bv_len = 0;
+			bptr++;
+		} else {
+			a->a_nvals = a->a_vals;
+		}
 	}
-	if (a)
-		a->a_next = NULL;
+
+	if (a) a->a_next = NULL;
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_DETAIL1,
-		"entry_decode:  %s\n", x->e_dn ));
+	LDAP_LOG( OPERATION, DETAIL1, "entry_decode:  %s\n", x->e_dn, 0, 0 );
 #else
 	Debug(LDAP_DEBUG_TRACE, "<= entry_decode(%s)\n",
 		x->e_dn, 0, 0 );
@@ -653,4 +745,3 @@ int entry_decode(struct berval *bv, Entry **e)
 	*e = x;
 	return 0;
 }
-#endif
