@@ -1,13 +1,12 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/main.c,v 1.132 2002/01/09 23:17:21 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/main.c,v 1.157 2003/05/14 23:12:17 ando Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 #include "portable.h"
 
 #include <stdio.h>
 
-#include <ac/signal.h>
 #include <ac/socket.h>
 #include <ac/string.h>
 #include <ac/time.h>
@@ -21,6 +20,10 @@
 #include "lutil.h"
 #include "ldif.h"
 
+#ifdef LDAP_SLAPI
+#include "slapi.h"
+#endif
+
 #ifdef LDAP_SIGCHLD
 static RETSIGTYPE wait4child( int sig );
 #endif
@@ -29,15 +32,10 @@ static RETSIGTYPE wait4child( int sig );
 #define MAIN_RETURN(x) return
 static struct sockaddr_in	bind_addr;
 
-void CommenceStartupProcessing( LPCTSTR serverName,
-							   void(*stopper)(int));
-void ReportSlapdShutdownComplete( void );
-void *getRegParam( char *svc, char *value );
-
 #define SERVICE_EXIT( e, n )	do { \
 	if ( is_NT_Service ) { \
-		SLAPDServiceStatus.dwWin32ExitCode				= (e); \
-		SLAPDServiceStatus.dwServiceSpecificExitCode	= (n); \
+		lutil_ServiceStatus.dwWin32ExitCode				= (e); \
+		lutil_ServiceStatus.dwServiceSpecificExitCode	= (n); \
 	} \
 } while ( 0 )
 
@@ -46,11 +44,6 @@ void *getRegParam( char *svc, char *value );
 #define MAIN_RETURN(x) return(x)
 #endif
 
-#ifdef HAVE_NT_EVENT_LOG
-void LogSlapdStartedEvent( char *svc, int slap_debug, char *configfile, char *urls );
-void LogSlapdStoppedEvent( char *svc );
-#endif
-
 /*
  * when more than one slapd is running on one machine, each one might have
  * it's own LOCAL for syslogging and must have its own pid/args files
@@ -82,13 +75,14 @@ static STRDISP	syslog_types[] = {
 	{ "LOCAL5", sizeof("LOCAL5"), LOG_LOCAL5 },
 	{ "LOCAL6", sizeof("LOCAL6"), LOG_LOCAL6 },
 	{ "LOCAL7", sizeof("LOCAL7"), LOG_LOCAL7 },
-	{ NULL }
+	{ NULL, 0, 0 }
 };
 
 static int   cnvt_str2int( char *, STRDISP_P, int );
 
 #endif	/* LOG_LOCAL4 */
 
+static int check_config = 0;
 
 static void
 usage( char *name )
@@ -96,22 +90,25 @@ usage( char *name )
 	fprintf( stderr,
 		"usage: %s options\n", name );
 	fprintf( stderr,
-		"\t-d level\tDebug Level" "\n"
-		"\t-f filename\tConfiguration File\n"
+		"\t-4\t\tIPv4 only\n"
+		"\t-6\t\tIPv6 only\n"
+		"\t-d level\tDebug level" "\n"
+		"\t-f filename\tConfiguration file\n"
 #if defined(HAVE_SETUID) && defined(HAVE_SETGID)
-		"\t-g group\tGroup (id or name) to ran as\n"
+		"\t-g group\tGroup (id or name) to run as\n"
 #endif
-		"\t-h URLs\tList of URLs to serve\n"
+		"\t-h URLs\t\tList of URLs to serve\n"
 #ifdef LOG_LOCAL4
-		"\t-l sysloguser\tSyslog User (default: LOCAL4)\n"
+		"\t-l facility\tSyslog facility (default: LOCAL4)\n"
 #endif
-		"\t-n serverName\tservice name\n"
+		"\t-n serverName\tService name\n"
 #ifdef HAVE_CHROOT
-		"\t-r directory\n"
+		"\t-r directory\tSandbox directory to chroot to\n"
 #endif
-		"\t-s level\tSyslog Level\n"
+		"\t-s level\tSyslog level\n"
+		"\t-t\t\tCheck configuration file and exit\n"
 #if defined(HAVE_SETUID) && defined(HAVE_SETGID)
-		"\t-u user\tUser (id or name) to ran as\n"
+		"\t-u user\t\tUser (id or name) to run as\n"
 #endif
     );
 }
@@ -154,6 +151,7 @@ int main( int argc, char **argv )
 	}
 #endif
 
+	sl_mem_init();
 
 #ifdef HAVE_NT_SERVICE_MANAGER
 	{
@@ -164,24 +162,26 @@ int main( int argc, char **argv )
 
 		if ( is_NT_Service ) {
 			serverName = argv[0];
-			CommenceStartupProcessing( serverName, slap_sig_shutdown );
+			lutil_CommenceStartupProcessing( serverName, slap_sig_shutdown );
 			if ( strcmp(serverName, SERVICE_NAME) )
 			    regService = serverName;
 		}
 
-		i = (int*)getRegParam( regService, "DebugLevel" );
+		i = (int*)lutil_getRegParam( regService, "DebugLevel" );
 		if ( i != NULL ) 
 		{
 			slap_debug = *i;
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-				   "main: new debug level from registry is: %d\n", slap_debug ));
+			lutil_log_initialize( argc, argv );
+			LDAP_LOG( SLAPD, INFO, 
+				"main: new debug level from registry is: %d\n", 
+				slap_debug, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_ANY, "new debug level from registry is: %d\n", slap_debug, 0, 0 );
 #endif
 		}
 
-		newUrls = (char *) getRegParam(regService, "Urls");
+		newUrls = (char *) lutil_getRegParam(regService, "Urls");
 		if (newUrls)
 		{
 		    if (urls)
@@ -189,8 +189,8 @@ int main( int argc, char **argv )
 
 		    urls = ch_strdup(newUrls);
 #ifdef NEW_LOGGING
-		    LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-			       "main: new urls from registry: %s\n", urls ));
+		    LDAP_LOG( SLAPD, INFO, 
+				"main: new urls from registry: %s\n", urls, 0, 0 );
 #else
 		    Debug(LDAP_DEBUG_ANY, "new urls from registry: %s\n",
 			  urls, 0, 0);
@@ -198,13 +198,13 @@ int main( int argc, char **argv )
 
 		}
 
-		newConfigFile = (char*)getRegParam( regService, "ConfigFile" );
+		newConfigFile = (char*)lutil_getRegParam( regService, "ConfigFile" );
 		if ( newConfigFile != NULL ) 
 		{
 			configfile = newConfigFile;
 #ifdef NEW_LOGGING
-			LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-				   "main: new config file from registry is: %s\n", configfile ));
+			LDAP_LOG( SLAPD, INFO, 
+				"main: new config file from registry is: %s\n", configfile, 0, 0 );
 #else
 			Debug ( LDAP_DEBUG_ANY, "new config file from registry is: %s\n", configfile, 0, 0 );
 #endif
@@ -214,7 +214,10 @@ int main( int argc, char **argv )
 #endif
 
 	while ( (i = getopt( argc, argv,
-			     "d:f:h:s:n:"
+			     "d:f:h:s:n:t"
+#if LDAP_PF_INET6
+				"46"
+#endif
 #ifdef HAVE_CHROOT
 				"r:"
 #endif
@@ -226,6 +229,15 @@ int main( int argc, char **argv )
 #endif
 			     )) != EOF ) {
 		switch ( i ) {
+#ifdef LDAP_PF_INET6
+		case '4':
+			slap_inet4or6 = AF_INET;
+			break;
+		case '6':
+			slap_inet4or6 = AF_INET6;
+			break;
+#endif
+
 		case 'h':	/* listen URLs */
 			if ( urls != NULL ) free( urls );
 			urls = ch_strdup( optarg );
@@ -281,6 +293,10 @@ int main( int argc, char **argv )
 			serverName = ch_strdup( optarg );
 			break;
 
+		case 't':
+			check_config++;
+			break;
+
 		default:
 			usage( argv[0] );
 			rc = 1;
@@ -291,21 +307,19 @@ int main( int argc, char **argv )
 
 #ifdef NEW_LOGGING
 	lutil_log_initialize( argc, argv );
-#endif
-
+#else
 	lutil_set_debug_level( "slapd", slap_debug );
 	ber_set_option(NULL, LBER_OPT_DEBUG_LEVEL, &slap_debug);
 	ldap_set_option(NULL, LDAP_OPT_DEBUG_LEVEL, &slap_debug);
 	ldif_debug = slap_debug;
+#endif
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-		   "%s", Versionstr ));
+	LDAP_LOG( SLAPD, INFO, "%s", Versionstr, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_TRACE, "%s", Versionstr, 0, 0 );
 #endif
 
-
 	if( serverName == NULL ) {
 		if ( (serverName = strrchr( argv[0], *LDAP_DIRSEP )) == NULL ) {
 			serverName = argv[0];
@@ -320,7 +334,7 @@ int main( int argc, char **argv )
 	openlog( serverName, OPENLOG_OPTIONS );
 #endif
 
-	if( slapd_daemon_init( urls ) != 0 ) {
+	if( !check_config && slapd_daemon_init( urls ) != 0 ) {
 		rc = 1;
 		SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 16 );
 		goto stop;
@@ -348,6 +362,8 @@ int main( int argc, char **argv )
 #endif
 
 	extops_init();
+	lutil_passwd_init();
+	slap_op_init();
 
 #ifdef SLAPD_MODULES
 	if ( module_init() != 0 ) {
@@ -365,8 +381,7 @@ int main( int argc, char **argv )
 
 	if ( slap_schema_init( ) != 0 ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-			   "main: schema initialization error\n" ));
+		LDAP_LOG( OPERATION, CRIT, "main: schema initialization error\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "schema initialization error\n",
@@ -376,16 +391,62 @@ int main( int argc, char **argv )
 		goto destroy;
 	}
 
-	if ( read_config( configfile ) != 0 ) {
+	if ( slap_controls_init( ) != 0 ) {
+#ifdef NEW_LOGGING
+		LDAP_LOG( OPERATION, CRIT, "main: controls initialization error\n", 0, 0, 0 );
+#else
+		Debug( LDAP_DEBUG_ANY,
+		    "controls initialization error\n",
+		    0, 0, 0 );
+#endif
+
+		goto destroy;
+	}
+
+#ifdef HAVE_TLS
+	/* Library defaults to full certificate checking. This is correct when
+	 * a client is verifying a server because all servers should have a
+	 * valid cert. But few clients have valid certs, so we want our default
+	 * to be no checking. The config file can override this as usual.
+	 */
+	rc = 0;
+	(void) ldap_pvt_tls_set_option( NULL, LDAP_OPT_X_TLS_REQUIRE_CERT, &rc );
+#endif
+
+#ifdef LDAP_SLAPI
+	if ( slapi_init() != 0 ) {
+#ifdef NEW_LOGGING
+		LDAP_LOG( OPERATION, CRIT, "main: slapi initialization error\n", 0, 0, 0 );
+#else
+		Debug( LDAP_DEBUG_ANY,
+		    "slapi initialization error\n",
+		    0, 0, 0 );
+#endif
+
+		goto destroy;
+	}
+#endif /* LDAP_SLAPI */
+
+	if ( read_config( configfile, 0 ) != 0 ) {
 		rc = 1;
 		SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 19 );
+
+		if ( check_config ) {
+			fprintf( stderr, "config check failed\n" );
+		}
+
+		goto destroy;
+	}
+
+	if ( check_config ) {
+		rc = 0;
+		fprintf( stderr, "config check succeeded\n" );
 		goto destroy;
 	}
 
 	if ( glue_sub_init( ) != 0 ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-			   "main: subordinate config error\n"));
+		LDAP_LOG( SLAPD, CRIT, "main: subordinate config error\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "subordinate config error\n",
@@ -396,8 +457,7 @@ int main( int argc, char **argv )
 
 	if ( slap_schema_check( ) != 0 ) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-			   "main: schema prep error\n"));
+		LDAP_LOG( SLAPD, CRIT, "main: schema prep error\n", 0, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "schema prep error\n",
@@ -411,8 +471,7 @@ int main( int argc, char **argv )
 	rc = ldap_pvt_tls_init();
 	if( rc != 0) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-			   "main: tls init failed: %d\n", rc ));
+		LDAP_LOG( SLAPD, CRIT, "main: tls init failed: %d\n", rc, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "main: TLS init failed: %d\n",
@@ -426,12 +485,11 @@ int main( int argc, char **argv )
 	rc = ldap_pvt_tls_init_def_ctx();
 	if( rc != 0) {
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-			   "main: tls init def ctx failed: %d\n", rc ));
+		LDAP_LOG( SLAPD, CRIT, "main: tls init def ctx failed: %d\n", rc, 0, 0 );
 #else
 		Debug( LDAP_DEBUG_ANY,
 		    "main: TLS init def ctx failed: %d\n",
-		    0, 0, 0 );
+		    rc, 0, 0 );
 #endif
 		rc = 1;
 		SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 20 );
@@ -466,44 +524,56 @@ int main( int argc, char **argv )
 	mal_leaktrace(1);
 #endif
 
+	/*
+	 * FIXME: moved here from slapd_daemon_task()
+	 * because back-monitor db_open() needs it
+	 */
+	time( &starttime );
+
 	if ( slap_startup( NULL )  != 0 ) {
 		rc = 1;
 		SERVICE_EXIT( ERROR_SERVICE_SPECIFIC_ERROR, 21 );
 		goto shutdown;
 	}
 
-	{
-		FILE *fp;
-
 #ifdef NEW_LOGGING
-		LDAP_LOG(( "operation", LDAP_LEVEL_INFO,
-			   "main: slapd starting.\n" ));
+	LDAP_LOG( SLAPD, INFO, "main: slapd starting.\n", 0, 0, 0 );
 #else
-		Debug( LDAP_DEBUG_ANY, "slapd starting\n", 0, 0, 0 );
+	Debug( LDAP_DEBUG_ANY, "slapd starting\n", 0, 0, 0 );
 #endif
 
 
-		if (( slapd_pid_file != NULL ) &&
-			(( fp = fopen( slapd_pid_file, "w" )) != NULL ))
-		{
+	if ( slapd_pid_file != NULL ) {
+		FILE *fp = fopen( slapd_pid_file, "w" );
+
+		if( fp != NULL ) {
 			fprintf( fp, "%d\n", (int) getpid() );
 			fclose( fp );
-		}
 
-		if (( slapd_args_file != NULL ) &&
-			(( fp = fopen( slapd_args_file, "w" )) != NULL ))
-		{
+		} else {
+			free(slapd_pid_file);
+			slapd_pid_file = NULL;
+		}
+	}
+
+	if ( slapd_args_file != NULL ) {
+		FILE *fp = fopen( slapd_args_file, "w" );
+
+		if( fp != NULL ) {
 			for ( i = 0; i < g_argc; i++ ) {
 				fprintf( fp, "%s ", g_argv[i] );
 			}
 			fprintf( fp, "\n" );
 			fclose( fp );
+		} else {
+			free(slapd_args_file);
+			slapd_args_file = NULL;
 		}
 	}
 
 #ifdef HAVE_NT_EVENT_LOG
 	if (is_NT_Service)
-	LogSlapdStartedEvent( serverName, slap_debug, configfile, urls );
+	lutil_LogStartedEvent( serverName, slap_debug, configfile, urls );
 #endif
 
 	rc = slapd_daemon();
@@ -526,24 +596,25 @@ destroy:
 	module_kill();
 #endif
 
+	slap_op_destroy();
+
 	extops_kill();
 
 stop:
 #ifdef HAVE_NT_EVENT_LOG
 	if (is_NT_Service)
-	LogSlapdStoppedEvent( serverName );
+	lutil_LogStoppedEvent( serverName );
 #endif
 
 #ifdef NEW_LOGGING
-	LDAP_LOG(( "operation", LDAP_LEVEL_CRIT,
-		   "main: slapd stopped.\n" ));
+	LDAP_LOG( SLAPD, CRIT, "main: slapd stopped.\n", 0, 0, 0 );
 #else
 	Debug( LDAP_DEBUG_ANY, "slapd stopped.\n", 0, 0, 0 );
 #endif
 
 
 #ifdef HAVE_NT_SERVICE_MANAGER
-	ReportSlapdShutdownComplete();
+	lutil_ReportShutdownComplete();
 #endif
 
 #ifdef LOG_DEBUG
@@ -551,12 +622,23 @@ stop:
 #endif
 	slapd_daemon_destroy();
 
+	controls_destroy();
+
 	schema_destroy();
 
+	lutil_passwd_destroy();
+
 #ifdef HAVE_TLS
 	ldap_pvt_tls_destroy();
 #endif
 
+	if ( slapd_pid_file != NULL ) {
+		unlink( slapd_pid_file );
+	}
+	if ( slapd_args_file != NULL ) {
+		unlink( slapd_args_file );
+	}
+
 	config_destroy();
 
 #ifdef CSRIMALLOC
@@ -594,7 +676,7 @@ wait4child( int sig )
     errno = save_errno;
 }
 
-#endif /* SIGCHLD || SIGCLD */
+#endif /* LDAP_SIGCHLD */
 
 
 #ifdef LOG_LOCAL4
