@@ -1,12 +1,13 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/proto-slap.h,v 1.329 2002/02/15 15:27:31 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/proto-slap.h,v 1.429.2.2 2003/05/31 22:14:11 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 #ifndef PROTO_SLAP_H
 #define PROTO_SLAP_H
 
 #include <ldap_cdefs.h>
+#include "ldap_pvt.h"
 
 LDAP_BEGIN_DECL
 
@@ -14,13 +15,12 @@ LDAP_BEGIN_DECL
  * acl.c
  */
 LDAP_SLAPD_F (int) access_allowed LDAP_P((
-	Backend *be, Connection *conn, Operation *op,
+	Operation *op,
 	Entry *e, AttributeDescription *desc, struct berval *val,
 	slap_access_t access,
 	AccessControlState *state ));
 LDAP_SLAPD_F (int) acl_check_modlist LDAP_P((
-	Backend *be, Connection *conn, Operation *op,
-	Entry *e, Modifications *ml ));
+	Operation *op, Entry *e, Modifications *ml ));
 
 LDAP_SLAPD_F (void) acl_append( AccessControl **l, AccessControl *a );
 
@@ -53,9 +53,6 @@ LDAP_SLAPD_F (int) slap_bv2ad LDAP_P((
 	AttributeDescription **ad,
 	const char **text ));
 
-LDAP_SLAPD_F (AttributeDescription *) ad_dup LDAP_P((
-	AttributeDescription *desc ));
-
 LDAP_SLAPD_F (void) ad_destroy LDAP_P(( AttributeDescription * ));
 
 #define ad_cmp(l,r)	(((l)->ad_cname.bv_len < (r)->ad_cname.bv_len) \
@@ -80,13 +77,25 @@ LDAP_SLAPD_F (int) slap_bv2undef_ad LDAP_P((
 	AttributeDescription **ad,
 	const char **text ));
 
-LDAP_SLAPD_F (AttributeDescription *) ad_find_lang LDAP_P((
+LDAP_SLAPD_F (AttributeDescription *) ad_find_tags LDAP_P((
 	AttributeType *type,
-	struct berval *lang ));
+	struct berval *tags ));
 
 LDAP_SLAPD_F (AttributeName *) str2anlist LDAP_P(( AttributeName *an,
 	char *str, const char *brkstr ));
-LDAP_SLAPD_F (int) an_find LDAP_P(( AttributeName *a, struct berval *s ));	
+LDAP_SLAPD_F (int) an_find LDAP_P(( AttributeName *a, struct berval *s ));
+LDAP_SLAPD_F (int) ad_define_option LDAP_P(( const char *name,
+	const char *fname, int lineno ));
+
+LDAP_SLAPD_F (MatchingRule *) ad_mr(
+	AttributeDescription *ad,
+	unsigned usage );
+
+/*
+ * add.c
+ */
+LDAP_SLAPD_F (int) slap_mods2entry LDAP_P(( Modifications *mods, Entry **e,
+	int repl_user, const char **text, char *textbuf, size_t textlen ));
 
 /*
  * at.c
@@ -117,15 +126,32 @@ LDAP_SLAPD_F (int) is_at_syntax LDAP_P((
 	AttributeType *at,
 	const char *oid ));
 
+LDAP_SLAPD_F (int) at_start LDAP_P(( AttributeType **at ));
+LDAP_SLAPD_F (int) at_next LDAP_P(( AttributeType **at ));
+
 /*
  * attr.c
  */
 LDAP_SLAPD_F (void) attr_free LDAP_P(( Attribute *a ));
 LDAP_SLAPD_F (Attribute *) attr_dup LDAP_P(( Attribute *a ));
 
+#define attr_mergeit( e, d, v ) attr_merge( e, d, v, NULL /* FIXME */ )
+#define attr_mergeit_one( e, d, v ) attr_merge_one( e, d, v, NULL /* FIXME */ )
+
 LDAP_SLAPD_F (int) attr_merge LDAP_P(( Entry *e,
 	AttributeDescription *desc,
-	BerVarray vals ));
+	BerVarray vals,
+	BerVarray nvals ));
+LDAP_SLAPD_F (int) attr_merge_one LDAP_P(( Entry *e,
+	AttributeDescription *desc,
+	struct berval *val,
+	struct berval *nval ));
+LDAP_SLAPD_F (int) attr_merge_normalize LDAP_P(( Entry *e,
+	AttributeDescription *desc,
+	BerVarray vals, void *memctx ));
+LDAP_SLAPD_F (int) attr_merge_normalize_one LDAP_P(( Entry *e,
+	AttributeDescription *desc,
+	struct berval *val, void *memctx ));
 LDAP_SLAPD_F (Attribute *) attrs_find LDAP_P((
 	Attribute *a, AttributeDescription *desc ));
 LDAP_SLAPD_F (Attribute *) attr_find LDAP_P((
@@ -141,11 +167,13 @@ LDAP_SLAPD_F (Attribute *) attrs_dup LDAP_P(( Attribute *a ));
  * ava.c
  */
 LDAP_SLAPD_F (int) get_ava LDAP_P((
+	Operation *op,
 	BerElement *ber,
 	AttributeAssertion **ava,
 	unsigned usage,
 	const char **text ));
 LDAP_SLAPD_F (void) ava_free LDAP_P((
+	Operation *op,
 	AttributeAssertion *ava,
 	int freeit ));
 
@@ -172,36 +200,31 @@ LDAP_SLAPD_F (int) be_issuffix LDAP_P(( Backend *be,
 	struct berval *suffix ));
 LDAP_SLAPD_F (int) be_isroot LDAP_P(( Backend *be,
 	struct berval *ndn ));
-LDAP_SLAPD_F (int) be_isroot_pw LDAP_P(( Backend *be,
-	Connection *conn, struct berval *ndn, struct berval *cred ));
+LDAP_SLAPD_F (int) be_isroot_pw LDAP_P(( Operation *op ));
 LDAP_SLAPD_F (int) be_isupdate LDAP_P(( Backend *be, struct berval *ndn ));
 LDAP_SLAPD_F (struct berval *) be_root_dn LDAP_P(( Backend *be ));
+LDAP_SLAPD_F (int) be_entry_get_rw LDAP_P(( struct slap_op *o,
+		struct berval *ndn, ObjectClass *oc,
+		AttributeDescription *at, int rw, Entry **e ));
 LDAP_SLAPD_F (int) be_entry_release_rw LDAP_P((
-	BackendDB *be, Connection *c, Operation *o, Entry *e, int rw ));
-#define be_entry_release_r( be, c, o, e ) be_entry_release_rw( be, c, o, e, 0 )
-#define be_entry_release_w( be, c, o, e ) be_entry_release_rw( be, c, o, e, 1 )
-
-LDAP_SLAPD_F (int) backend_unbind LDAP_P((Connection *conn, Operation *op));
-
-LDAP_SLAPD_F( int )	backend_check_restrictions LDAP_P((
-	BackendDB *be,
-	Connection *conn,
-	Operation *op,
-	const void *opdata,
-	const char **text ));
-
-LDAP_SLAPD_F( int )	backend_check_referrals LDAP_P((
-	BackendDB *be,
-	Connection *conn,
-	Operation *op,
-	struct berval *dn,
-	struct berval *ndn ));
+	Operation *o, Entry *e, int rw ));
+#define be_entry_release_r( o, e ) be_entry_release_rw( o, e, 0 )
+#define be_entry_release_w( o, e ) be_entry_release_rw( o, e, 1 )
 
+LDAP_SLAPD_F (int) backend_unbind LDAP_P((Operation *op, SlapReply *rs));
 LDAP_SLAPD_F (int) backend_connection_init LDAP_P((Connection *conn));
 LDAP_SLAPD_F (int) backend_connection_destroy LDAP_P((Connection *conn));
 
-LDAP_SLAPD_F (int) backend_group LDAP_P((BackendDB *be,
-	Connection *conn,
+LDAP_SLAPD_F( int )	backend_check_restrictions LDAP_P((
+	Operation *op,
+	SlapReply *rs,
+	struct berval *opdata ));
+
+LDAP_SLAPD_F( int )	backend_check_referrals LDAP_P((
+	Operation *op,
+	SlapReply *rs ));
+
+LDAP_SLAPD_F (int) backend_group LDAP_P((
 	Operation *op,
 	Entry *target,
 	struct berval *gr_ndn,
@@ -210,8 +233,7 @@ LDAP_SLAPD_F (int) backend_group LDAP_P((BackendDB *be,
 	AttributeDescription *group_at
 ));
 
-LDAP_SLAPD_F (int) backend_attribute LDAP_P((BackendDB *be,
-	Connection *conn,
+LDAP_SLAPD_F (int) backend_attribute LDAP_P((
 	Operation *op,
 	Entry *target,
 	struct berval *entry_ndn,
@@ -220,11 +242,8 @@ LDAP_SLAPD_F (int) backend_attribute LDAP_P((BackendDB *be,
 ));
 
 LDAP_SLAPD_F (Attribute *) backend_operational(
-	BackendDB *,
-	Connection *conn,
 	Operation *op,
-	Entry *e,
-	AttributeName *attrs,
+	SlapReply *rs,
 	int opattrs );
 
 /*
@@ -237,14 +256,7 @@ LDAP_SLAPD_F (int) glue_sub_init( void );
 /*
  * ch_malloc.c
  */
-#ifdef CSRIMALLOC
-#define ch_malloc malloc
-#define ch_realloc realloc
-#define ch_calloc calloc
-#define ch_strdup strdup
-#define ch_free free
-
-#else
+LDAP_SLAPD_V (BerMemoryFunctions) ch_mfuncs;
 LDAP_SLAPD_F (void *) ch_malloc LDAP_P(( ber_len_t size ));
 LDAP_SLAPD_F (void *) ch_realloc LDAP_P(( void *block, ber_len_t size ));
 LDAP_SLAPD_F (void *) ch_calloc LDAP_P(( ber_len_t nelem, ber_len_t size ));
@@ -255,40 +267,31 @@ LDAP_SLAPD_F (void) ch_free LDAP_P(( void * ));
 #undef free
 #define free ch_free
 #endif
-#endif
-
-/*
- * charray.c
- */
-LDAP_SLAPD_F (void) charray_add LDAP_P(( char ***a, const char *s ));
-LDAP_SLAPD_F (void) charray_add_n LDAP_P(( char ***a, const char *s, int l ));
-LDAP_SLAPD_F (void) charray_merge LDAP_P(( char ***a, char **s ));
-LDAP_SLAPD_F (void) charray_free LDAP_P(( char **array ));
-LDAP_SLAPD_F (int) charray_inlist LDAP_P(( char **a, const char *s ));
-LDAP_SLAPD_F (char **) charray_dup LDAP_P(( char **a ));
-LDAP_SLAPD_F (char **) str2charray LDAP_P(( const char *str, const char *brkstr ));
-LDAP_SLAPD_F (int) charray_strcmp LDAP_P(( const char **a1, const char **a2 ));
-LDAP_SLAPD_F (int) charray_strcasecmp LDAP_P(( const char **a1, const char **a2 ));
-
-LDAP_SLAPD_F (char *) slap_strcopy LDAP_P((
-	char *dst, const char *src ));
-LDAP_SLAPD_F (char *) slap_strncopy LDAP_P((
-	char *dst, const char *src, size_t n ));
 
 /*
  * controls.c
  */
-LDAP_SLAPD_F (int) get_ctrls LDAP_P((
-	Connection *co,
+LDAP_SLAPD_F (void) slap_free_ctrls LDAP_P((
 	Operation *op,
+	LDAPControl **ctrls ));
+LDAP_SLAPD_F (int) get_ctrls LDAP_P((
+	Operation *op,
+	SlapReply *rs,
 	int senderrors ));
-
-LDAP_SLAPD_F (char *) get_supported_ctrl LDAP_P((int index));
+LDAP_SLAPD_F (int) register_supported_control LDAP_P((
+	const char *controloid,
+	slap_mask_t controlmask,
+	char **controlexops,
+	SLAP_CTRL_PARSE_FN *controlparsefn ));
+LDAP_SLAPD_F (int) slap_controls_init LDAP_P ((void));
+LDAP_SLAPD_F (void) controls_destroy LDAP_P ((void));
+LDAP_SLAPD_F (int) controls_root_dse_info LDAP_P ((Entry *e));
+LDAP_SLAPD_F (int) get_supported_controls LDAP_P (( char ***ctrloidsp, slap_mask_t **ctrlmasks ));
 
 /*
  * config.c
  */
-LDAP_SLAPD_F (int) read_config LDAP_P(( const char *fname ));
+LDAP_SLAPD_F (int) read_config LDAP_P(( const char *fname, int depth ));
 LDAP_SLAPD_F (void) config_destroy LDAP_P ((void));
 
 /*
@@ -301,10 +304,9 @@ LDAP_SLAPD_F (int) connections_timeout_idle LDAP_P((time_t));
 
 LDAP_SLAPD_F (long) connection_init LDAP_P((
 	ber_socket_t s,
-	const char* url,
+	Listener* url,
 	const char* dnsname,
 	const char* peername,
-	const char* sockname,
 	int use_tls,
 	slap_ssf_t ssf,
 	const char *id ));
@@ -326,6 +328,22 @@ LDAP_SLAPD_F (void) connection_done LDAP_P((Connection *));
 
 LDAP_SLAPD_F (void) connection2anonymous LDAP_P((Connection *));
 
+/*
+ * cr.c
+ */
+LDAP_SLAPD_F (int) cr_schema_info( Entry *e );
+
+LDAP_SLAPD_F (int) cr_add LDAP_P((
+	LDAPContentRule *oc,
+	int user,
+	const char **err));
+LDAP_SLAPD_F (void) cr_destroy LDAP_P(( void ));
+
+LDAP_SLAPD_F (ContentRule *) cr_find LDAP_P((
+	const char *crname));
+LDAP_SLAPD_F (ContentRule *) cr_bvfind LDAP_P((
+	struct berval *crname));
+
 /*
  * daemon.c
  */
@@ -349,38 +367,22 @@ LDAP_SLAPD_F (void) slapd_clr_read LDAP_P((ber_socket_t s, int wake));
  */
 
 #define dn_match(dn1, dn2) 	( ber_bvcmp((dn1), (dn2)) == 0 )
-
-LDAP_SLAPD_V( const struct berval ) slap_empty_bv;
+#define bvmatch(bv1, bv2)	( ((bv1)->bv_len == (bv2)->bv_len) && (memcmp((bv1)->bv_val, (bv2)->bv_val, (bv1)->bv_len) == 0) )
 
 LDAP_SLAPD_F (int) dnValidate LDAP_P((
 	Syntax *syntax, 
 	struct berval *val ));
 
-LDAP_SLAPD_F (int) dnNormalize LDAP_P((
-	Syntax *syntax, 
-	struct berval *val, 
-	struct berval **normalized ));
+LDAP_SLAPD_F (slap_mr_normalize_func) dnNormalize;
 
-LDAP_SLAPD_F (int) dnNormalize2 LDAP_P((
-	Syntax *syntax, 
-	struct berval *val, 
-	struct berval *normalized ));
-
-LDAP_SLAPD_F (int) dnPretty LDAP_P(( 
-	Syntax *syntax, 
-	struct berval *val, 
-	struct berval **pretty ));
-
-LDAP_SLAPD_F (int) dnPretty2 LDAP_P(( 
-	Syntax *syntax, 
-	struct berval *val, 
-	struct berval *pretty ));
+LDAP_SLAPD_F (slap_syntax_transform_func) dnPretty;
 
 LDAP_SLAPD_F (int) dnPrettyNormal LDAP_P(( 
 	Syntax *syntax, 
 	struct berval *val, 
 	struct berval *pretty,
-	struct berval *normal ));
+	struct berval *normal,
+	void *ctx ));
 
 LDAP_SLAPD_F (int) dnMatch LDAP_P(( 
 	int *matchp, 
@@ -394,7 +396,7 @@ LDAP_SLAPD_F (int) dnIsSuffix LDAP_P((
 	const struct berval *dn, const struct berval *suffix ));
 
 LDAP_SLAPD_F (int) dnExtractRdn LDAP_P((
-	struct berval *dn, struct berval *rdn ));
+	struct berval *dn, struct berval *rdn, void *ctx ));
 
 LDAP_SLAPD_F (int) rdnValidate LDAP_P(( struct berval * rdn ));
 
@@ -407,6 +409,17 @@ LDAP_SLAPD_F (void) build_new_dn LDAP_P((
 
 LDAP_SLAPD_F (void) dnParent LDAP_P(( struct berval *dn, struct berval *pdn ));
 
+LDAP_SLAPD_F (int) dnX509normalize LDAP_P(( void *x509_name, struct berval *out ));
+
+LDAP_SLAPD_F (int) dnX509peerNormalize LDAP_P(( void *ssl, struct berval *dn ));
+
+LDAP_SLAPD_F (int) dnPrettyNormalDN LDAP_P(( Syntax *syntax, struct berval *val, LDAPDN *dn, int flags, void *ctx ));
+#define dnPrettyDN(syntax, val, dn, ctx) \
+	dnPrettyNormalDN((syntax),(val),(dn), SLAP_LDAPDN_PRETTY, ctx)
+#define dnNormalDN(syntax, val, dn, ctx) \
+	dnPrettyNormalDN((syntax),(val),(dn), 0, ctx)
+
+
 /*
  * entry.c
  */
@@ -417,32 +430,36 @@ LDAP_SLAPD_F (int) entry_destroy LDAP_P((void));
 LDAP_SLAPD_F (Entry *) str2entry LDAP_P(( char	*s ));
 LDAP_SLAPD_F (char *) entry2str LDAP_P(( Entry *e, int *len ));
 
+LDAP_SLAPD_F (void) entry_flatsize LDAP_P((
+	Entry *e, ber_len_t *siz, ber_len_t *len, int norm ));
 LDAP_SLAPD_F (int) entry_decode LDAP_P(( struct berval *bv, Entry **e ));
 LDAP_SLAPD_F (int) entry_encode LDAP_P(( Entry *e, struct berval *bv ));
 
 LDAP_SLAPD_F (void) entry_free LDAP_P(( Entry *e ));
 LDAP_SLAPD_F (int) entry_cmp LDAP_P(( Entry *a, Entry *b ));
-LDAP_SLAPD_F (int) entry_dn_cmp LDAP_P(( Entry *a, Entry *b ));
-LDAP_SLAPD_F (int) entry_id_cmp LDAP_P(( Entry *a, Entry *b ));
+LDAP_SLAPD_F (int) entry_dn_cmp LDAP_P(( const void *v_a, const void *v_b ));
+LDAP_SLAPD_F (int) entry_id_cmp LDAP_P(( const void *v_a, const void *v_b ));
 
 /*
  * extended.c
  */
-typedef int (SLAP_EXTOP_MAIN_FN) LDAP_P((
-	Connection *conn, Operation *op,
-	const char * reqoid,
-	struct berval * reqdata,
-	char ** rspoid,
-	struct berval ** rspdata,
-	LDAPControl *** rspctrls,
-	const char ** text,
-	BerVarray *refs ));
+LDAP_SLAPD_F (int) exop_root_dse_info LDAP_P ((Entry *e));
+
+#ifdef LDAP_EXOP_X_CANCEL
+LDAP_SLAPD_V( const struct berval ) slap_EXOP_CANCEL;
+#endif
+LDAP_SLAPD_V( const struct berval ) slap_EXOP_WHOAMI;
+LDAP_SLAPD_V( const struct berval ) slap_EXOP_MODIFY_PASSWD;
+LDAP_SLAPD_V( const struct berval ) slap_EXOP_START_TLS;
+
+typedef int (SLAP_EXTOP_MAIN_FN) LDAP_P(( Operation *op, SlapReply *rs ));
 
 typedef int (SLAP_EXTOP_GETOID_FN) LDAP_P((
-	int index, char *oid, int blen ));
+	int index, struct berval *oid, int blen ));
 
 LDAP_SLAPD_F (int) load_extop LDAP_P((
-	const char *ext_oid,
+	struct berval *ext_oid,
+	slap_mask_t flags,
 	SLAP_EXTOP_MAIN_FN *ext_main ));
 
 LDAP_SLAPD_F (int) extops_init LDAP_P(( void ));
@@ -451,25 +468,50 @@ LDAP_SLAPD_F (int) extops_kill LDAP_P(( void ));
 
 LDAP_SLAPD_F (struct berval *) get_supported_extop LDAP_P((int index));
 
+/*
+ *  * cancel.c
+ *   */
+LDAP_SLAPD_F ( SLAP_EXTOP_MAIN_FN ) cancel_extop;
+
 /*
  * filter.c
  */
 LDAP_SLAPD_F (int) get_filter LDAP_P((
-	Connection *conn,
+	Operation *op,
 	BerElement *ber,
 	Filter **filt,
-	struct berval *fstr,
 	const char **text ));
 
 LDAP_SLAPD_F (void) filter_free LDAP_P(( Filter *f ));
-LDAP_SLAPD_F (void) filter_print LDAP_P(( Filter *f ));
+LDAP_SLAPD_F (void) filter_free_x LDAP_P(( Operation *op, Filter *f ));
+LDAP_SLAPD_F (void) filter2bv LDAP_P(( Filter *f, struct berval *bv ));
+LDAP_SLAPD_F (void) filter2bv_x LDAP_P(( Operation *op, Filter *f, struct berval *bv ));
+
+LDAP_SLAPD_F (int) get_vrFilter LDAP_P(( Operation *op, BerElement *ber,
+	ValuesReturnFilter **f,
+	const char **text ));
+
+LDAP_SLAPD_F (void) vrFilter_free LDAP_P(( Operation *op, ValuesReturnFilter *f ));
+LDAP_SLAPD_F (void) vrFilter2bv LDAP_P(( Operation *op, ValuesReturnFilter *f, struct berval *fstr ));
+
+LDAP_SLAPD_F (int) filter_has_subordinates LDAP_P(( Filter *filter ));
+LDAP_SLAPD_F (int) filter_escape_value LDAP_P(( struct berval *in, 
+	struct berval *out ));
 
 /*
  * filterentry.c
  */
-LDAP_SLAPD_F (int) test_filter LDAP_P((
-	Backend *be, Connection *conn, Operation *op,
-	Entry *e, Filter *f ));
+
+LDAP_SLAPD_F (int) test_filter LDAP_P(( Operation *op, Entry *e, Filter *f ));
+
+/*
+ * globals.c
+ */
+
+LDAP_SLAPD_V( const struct berval ) slap_empty_bv;
+LDAP_SLAPD_V( const struct berval ) slap_unknown_bv;
+LDAP_SLAPD_V( const struct berval ) slap_true_bv;
+LDAP_SLAPD_V( const struct berval ) slap_false_bv;
 
 /*
  * index.c
@@ -484,6 +526,8 @@ LDAP_SLAPD_F (int)	slap_startup LDAP_P(( Backend *be ));
 LDAP_SLAPD_F (int)	slap_shutdown LDAP_P(( Backend *be ));
 LDAP_SLAPD_F (int)	slap_destroy LDAP_P((void));
 
+LDAP_SLAPD_V (char **)	slap_known_controls;
+
 /*
  * kerberos.c
  */
@@ -511,6 +555,25 @@ LDAP_SLAPD_F (FILE *) lock_fopen LDAP_P(( const char *fname,
 	const char *type, FILE **lfp ));
 LDAP_SLAPD_F (int) lock_fclose LDAP_P(( FILE *fp, FILE *lfp ));
 
+/*
+ * matchedValues.c
+ */
+LDAP_SLAPD_F (int) filter_matched_values( 
+	Operation	*op,
+	Attribute	*a,
+	char		***e_flags );
+
+/*
+ * modrdn.c
+ */
+LDAP_SLAPD_F (int) slap_modrdn2mods(
+	Operation	*op,
+	SlapReply	*rs,
+	Entry		*e,
+	LDAPRDN		oldrdn,
+	LDAPRDN		newrdn,
+	Modifications	**pmod );
+
 /*
  * modify.c
  */
@@ -518,7 +581,7 @@ LDAP_SLAPD_F( int ) slap_mods_check(
 	Modifications *ml,
 	int update,
 	const char **text,
-	char *textbuf, size_t textlen );
+	char *textbuf, size_t textlen, void *ctx );
 
 LDAP_SLAPD_F( int ) slap_mods_opattrs(
 	Operation *op,
@@ -530,14 +593,21 @@ LDAP_SLAPD_F( int ) slap_mods_opattrs(
 /*
  * mods.c
  */
+LDAP_SLAPD_F( int ) modify_check_duplicates(
+	AttributeDescription *ad, MatchingRule *mr, 
+	BerVarray vals, BerVarray mods, int permissive, 
+	const char **text, char *textbuf, size_t textlen );
 LDAP_SLAPD_F( int ) modify_add_values( Entry *e,
 	Modification *mod,
+	int permissive,
 	const char **text, char *textbuf, size_t textlen );
 LDAP_SLAPD_F( int ) modify_delete_values( Entry *e,
 	Modification *mod,
+	int permissive,
 	const char **text, char *textbuf, size_t textlen );
 LDAP_SLAPD_F( int ) modify_replace_values( Entry *e,
 	Modification *mod,
+	int permissive,
 	const char **text, char *textbuf, size_t textlen );
 
 LDAP_SLAPD_F( void ) slap_mod_free( Modification *mod, int freeit );
@@ -571,42 +641,40 @@ LDAP_SLAPD_F (void) *module_resolve LDAP_P((
 LDAP_SLAPD_F (MatchingRule *) mr_bvfind LDAP_P((struct berval *mrname));
 LDAP_SLAPD_F (MatchingRule *) mr_find LDAP_P((const char *mrname));
 LDAP_SLAPD_F (int) mr_add LDAP_P(( LDAPMatchingRule *mr,
-	unsigned usage,
-	slap_mr_convert_func *convert,
-	slap_mr_normalize_func *normalize,
-	slap_mr_match_func *match,
-	slap_mr_indexer_func *indexer,
-	slap_mr_filter_func *filter,
+	slap_mrule_defs_rec *def,
 	MatchingRule * associated,
 	const char **err ));
 LDAP_SLAPD_F (void) mr_destroy LDAP_P(( void ));
 
 LDAP_SLAPD_F (int) register_matching_rule LDAP_P((
-	const char * desc,
-	unsigned usage,
-	slap_mr_convert_func *convert,
-	slap_mr_normalize_func *normalize,
-	slap_mr_match_func *match,
-	slap_mr_indexer_func *indexer,
-	slap_mr_filter_func *filter,
-	const char *associated ));
+	slap_mrule_defs_rec *def ));
+
+LDAP_SLAPD_F (void) mru_destroy LDAP_P(( void ));
+LDAP_SLAPD_F (int) matching_rule_use_init LDAP_P(( void ));
 
 LDAP_SLAPD_F (int) mr_schema_info( Entry *e );
+LDAP_SLAPD_F (int) mru_schema_info( Entry *e );
+
+LDAP_SLAPD_F (int) mr_usable_with_at( MatchingRule *mr,
+	AttributeType *at );
 
 /*
  * mra.c
  */
 LDAP_SLAPD_F (int) get_mra LDAP_P((
+	Operation *op,
 	BerElement *ber,
 	MatchingRuleAssertion **mra,
 	const char **text ));
 LDAP_SLAPD_F (void) mra_free LDAP_P((
+	Operation *op,
 	MatchingRuleAssertion *mra,
 	int freeit ));
 
 /* oc.c */
 LDAP_SLAPD_F (int) oc_add LDAP_P((
 	LDAPObjectClass *oc,
+	int user,
 	const char **err));
 LDAP_SLAPD_F (void) oc_destroy LDAP_P(( void ));
 
@@ -621,20 +689,25 @@ LDAP_SLAPD_F (int) is_object_subclass LDAP_P((
 LDAP_SLAPD_F (int) is_entry_objectclass LDAP_P((
 	Entry *, ObjectClass *oc, int set_flags ));
 #define is_entry_alias(e)		\
-	(((e)->e_ocflags & SLAP_OC__END) ? ((e)->e_ocflags & SLAP_OC_ALIAS) : \
-	is_entry_objectclass((e), slap_schema.si_oc_alias, 1))
+	(((e)->e_ocflags & SLAP_OC__END) \
+	 ? (((e)->e_ocflags & SLAP_OC_ALIAS) != 0) \
+	 : is_entry_objectclass((e), slap_schema.si_oc_alias, 1))
 #define is_entry_referral(e)	\
-	(((e)->e_ocflags & SLAP_OC__END) ? ((e)->e_ocflags & SLAP_OC_REFERRAL) : \
-	is_entry_objectclass((e), slap_schema.si_oc_referral, 1))
+	(((e)->e_ocflags & SLAP_OC__END) \
+	 ? (((e)->e_ocflags & SLAP_OC_REFERRAL) != 0) \
+	 : is_entry_objectclass((e), slap_schema.si_oc_referral, 1))
 #define is_entry_subentry(e)	\
-	(((e)->e_ocflags & SLAP_OC__END) ? ((e)->e_ocflags & SLAP_OC_SUBENTRY) : \
-	is_entry_objectclass((e), slap_schema.si_oc_subentry, 1))
+	(((e)->e_ocflags & SLAP_OC__END) \
+	 ? (((e)->e_ocflags & SLAP_OC_SUBENTRY) != 0) \
+	 : is_entry_objectclass((e), slap_schema.si_oc_subentry, 1))
 #define is_entry_collectiveAttributeSubentry(e)	\
-	(((e)->e_ocflags & SLAP_OC__END) ? ((e)->e_ocflags & SLAP_OC_COLLECTIVEATTRIBUTESUBENTRY) : \
-	is_entry_objectclass((e), slap_schema.si_oc_collectiveAttributeSubentry, 1))
+	(((e)->e_ocflags & SLAP_OC__END) \
+	 ? (((e)->e_ocflags & SLAP_OC_COLLECTIVEATTRIBUTESUBENTRY) != 0) \
+	 : is_entry_objectclass((e), slap_schema.si_oc_collectiveAttributeSubentry, 1))
 #define is_entry_dynamicObject(e)	\
-	(((e)->e_ocflags & SLAP_OC__END) ? ((e)->e_ocflags & SLAP_OC_DYNAMICOBJECT) : \
-	is_entry_objectclass((e), slap_schema.si_oc_dynamicObject, 1))
+	(((e)->e_ocflags & SLAP_OC__END) \
+	 ? (((e)->e_ocflags & SLAP_OC_DYNAMICOBJECT) != 0) \
+	 : is_entry_objectclass((e), slap_schema.si_oc_dynamicObject, 1))
 
 LDAP_SLAPD_F (int) oc_schema_info( Entry *e );
 
@@ -649,6 +722,8 @@ LDAP_SLAPD_F (int) parse_oidm LDAP_P((
 /*
  * operation.c
  */
+LDAP_SLAPD_F (void) slap_op_init LDAP_P(( void ));
+LDAP_SLAPD_F (void) slap_op_destroy LDAP_P(( void ));
 LDAP_SLAPD_F (void) slap_op_free LDAP_P(( Operation *op ));
 LDAP_SLAPD_F (Operation *) slap_op_alloc LDAP_P((
 	BerElement *ber, ber_int_t msgid,
@@ -661,7 +736,7 @@ LDAP_SLAPD_F (Operation *) slap_op_pop LDAP_P(( Operation **olist ));
 /*
  * operational.c
  */
-LDAP_SLAPD_F (Attribute *) slap_operational_subschemaSubentry( void );
+LDAP_SLAPD_F (Attribute *) slap_operational_subschemaSubentry( Backend *be );
 LDAP_SLAPD_F (Attribute *) slap_operational_hasSubordinate( int has );
 
 /*
@@ -672,13 +747,15 @@ LDAP_SLAPD_F (SLAP_EXTOP_MAIN_FN) passwd_extop;
 LDAP_SLAPD_F (int) slap_passwd_check(
 	Connection			*conn,
 	Attribute			*attr,
-	struct berval		*cred );
+	struct berval		*cred,
+	const char			**text );
 
 LDAP_SLAPD_F (void) slap_passwd_generate( struct berval * );
 
 LDAP_SLAPD_F (void) slap_passwd_hash(
 	struct berval		*cred,
-	struct berval		*hash );
+	struct berval		*hash,
+	const char		**text );
 
 LDAP_SLAPD_F (struct berval *) slap_passwd_return(
 	struct berval		*cred );
@@ -702,7 +779,7 @@ LDAP_SLAPD_F (int) validate_global_referral LDAP_P((
 	const char *url ));
 
 LDAP_SLAPD_F (BerVarray) get_entry_referrals LDAP_P((
-	Backend *be, Connection *conn, Operation *op, Entry *e ));
+	Operation *op, Entry *e ));
 
 LDAP_SLAPD_F (BerVarray) referral_rewrite LDAP_P((
 	BerVarray refs,
@@ -710,6 +787,12 @@ LDAP_SLAPD_F (BerVarray) referral_rewrite LDAP_P((
 	struct berval *target,
 	int scope ));
 
+LDAP_SLAPD_F (int) get_alias_dn LDAP_P((
+	Entry *e,
+	struct berval *ndn,
+	int *err,
+	const char **text ));
+
 /*
  * repl.c
  */
@@ -718,60 +801,20 @@ LDAP_SLAPD_F (int) add_replica_info LDAP_P(( Backend *be,
 LDAP_SLAPD_F (int) add_replica_suffix LDAP_P(( Backend *be,
 	int nr, const char *suffix ));
 LDAP_SLAPD_F (int) add_replica_attrs LDAP_P(( Backend *be,
-	int nr, char *attrs ));
-LDAP_SLAPD_F (void) replog LDAP_P(( Backend *be, Operation *op,
-	struct berval *dn, struct berval *ndn, void *change ));
+	int nr, char *attrs, int exclude ));
+LDAP_SLAPD_F (void) replog LDAP_P(( Operation *op ));
 
 /*
  * result.c
  */
-LDAP_SLAPD_F (void) send_ldap_result LDAP_P((
-	Connection *conn, Operation *op,
-	ber_int_t err, const char *matched, const char *text,
-	BerVarray refs,
-	LDAPControl **ctrls ));
-
-LDAP_SLAPD_F (void) send_ldap_sasl LDAP_P((
-	Connection *conn, Operation *op,
-	ber_int_t err, const char *matched,
-	const char *text,
-	BerVarray refs,
-	LDAPControl **ctrls,
-	struct berval *cred ));
-
-LDAP_SLAPD_F (void) send_ldap_disconnect LDAP_P((
-	Connection *conn, Operation *op,
-	ber_int_t err, const char *text ));
-
-LDAP_SLAPD_F (void) send_ldap_extended LDAP_P((
-	Connection *conn, Operation *op,
-	ber_int_t err, const char *matched,
-	const char *text, BerVarray refs,
-	const char *rspoid, struct berval *rspdata,
-	LDAPControl **ctrls ));
-
-LDAP_SLAPD_F (void) send_ldap_partial LDAP_P((
-	Connection *conn, Operation *op,
-	const char *rspoid, struct berval *rspdata,
-	LDAPControl **ctrls ));
-
-LDAP_SLAPD_F (void) send_search_result LDAP_P((
-	Connection *conn, Operation *op,
-	ber_int_t err, const char *matched, const char *text,
-	BerVarray refs,
-	LDAPControl **ctrls,
-	int nentries ));
-
-LDAP_SLAPD_F (int) send_search_reference LDAP_P((
-	Backend *be, Connection *conn, Operation *op,
-	Entry *e, BerVarray refs,
-	LDAPControl **ctrls,
-	BerVarray *v2refs ));
-
-LDAP_SLAPD_F (int) send_search_entry LDAP_P((
-	Backend *be, Connection *conn, Operation *op,
-	Entry *e, AttributeName *attrs, int attrsonly,
-	LDAPControl **ctrls ));
+LDAP_SLAPD_F (void) slap_send_ldap_result LDAP_P(( Operation *op, SlapReply *rs ));
+LDAP_SLAPD_F (void) send_ldap_sasl LDAP_P(( Operation *op, SlapReply *rs ));
+LDAP_SLAPD_F (void) send_ldap_disconnect LDAP_P(( Operation *op, SlapReply *rs ));
+LDAP_SLAPD_F (void) slap_send_ldap_extended LDAP_P(( Operation *op, SlapReply *rs ));
+LDAP_SLAPD_F (void) slap_send_ldap_intermediate LDAP_P(( Operation *op, SlapReply *rs ));
+LDAP_SLAPD_F (void) slap_send_search_result LDAP_P(( Operation *op, SlapReply *rs ));
+LDAP_SLAPD_F (int) slap_send_search_reference LDAP_P(( Operation *op, SlapReply *rs ));
+LDAP_SLAPD_F (int) slap_send_search_entry LDAP_P(( Operation *op, SlapReply *rs ));
 
 LDAP_SLAPD_F (int) str2result LDAP_P(( char *s,
 	int *code, char **matched, char **info ));
@@ -794,7 +837,7 @@ LDAP_SLAPD_F (int) slap_sasl_init(void);
 LDAP_SLAPD_F (char *) slap_sasl_secprops( const char * );
 LDAP_SLAPD_F (int) slap_sasl_destroy(void);
 
-LDAP_SLAPD_F (int) slap_sasl_open( Connection *c );
+LDAP_SLAPD_F (int) slap_sasl_open( Connection *c, int reopen );
 LDAP_SLAPD_F (char **) slap_sasl_mechs( Connection *c );
 
 LDAP_SLAPD_F (int) slap_sasl_external( Connection *c,
@@ -804,23 +847,38 @@ LDAP_SLAPD_F (int) slap_sasl_external( Connection *c,
 LDAP_SLAPD_F (int) slap_sasl_reset( Connection *c );
 LDAP_SLAPD_F (int) slap_sasl_close( Connection *c );
 
-LDAP_SLAPD_F (int) slap_sasl_bind LDAP_P((
-	Connection *conn, Operation *op, 
-	struct berval *dn, struct berval *ndn,
-	struct berval *cred,
-	struct berval *edn, slap_ssf_t *ssf ));
+LDAP_SLAPD_F (int) slap_sasl_bind LDAP_P(( Operation *op, SlapReply *rs ));
+
+LDAP_SLAPD_F (int) slap_sasl_setpass(
+	Operation       *op,
+	SlapReply	*rs );
+
+LDAP_SLAPD_F (int) slap_sasl_config(
+	int cargc,
+	char **cargv,
+	char *line,
+	const char *fname,
+	int lineno );
+
+LDAP_SLAPD_F (int) slap_sasl_getdn( Connection *conn, Operation *op,
+	char *id, int len,
+	char *user_realm, struct berval *dn, int flags );
 
 /*
  * saslauthz.c
  */
 LDAP_SLAPD_F (void) slap_sasl2dn LDAP_P((
+	Operation *op,
 	struct berval *saslname,
 	struct berval *dn ));
 LDAP_SLAPD_F (int) slap_sasl_authorized LDAP_P((
+	Operation *op,
 	struct berval *authcid,
 	struct berval *authzid ));
 LDAP_SLAPD_F (int) slap_sasl_regexp_config LDAP_P((
 	const char *match, const char *replace ));
+LDAP_SLAPD_F (int) slap_sasl_setpolicy LDAP_P(( const char * ));
+
 
 /*
  * schema.c
@@ -860,6 +918,10 @@ LDAP_SLAPD_V( int ) schema_init_done;
 LDAP_SLAPD_F (int) slap_schema_init LDAP_P((void));
 LDAP_SLAPD_F (void) schema_destroy LDAP_P(( void ));
 
+LDAP_SLAPD_F( slap_mr_indexer_func ) octetStringIndexer;
+
+LDAP_SLAPD_F( slap_mr_filter_func ) octetStringFilter;
+
 /*
  * schema_prep.c
  */
@@ -872,8 +934,8 @@ LDAP_SLAPD_F (int) slap_schema_check LDAP_P((void));
  */
 LDAP_SLAPD_F( int ) slap_valid_descr( const char * );
 
-LDAP_SLAPD_F (int) parse_oc_old LDAP_P((
-	Backend *be, const char *fname, int lineno, int argc, char **argv ));
+LDAP_SLAPD_F (int) parse_cr LDAP_P((
+	const char *fname, int lineno, char *line, char **argv ));
 LDAP_SLAPD_F (int) parse_oc LDAP_P((
 	const char *fname, int lineno, char *line, char **argv ));
 LDAP_SLAPD_F (int) parse_at LDAP_P((
@@ -882,6 +944,22 @@ LDAP_SLAPD_F (char *) scherr2str LDAP_P((int code)) LDAP_GCCATTR((const));
 LDAP_SLAPD_F (int) dscompare LDAP_P(( const char *s1, const char *s2del,
 	char delim ));
 
+/*
+ * sl_malloc.c
+ */
+LDAP_SLAPD_V (BerMemoryFunctions) sl_mfuncs;
+LDAP_SLAPD_F (void *) sl_malloc LDAP_P(( ber_len_t size, void *ctx ));
+LDAP_SLAPD_F (void *) sl_realloc LDAP_P(( void *block, ber_len_t size, void *ctx ));
+LDAP_SLAPD_F (void *) sl_calloc LDAP_P(( ber_len_t nelem, ber_len_t size, void *ctx ));
+LDAP_SLAPD_F (void) sl_free LDAP_P(( void *, void *ctx ));
+LDAP_SLAPD_F (void) sl_mem_init LDAP_P(( void ));
+LDAP_SLAPD_F (void *) sl_mem_create LDAP_P(( ber_len_t size, void *ctx ));
+LDAP_SLAPD_F (void) sl_mem_detach LDAP_P(( void *ctx, void *memctx ));
+LDAP_SLAPD_F (void) sl_mem_destroy LDAP_P(( void *key, void *data ));
+LDAP_SLAPD_F (void *) sl_mark LDAP_P(( void *ctx ));
+LDAP_SLAPD_F (void) sl_release LDAP_P(( void *, void *ctx ));
+LDAP_SLAPD_F (void *) sl_context LDAP_P(( void *ptr ));
+
 /*
  * starttls.c
  */
@@ -891,44 +969,21 @@ LDAP_SLAPD_F (SLAP_EXTOP_MAIN_FN) starttls_extop;
  * str2filter.c
  */
 LDAP_SLAPD_F (Filter *) str2filter LDAP_P(( const char *str ));
-
-/*
- * suffixalias.c
- */
-LDAP_SLAPD_F (void) suffix_alias LDAP_P(( Backend *be, struct berval *ndn ));
+LDAP_SLAPD_F (Filter *) str2filter_x LDAP_P(( Operation *op, const char *str ));
 
 /* syntax.c */
 LDAP_SLAPD_F (Syntax *) syn_find LDAP_P((
 	const char *synname ));
 LDAP_SLAPD_F (Syntax *) syn_find_desc LDAP_P((
 	const char *syndesc, int *slen ));
-#ifdef SLAPD_BINARY_CONVERSION
 LDAP_SLAPD_F (int) syn_add LDAP_P((
 	LDAPSyntax *syn,
-	unsigned flags,
-	slap_syntax_validate_func *validate,
-	slap_syntax_transform_func *normalize,
-	slap_syntax_transform_func *pretty,
-	slap_syntax_transform_func *ber2str,
-	slap_syntax_transform_func *str2ber,
+	slap_syntax_defs_rec *def,
 	const char **err ));
-#else
-LDAP_SLAPD_F (int) syn_add LDAP_P((
-	LDAPSyntax *syn,
-	unsigned flags,
-	slap_syntax_validate_func *validate,
-	slap_syntax_transform_func *normalize,
-	slap_syntax_transform_func *pretty,
-	const char **err ));
-#endif
 LDAP_SLAPD_F (void) syn_destroy LDAP_P(( void ));
 
 LDAP_SLAPD_F (int) register_syntax LDAP_P((
-	const char *desc,
-	unsigned flags,
-	slap_syntax_validate_func *validate,
-	slap_syntax_transform_func *normalize,
-	slap_syntax_transform_func *pretty ));
+	slap_syntax_defs_rec *def ));
 
 LDAP_SLAPD_F (int) syn_schema_info( Entry *e );
 
@@ -942,12 +997,15 @@ LDAP_SLAPD_F (void) slap_init_user LDAP_P(( char *username, char *groupname ));
 /*
  * value.c
  */
-LDAP_SLAPD_F (int) value_normalize LDAP_P((
+LDAP_SLAPD_F (int) asserted_value_validate_normalize LDAP_P((
 	AttributeDescription *ad,
+	MatchingRule *mr,
 	unsigned usage,
 	struct berval *in,
 	struct berval *out,
-	const char ** text ));
+	const char ** text,
+	void *ctx ));
+
 LDAP_SLAPD_F (int) value_match LDAP_P((
 	int *match,
 	AttributeDescription *ad,
@@ -956,15 +1014,22 @@ LDAP_SLAPD_F (int) value_match LDAP_P((
 	struct berval *v1,
 	void *v2,
 	const char ** text ));
-#define value_find(ad,values,value) (value_find_ex((ad),0,(values),(value)))
 LDAP_SLAPD_F (int) value_find_ex LDAP_P((
 	AttributeDescription *ad,
 	unsigned flags,
 	BerVarray values,
-	struct berval *value ));
+	struct berval *value,
+	void *ctx ));
+
 LDAP_SLAPD_F (int) value_add LDAP_P((
 	BerVarray *vals,
 	BerVarray addvals ));
+LDAP_SLAPD_F (int) value_add_one LDAP_P((
+	BerVarray *vals,
+	struct berval *addval ));
+
+/* assumes (x) > (y) returns 1 if true, 0 otherwise */
+#define SLAP_PTRCMP(x, y) ((x) < (y) ? -1 : (x) > (y))
 
 /*
  * Other...
@@ -973,6 +1038,8 @@ LDAP_SLAPD_V(unsigned) num_subordinates;
 
 LDAP_SLAPD_V (ber_len_t) sockbuf_max_incoming;
 LDAP_SLAPD_V (ber_len_t) sockbuf_max_incoming_auth;
+LDAP_SLAPD_V (int)		slap_conn_max_pending;
+LDAP_SLAPD_V (int)		slap_conn_max_pending_auth;
 
 LDAP_SLAPD_V (slap_mask_t)	global_restrictops;
 LDAP_SLAPD_V (slap_mask_t)	global_allows;
@@ -986,19 +1053,19 @@ LDAP_SLAPD_V (const char) 	Versionstr[];
 LDAP_SLAPD_V (struct slap_limits_set)		deflimit;
 
 LDAP_SLAPD_V (slap_access_t)	global_default_access;
+LDAP_SLAPD_V (int)		global_gentlehup;
 LDAP_SLAPD_V (int)		global_idletimeout;
 LDAP_SLAPD_V (int)		global_schemacheck;
 LDAP_SLAPD_V (char *)	global_host;
 LDAP_SLAPD_V (char *)	global_realm;
-LDAP_SLAPD_V (int)		sasl_external_x509dn_convert;
 LDAP_SLAPD_V (char *)	default_passwd_hash;
 LDAP_SLAPD_V (int)		lber_debug;
 LDAP_SLAPD_V (int)		ldap_syslog;
 LDAP_SLAPD_V (struct berval)	default_search_base;
 LDAP_SLAPD_V (struct berval)	default_search_nbase;
 
-LDAP_SLAPD_V (int)		nSaslRegexp;
-LDAP_SLAPD_V (SaslRegexp_t*) SaslRegexp;
+LDAP_SLAPD_V (struct berval)	global_schemadn;
+LDAP_SLAPD_V (struct berval)	global_schemandn;
 
 LDAP_SLAPD_V (ldap_pvt_thread_mutex_t)	num_sent_mutex;
 LDAP_SLAPD_V (unsigned long)		num_bytes_sent;
@@ -1009,6 +1076,10 @@ LDAP_SLAPD_V (unsigned long)		num_refs_sent;
 LDAP_SLAPD_V (ldap_pvt_thread_mutex_t)	num_ops_mutex;
 LDAP_SLAPD_V (unsigned long)		num_ops_completed;
 LDAP_SLAPD_V (unsigned long)		num_ops_initiated;
+#ifdef SLAPD_MONITOR
+LDAP_SLAPD_V (unsigned long)		num_ops_completed_[SLAP_OP_LAST];
+LDAP_SLAPD_V (unsigned long)		num_ops_initiated_[SLAP_OP_LAST];
+#endif /* SLAPD_MONITOR */
 
 LDAP_SLAPD_V (char *)		slapd_pid_file;
 LDAP_SLAPD_V (char *)		slapd_args_file;
@@ -1032,19 +1103,38 @@ LDAP_SLAPD_V (AccessControl *) global_acl;
 
 LDAP_SLAPD_V (ber_socket_t)	dtblsize;
 
+LDAP_SLAPD_V (int)		use_reverse_lookup;
+
+LDAP_SLAPD_V (struct berval)	AllUser;
+LDAP_SLAPD_V (struct berval)	AllOper;
+LDAP_SLAPD_V (struct berval)	NoAttrs;
+
 /*
  * operations
  */
-LDAP_SLAPD_F (int) do_abandon LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_add LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_bind LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_compare LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_delete LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_modify LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_modrdn LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_search LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_unbind LDAP_P((Connection *conn, Operation *op));
-LDAP_SLAPD_F (int) do_extended LDAP_P((Connection *conn, Operation *op));
+LDAP_SLAPD_F (int) do_abandon LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_add LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_bind LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_compare LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_delete LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_modify LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_modrdn LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_search LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_unbind LDAP_P((Operation *op, SlapReply *rs));
+LDAP_SLAPD_F (int) do_extended LDAP_P((Operation *op, SlapReply *rs));
+
+/*
+ * syncrepl
+ */
+
+#ifdef LDAP_SYNCREPL
+LDAP_SLAPD_V (struct runqueue_s) syncrepl_rq;
+
+LDAP_SLAPD_F (void) init_syncrepl LDAP_P(());
+LDAP_SLAPD_F (void*) do_syncrepl LDAP_P((void *, void *));
+
+LDAP_SLAPD_F (char **) str2clist( char **, char *, const char * );
+#endif
 
 LDAP_END_DECL
 
