@@ -1,7 +1,7 @@
 /* repl.c - log modifications for replication purposes */
-/* $OpenLDAP: pkg/ldap/servers/slapd/repl.c,v 1.40 2002/02/15 14:21:13 hyc Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/repl.c,v 1.49 2003/04/29 16:28:14 kurt Exp $ */
 /*
- * Copyright 1998-2002 The OpenLDAP Foundation, All Rights Reserved.
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
  * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
  */
 
@@ -41,6 +41,8 @@ add_replica_info(
 	be->be_replica[ i ] 
 		= ch_calloc( sizeof( struct slap_replica_info ), 1 );
 	be->be_replica[ i ]->ri_host = ch_strdup( host );
+	be->be_replica[ i ]->ri_nsuffix = NULL;
+	be->be_replica[ i ]->ri_attrs = NULL;
 	be->be_replica[ i + 1 ] = NULL;
 
 	return( i );
@@ -53,23 +55,23 @@ add_replica_suffix(
     const char  *suffix
 )
 {
-	struct berval dn, *ndn = NULL;
+	struct berval dn, ndn;
 	int rc;
 
 	dn.bv_val = (char *) suffix;
 	dn.bv_len = strlen( dn.bv_val );
 
-	rc = dnNormalize( NULL, &dn, &ndn );
+	rc = dnNormalize( 0, NULL, NULL, &dn, &ndn, NULL );
 	if( rc != LDAP_SUCCESS ) {
 		return 2;
 	}
 
-	if ( select_backend( ndn, 0, 0 ) != be ) {
-		ber_bvfree( ndn );
+	if ( select_backend( &ndn, 0, 0 ) != be ) {
+		free( ndn.bv_val );
 		return 1;
 	}
 
-	ber_bvecadd( &be->be_replica[nr]->ri_nsuffix, ndn );
+	ber_bvarray_add( &be->be_replica[nr]->ri_nsuffix, &ndn );
 	return 0;
 }
 
@@ -77,9 +79,19 @@ int
 add_replica_attrs(
 	Backend	*be,
 	int	nr,
-	char	*attrs
+	char	*attrs,
+	int	exclude
 )
 {
+	if ( be->be_replica[nr]->ri_attrs != NULL ) {
+		if ( be->be_replica[nr]->ri_exclude != exclude ) {
+			fprintf( stderr, "attr selective replication directive '%s' conflicts with previous one (discarded)\n", attrs );
+			ch_free( be->be_replica[nr]->ri_attrs );
+			be->be_replica[nr]->ri_attrs = NULL;
+		}
+	}
+
+	be->be_replica[nr]->ri_exclude = exclude;
 	be->be_replica[nr]->ri_attrs = str2anlist( be->be_replica[nr]->ri_attrs,
 		attrs, "," );
 	return ( be->be_replica[nr]->ri_attrs == NULL );
@@ -88,20 +100,13 @@ add_replica_attrs(
 static void
 print_vals( FILE *fp, struct berval *type, struct berval *bv );
 static void
-replog1( struct slap_replica_info *ri, Operation *op, void *change, FILE *fp, void *first);
+replog1( struct slap_replica_info *ri, Operation *op, FILE *fp, void *first);
 
 void
-replog(
-    Backend	*be,
-    Operation *op,
-    struct berval *dn,
-    struct berval *ndn,
-    void	*change
-)
+replog( Operation *op )
 {
 	Modifications	*ml = NULL;
 	Attribute	*a = NULL;
-	Entry	*e;
 	FILE	*fp, *lfp;
 	int	i;
 /* undef NO_LOG_WHEN_NO_REPLICAS */
@@ -111,35 +116,35 @@ replog(
 	int	subsets = 0;
 	long now = slap_get_time();
 
-	if ( be->be_replogfile == NULL && replogfile == NULL ) {
+	if ( op->o_bd->be_replogfile == NULL && replogfile == NULL ) {
 		return;
 	}
 
 	ldap_pvt_thread_mutex_lock( &replog_mutex );
-	if ( (fp = lock_fopen( be->be_replogfile ? be->be_replogfile :
+	if ( (fp = lock_fopen( op->o_bd->be_replogfile ? op->o_bd->be_replogfile :
 	    replogfile, "a", &lfp )) == NULL ) {
 		ldap_pvt_thread_mutex_unlock( &replog_mutex );
 		return;
 	}
 
-	for ( i = 0; be->be_replica != NULL && be->be_replica[i] != NULL; i++ ) {
+	for ( i = 0; op->o_bd->be_replica != NULL && op->o_bd->be_replica[i] != NULL; i++ ) {
 		/* check if dn's suffix matches legal suffixes, if any */
-		if ( be->be_replica[i]->ri_nsuffix != NULL ) {
+		if ( op->o_bd->be_replica[i]->ri_nsuffix != NULL ) {
 			int j;
 
-			for ( j = 0; be->be_replica[i]->ri_nsuffix[j]; j++ ) {
-				if ( dnIsSuffix( ndn, be->be_replica[i]->ri_nsuffix[j] ) ) {
+			for ( j = 0; op->o_bd->be_replica[i]->ri_nsuffix[j].bv_val; j++ ) {
+				if ( dnIsSuffix( &op->o_req_ndn, &op->o_bd->be_replica[i]->ri_nsuffix[j] ) ) {
 					break;
 				}
 			}
 
-			if ( !be->be_replica[i]->ri_nsuffix[j] ) {
+			if ( !op->o_bd->be_replica[i]->ri_nsuffix[j].bv_val ) {
 				/* do not add "replica:" line */
 				continue;
 			}
 		}
 		/* See if we only want a subset of attributes */
-		if ( be->be_replica[i]->ri_attrs != NULL &&
+		if ( op->o_bd->be_replica[i]->ri_attrs != NULL &&
 			( op->o_tag == LDAP_REQ_MODIFY || op->o_tag == LDAP_REQ_ADD || op->o_tag == LDAP_REQ_EXTENDED ) ) {
 			if ( !subsets ) {
 				subsets = i + 1;
@@ -148,7 +153,7 @@ replog(
 			continue;
 		}
 
-		fprintf( fp, "replica: %s\n", be->be_replica[i]->ri_host );
+		fprintf( fp, "replica: %s\n", op->o_bd->be_replica[i]->ri_host );
 #ifdef NO_LOG_WHEN_NO_REPLICAS
 		++count;
 #endif
@@ -166,30 +171,30 @@ replog(
 #endif
 
 	fprintf( fp, "time: %ld\n", now );
-	fprintf( fp, "dn: %s\n", dn->bv_val );
+	fprintf( fp, "dn: %s\n", op->o_req_dn.bv_val );
 
-	replog1( NULL, op, change, fp, NULL );
+	replog1( NULL, op, fp, NULL );
 
 	if ( subsets > 0 ) {
 		void *first;
-		for ( i = subsets - 1; be->be_replica != NULL && be->be_replica[i] != NULL; i++ ) {
+		for ( i = subsets - 1; op->o_bd->be_replica != NULL && op->o_bd->be_replica[i] != NULL; i++ ) {
 
 			/* If no attrs, we already did this above */
-			if ( be->be_replica[i]->ri_attrs == NULL ) {
+			if ( op->o_bd->be_replica[i]->ri_attrs == NULL ) {
 				continue;
 			}
 
 			/* check if dn's suffix matches legal suffixes, if any */
-			if ( be->be_replica[i]->ri_nsuffix != NULL ) {
+			if ( op->o_bd->be_replica[i]->ri_nsuffix != NULL ) {
 				int j;
 
-				for ( j = 0; be->be_replica[i]->ri_nsuffix[j]; j++ ) {
-					if ( dnIsSuffix( ndn, be->be_replica[i]->ri_nsuffix[j] ) ) {
+				for ( j = 0; op->o_bd->be_replica[i]->ri_nsuffix[j].bv_val; j++ ) {
+					if ( dnIsSuffix( &op->o_req_ndn, &op->o_bd->be_replica[i]->ri_nsuffix[j] ) ) {
 						break;
 					}
 				}
 
-				if ( !be->be_replica[i]->ri_nsuffix[j] ) {
+				if ( !op->o_bd->be_replica[i]->ri_nsuffix[j].bv_val ) {
 					/* do not add "replica:" line */
 					continue;
 				}
@@ -202,8 +207,17 @@ replog(
 				/* assume change parameter is a Modfications* */
 				/* fall thru */
 			case LDAP_REQ_MODIFY:
-				for ( ml = change; ml != NULL; ml = ml->sml_next ) {
-					if ( ad_inlist( ml->sml_desc, be->be_replica[i]->ri_attrs ) ) {
+				for ( ml = op->orm_modlist; ml != NULL; ml = ml->sml_next ) {
+					int is_in, exclude;
+
+   					is_in = ad_inlist( ml->sml_desc, op->o_bd->be_replica[i]->ri_attrs );
+					exclude = op->o_bd->be_replica[i]->ri_exclude;
+					
+					/*
+					 * there might be a more clever way to do this test,
+					 * but this way, at least, is comprehensible :)
+					 */
+					if ( ( is_in && !exclude ) || ( !is_in && exclude ) ) {
 						subsets = 1;
 						first = ml;
 						break;
@@ -214,9 +228,13 @@ replog(
 				}
 				break;
 			case LDAP_REQ_ADD:
-				e = change;
-				for ( a = e->e_attrs; a != NULL; a = a->a_next ) {
-					if ( ad_inlist( a->a_desc, be->be_replica[i]->ri_attrs ) ) {
+				for ( a = op->ora_e->e_attrs; a != NULL; a = a->a_next ) {
+					int is_in, exclude;
+
+   					is_in = ad_inlist( a->a_desc, op->o_bd->be_replica[i]->ri_attrs );
+					exclude = op->o_bd->be_replica[i]->ri_exclude;
+					
+					if ( ( is_in && !exclude ) || ( !is_in && exclude ) ) {
 						subsets = 1;
 						first = a;
 						break;
@@ -230,10 +248,10 @@ replog(
 				/* Other operations were logged in the first pass */
 				continue;
 			}
-			fprintf( fp, "replica: %s\n", be->be_replica[i]->ri_host );
+			fprintf( fp, "replica: %s\n", op->o_bd->be_replica[i]->ri_host );
 			fprintf( fp, "time: %ld\n", now );
-			fprintf( fp, "dn: %s\n", dn->bv_val );
-			replog1( be->be_replica[i], op, change, fp, first );
+			fprintf( fp, "dn: %s\n", op->o_req_dn.bv_val );
+			replog1( op->o_bd->be_replica[i], op, fp, first );
 		}
 	}
 
@@ -246,15 +264,12 @@ static void
 replog1(
     struct slap_replica_info *ri,
     Operation *op,
-    void	*change,
     FILE	*fp,
 	void	*first
 )
 {
 	Modifications	*ml;
 	Attribute	*a;
-	Entry		*e;
-	struct slap_replog_moddn *moddn;
 
 	switch ( op->o_tag ) {
 	case LDAP_REQ_EXTENDED:
@@ -264,11 +279,15 @@ replog1(
 
 	case LDAP_REQ_MODIFY:
 		fprintf( fp, "changetype: modify\n" );
-		ml = first ? first : change;
+		ml = first ? first : op->orm_modlist;
 		for ( ; ml != NULL; ml = ml->sml_next ) {
 			char *type;
-			if ( ri && ri->ri_attrs && !ad_inlist( ml->sml_desc, ri->ri_attrs ) ) {
-				continue;
+			if ( ri && ri->ri_attrs ) {
+				int is_in = ad_inlist( ml->sml_desc, ri->ri_attrs );
+
+				if ( ( !is_in && !ri->ri_exclude ) || ( is_in && ri->ri_exclude ) ) {
+					continue;
+				}
 			}
 			type = ml->sml_desc->ad_cname.bv_val;
 			switch ( ml->sml_op ) {
@@ -291,12 +310,12 @@ replog1(
 		break;
 
 	case LDAP_REQ_ADD:
-		e = change;
 		fprintf( fp, "changetype: add\n" );
-		a = first ? first : e->e_attrs;
+		a = first ? first : op->ora_e->e_attrs;
 		for ( ; a != NULL; a=a->a_next ) {
 			if ( ri && ri->ri_attrs ) {
-				if ( !ad_inlist( a->a_desc, ri->ri_attrs ) ) {
+				int is_in = ad_inlist( a->a_desc, ri->ri_attrs );
+				if ( ( !is_in && !ri->ri_exclude ) || ( is_in && ri->ri_exclude ) ) {
 					continue;
 				}
 				/* If the list includes objectClass names,
@@ -336,12 +355,11 @@ replog1(
 		break;
 
 	case LDAP_REQ_MODRDN:
-		moddn = change;
 		fprintf( fp, "changetype: modrdn\n" );
-		fprintf( fp, "newrdn: %s\n", moddn->newrdn->bv_val );
-		fprintf( fp, "deleteoldrdn: %d\n", moddn->deloldrdn ? 1 : 0 );
-		if( moddn->newsup != NULL ) {
-			fprintf( fp, "newsuperior: %s\n", moddn->newsup->bv_val );
+		fprintf( fp, "newrdn: %s\n", op->orr_newrdn.bv_val );
+		fprintf( fp, "deleteoldrdn: %d\n", op->orr_deleteoldrdn ? 1 : 0 );
+		if( op->orr_newSup != NULL ) {
+			fprintf( fp, "newsuperior: %s\n", op->orr_newSup->bv_val );
 		}
 	}
 	fprintf( fp, "\n" );
@@ -353,7 +371,7 @@ print_vals(
 	struct berval *type,
 	struct berval *bv )
 {
-	int i, len;
+	ber_len_t i, len;
 	char	*buf, *bufp;
 
 	for ( i = 0, len = 0; bv && bv[i].bv_val; i++ ) {
