@@ -1,3 +1,8 @@
+/* $OpenLDAP: pkg/ldap/tests/progs/slapd-tester.c,v 1.22 2003/02/22 11:01:59 hyc Exp $ */
+/*
+ * Copyright 1998-2003 The OpenLDAP Foundation, All Rights Reserved.
+ * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+ */
 #include "portable.h"
 
 #include <stdio.h>
@@ -6,14 +11,12 @@
 
 #include <ac/ctype.h>
 #include <ac/dirent.h>
+#include <ac/param.h>
 #include <ac/socket.h>
 #include <ac/string.h>
 #include <ac/unistd.h>
 #include <ac/wait.h>
 
-#ifdef HAVE_SYS_PARAM_H
-#include <sys/param.h>
-#endif
 
 #include "ldap_defaults.h"
 
@@ -21,6 +24,7 @@
 #define SEARCHCMD		"slapd-search"
 #define READCMD			"slapd-read"
 #define ADDCMD			"slapd-addel"
+#define MODRDNCMD		"slapd-modrdn"
 #define MAXARGS      	100
 #define MAXREQS			20
 #define LOOPS			"100"
@@ -28,27 +32,37 @@
 #define TSEARCHFILE		"do_search.0"
 #define TREADFILE		"do_read.0"
 #define TADDFILE		"do_add."
+#define TMODRDNFILE		"do_modrdn.0"
 
 static char *get_file_name( char *dirname, char *filename );
 static int  get_search_filters( char *filename, char *filters[] );
 static int  get_read_entries( char *filename, char *entries[] );
-static void fork_child( char *prog, char *args[] );
+static void fork_child( char *prog, char **args );
 static void	wait4kids( int nkidval );
 
 static int      maxkids = 20;
 static int      nkids;
 
+#ifdef HAVE_WINSOCK
+static HANDLE	*children;
+static char argbuf[BUFSIZ];
+#define	ArgDup(x) strdup(strcat(strcat(strcpy(argbuf,"\""),x),"\""))
+#else
+#define	ArgDup(x) strdup(x)
+#endif
+
 static void
 usage( char *name )
 {
 	fprintf( stderr, "usage: %s [-h <host>] -p <port> -D <manager> -w <passwd> -d <datadir> -b <baseDN> [-j <maxchild>] [-l <loops>] -P <progdir>\n", name );
-	exit( 1 );
+	exit( EXIT_FAILURE );
 }
 
 int
 main( int argc, char **argv )
 {
 	int		i, j;
+	char		*uri = NULL;
 	char		*host = "localhost";
 	char		*port = NULL;
 	char		*manager = NULL;
@@ -67,6 +81,9 @@ main( int argc, char **argv )
 	int         rnum = 0;
 	char		*afiles[MAXREQS];
 	int         anum = 0;
+	char		*mfile = NULL;
+	char		*mreqs[MAXREQS];
+	int		mnum = 0;
 	char		*sargs[MAXARGS];
 	int			sanum;
 	char		scmd[MAXPATHLEN];
@@ -76,9 +93,16 @@ main( int argc, char **argv )
 	char		*aargs[MAXARGS];
 	int			aanum;
 	char		acmd[MAXPATHLEN];
+	char		*margs[MAXARGS];
+	int		manum;
+	char		mcmd[MAXPATHLEN];
 
-	while ( (i = getopt( argc, argv, "h:p:D:w:b:d:j:l:P:" )) != EOF ) {
+	while ( (i = getopt( argc, argv, "H:h:p:D:w:b:d:j:l:P:" )) != EOF ) {
 		switch( i ) {
+			case 'H':		/* slapd uri */
+				uri = strdup( optarg );
+			break;
+				
 			case 'h':		/* slapd host */
 				host = strdup( optarg );
 			break;
@@ -88,15 +112,15 @@ main( int argc, char **argv )
 				break;
 
 			case 'D':		/* slapd manager */
-				manager = strdup( optarg );
+				manager = ArgDup( optarg );
 			break;
 
 			case 'w':		/* the managers passwd */
-				passwd = strdup( optarg );
+				passwd = ArgDup( optarg );
 				break;
 
 			case 'b':		/* the base DN */
-				sbase = strdup( optarg );
+				sbase = ArgDup( optarg );
 				break;
 
 			case 'd':		/* data directory */
@@ -121,20 +145,23 @@ main( int argc, char **argv )
 		}
 	}
 
-	if (( dirname == NULL ) || ( sbase == NULL ) || ( port == NULL ) ||
+	if (( dirname == NULL ) || ( sbase == NULL ) || ( port == NULL && uri == NULL ) ||
 			( manager == NULL ) || ( passwd == NULL ) || ( progdir == NULL ))
 		usage( argv[0] );
 
+#ifdef HAVE_WINSOCK
+	children = malloc( maxkids * sizeof(HANDLE) );
+#endif
 	/* get the file list */
 	if ( ( datadir = opendir( dirname )) == NULL ) {
 
 		fprintf( stderr, "%s: couldn't open data directory \"%s\".\n",
 					argv[0], dirname );
-		exit( 1 );
+		exit( EXIT_FAILURE );
 
 	}
 
-	/*  look for search, read, and add/delete files */
+	/*  look for search, read, modrdn, and add/delete files */
 	for ( file = readdir( datadir ); file; file = readdir( datadir )) {
 
 		if ( !strcasecmp( file->d_name, TSEARCHFILE )) {
@@ -143,6 +170,9 @@ main( int argc, char **argv )
 		} else if ( !strcasecmp( file->d_name, TREADFILE )) {
 			rfile = get_file_name( dirname, file->d_name );
 			continue;
+		} else if ( !strcasecmp( file->d_name, TMODRDNFILE )) {
+			mfile = get_file_name( dirname, file->d_name );
+			continue;
 		} else if ( !strncasecmp( file->d_name, TADDFILE, strlen( TADDFILE ))
 			&& ( anum < MAXREQS )) {
 			afiles[anum++] = get_file_name( dirname, file->d_name );
@@ -162,17 +192,28 @@ main( int argc, char **argv )
 		rnum = get_read_entries( rfile, rreqs );
 	}
 
+	/* look for modrdn requests */
+	if ( mfile ) {
+		mnum = get_read_entries( mfile, mreqs );
+	}
+
 	/*
 	 * generate the search clients
 	 */
 
 	sanum = 0;
-	sprintf( scmd, "%s%s%s", progdir, LDAP_DIRSEP, SEARCHCMD );
+	snprintf( scmd, sizeof scmd, "%s" LDAP_DIRSEP SEARCHCMD,
+		progdir );
 	sargs[sanum++] = scmd;
-	sargs[sanum++] = "-h";
-	sargs[sanum++] = host;
-	sargs[sanum++] = "-p";
-	sargs[sanum++] = port;
+	if ( uri ) {
+		sargs[sanum++] = "-H";
+		sargs[sanum++] = uri;
+	} else {
+		sargs[sanum++] = "-h";
+		sargs[sanum++] = host;
+		sargs[sanum++] = "-p";
+		sargs[sanum++] = port;
+	}
 	sargs[sanum++] = "-b";
 	sargs[sanum++] = sbase;
 	sargs[sanum++] = "-l";
@@ -186,29 +227,68 @@ main( int argc, char **argv )
 	 */
 
 	ranum = 0;
-	sprintf( rcmd, "%s%s%s", progdir, LDAP_DIRSEP, READCMD );
+	snprintf( rcmd, sizeof rcmd, "%s" LDAP_DIRSEP READCMD,
+		progdir );
 	rargs[ranum++] = rcmd;
-	rargs[ranum++] = "-h";
-	rargs[ranum++] = host;
-	rargs[ranum++] = "-p";
-	rargs[ranum++] = port;
+	if ( uri ) {
+		rargs[ranum++] = "-H";
+		rargs[ranum++] = uri;
+	} else {
+		rargs[ranum++] = "-h";
+		rargs[ranum++] = host;
+		rargs[ranum++] = "-p";
+		rargs[ranum++] = port;
+	}
 	rargs[ranum++] = "-l";
 	rargs[ranum++] = loops;
 	rargs[ranum++] = "-e";
 	rargs[ranum++] = NULL;		/* will hold the read entry */
 	rargs[ranum++] = NULL;
 
+	/*
+	 * generate the modrdn clients
+	 */
+
+	manum = 0;
+	snprintf( mcmd, sizeof mcmd, "%s" LDAP_DIRSEP MODRDNCMD,
+		progdir );
+	margs[manum++] = mcmd;
+	if ( uri ) {
+		margs[manum++] = "-H";
+		margs[manum++] = uri;
+	} else {
+		margs[manum++] = "-h";
+		margs[manum++] = host;
+		margs[manum++] = "-p";
+		margs[manum++] = port;
+	}
+	margs[manum++] = "-D";
+	margs[manum++] = manager;
+	margs[manum++] = "-w";
+	margs[manum++] = passwd;
+	margs[manum++] = "-l";
+	margs[manum++] = loops;
+	margs[manum++] = "-e";
+	margs[manum++] = NULL;		/* will hold the modrdn entry */
+	margs[manum++] = NULL;
+
 	/*
 	 * generate the add/delete clients
 	 */
 
 	aanum = 0;
-	sprintf( acmd, "%s%s%s", progdir, LDAP_DIRSEP, ADDCMD );
+	snprintf( acmd, sizeof acmd, "%s" LDAP_DIRSEP ADDCMD,
+		progdir );
 	aargs[aanum++] = acmd;
-	aargs[aanum++] = "-h";
-	aargs[aanum++] = host;
-	aargs[aanum++] = "-p";
-	aargs[aanum++] = port;
+	if ( uri ) {
+		aargs[aanum++] = "-H";
+		aargs[aanum++] = uri;
+	} else {
+		aargs[aanum++] = "-h";
+		aargs[aanum++] = host;
+		aargs[aanum++] = "-p";
+		aargs[aanum++] = port;
+	}
 	aargs[aanum++] = "-D";
 	aargs[aanum++] = manager;
 	aargs[aanum++] = "-w";
@@ -235,6 +315,13 @@ main( int argc, char **argv )
 
 		}
 
+		if ( j < mnum ) {
+
+			margs[manum - 2] = mreqs[j];
+			fork_child( mcmd, margs );
+
+		}
+
 		if ( j < anum ) {
 
 			aargs[aanum - 2] = afiles[j];
@@ -246,7 +333,7 @@ main( int argc, char **argv )
 
 	wait4kids( -1 );
 
-	exit( 0 );
+	exit( EXIT_SUCCESS );
 }
 
 static char *
@@ -254,7 +341,8 @@ get_file_name( char *dirname, char *filename )
 {
 	char buf[MAXPATHLEN];
 
-	sprintf( buf, "%s%s%s", dirname, LDAP_DIRSEP, filename );
+	snprintf( buf, sizeof buf, "%s" LDAP_DIRSEP "%s",
+		dirname, filename );
 	return( strdup( buf ));
 }
 
@@ -273,7 +361,7 @@ get_search_filters( char *filename, char *filters[] )
 
 			if (( nl = strchr( line, '\r' )) || ( nl = strchr( line, '\n' )))
 				*nl = '\0';
-			filters[filter++] = strdup( line );
+			filters[filter++] = ArgDup( line );
 
 		}
 		fclose( fp );
@@ -297,7 +385,7 @@ get_read_entries( char *filename, char *entries[] )
 
 			if (( nl = strchr( line, '\r' )) || ( nl = strchr( line, '\n' )))
 				*nl = '\0';
-			entries[entry++] = strdup( line );
+			entries[entry++] = ArgDup( line );
 
 		}
 		fclose( fp );
@@ -306,9 +394,9 @@ get_read_entries( char *filename, char *entries[] )
 	return( entry );
 }
 
-
+#ifndef HAVE_WINSOCK
 static void
-fork_child( char *prog, char *args[] )
+fork_child( char *prog, char **args )
 {
 	pid_t	pid;
 
@@ -316,10 +404,24 @@ fork_child( char *prog, char *args[] )
 
 	switch ( pid = fork() ) {
 	case 0:		/* child */
+#ifdef HAVE_EBCDIC
+		/* The __LIBASCII execvp only handles ASCII "prog",
+		 * we still need to translate the arg vec ourselves.
+		 */
+		{ char *arg2[MAXREQS];
+		int i;
+
+		for (i=0; args[i]; i++) {
+			arg2[i] = ArgDup(args[i]);
+			__atoe(arg2[i]);
+		}
+		arg2[i] = NULL;
+		args = arg2; }
+#endif
 		execvp( prog, args );
 		fprintf( stderr, "%s: ", prog );
 		perror( "execv" );
-		exit( -1 );
+		exit( EXIT_FAILURE );
 		break;
 
 	case -1:	/* trouble */
@@ -369,3 +471,35 @@ wait4kids( int nkidval )
 		}
 	}
 }
+#else
+
+static void
+wait4kids( int nkidval )
+{
+	int rc, i;
+
+	while ( nkids >= nkidval ) {
+		rc = WaitForMultipleObjects( nkids, children, FALSE, INFINITE );
+		for ( i=rc - WAIT_OBJECT_0; i<nkids-1; i++)
+			children[i] = children[i+1];
+		nkids--;
+	}
+}
+
+static void
+fork_child( char *prog, char **args )
+{
+	int rc;
+
+	wait4kids( maxkids );
+
+	rc = _spawnvp( _P_NOWAIT, prog, args );
+
+	if ( rc == -1 ) {
+		fprintf( stderr, "%s: ", prog );
+		perror("spawnvp");
+	} else {
+		children[nkids++] = (HANDLE)rc;
+	}
+}
+#endif
